@@ -23,7 +23,7 @@
 namespace H5 {
 #endif
 
-class H5_DLLCPP DataSet : public AbstractDs {
+class H5_DLLCPP DataSet : public H5Object, public AbstractDs {
    public:
 	// Close this dataset.
 	virtual void close();
@@ -81,19 +81,13 @@ class H5_DLLCPP DataSet : public AbstractDs {
 	// Retrieves a dataspace with the region pointed to selected.
 	DataSpace getRegion(void *ref, H5R_type_t ref_type = H5R_DATASET_REGION) const;
 
-	// Creates a reference to a named Hdf5 object or to a dataset region
-	// in this object.
-	void* Reference(const char* name, DataSpace& dataspace, H5R_type_t ref_type = H5R_DATASET_REGION) const;
-
-	// Creates a reference to a named Hdf5 object in this object.
-	void* Reference(const char* name) const; // will be obsolete
-	void* Reference(const H5std_string& name) const; // will be obsolete
-
-	// Returns this class name
+	///\brief Returns this class name
 	virtual H5std_string fromClass () const { return("DataSet"); }
 
 	// Creates a dataset by way of dereference.
-	DataSet(IdComponent& obj, void* ref);
+	DataSet(H5Object& obj, const void* ref, H5R_type_t ref_type = H5R_OBJECT);
+	DataSet(H5File& h5file, const void* ref, H5R_type_t ref_type = H5R_OBJECT);
+	DataSet(Attribute& attr, const void* ref, H5R_type_t ref_type = H5R_OBJECT);
 
 	// Default constructor.
 	DataSet();
@@ -104,15 +98,26 @@ class H5_DLLCPP DataSet : public AbstractDs {
 	// Creates a copy of an existing DataSet using its id.
 	DataSet(const hid_t existing_id);
 
+        // Gets the dataset id.
+        virtual hid_t getId() const;
+
 	// Destructor: properly terminates access to this dataset.
 	virtual ~DataSet();
 
    private:
+	hid_t id;       // HDF5 dataset id
+
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
         // This function contains the common code that is used by
         // getTypeClass and various API functions getXxxType
         // defined in AbstractDs for generic datatype and specific
         // sub-types
 	virtual hid_t p_get_type() const;
+
+   protected:
+        // Sets the dataset id.
+        virtual void p_setId(const hid_t new_id);
+#endif // DOXYGEN_SHOULD_SKIP_THIS
 };
 #ifndef H5_NO_NAMESPACE
 }
