@@ -51,21 +51,14 @@
 !----------------------------------------------------------------------
           SUBROUTINE h5fcreate_f(name, access_flags, file_id, hdferr, &
                                  creation_prp, access_prp)
-!
-!This definition is needed for Windows DLLs
-!DEC$if defined(BUILD_HDF5_DLL)
-!DEC$attributes dllexport :: h5fcreate_f
-!DEC$endif
-!
-           
-            IMPLICIT NONE
+            IMPLICIT NONE 
             CHARACTER(LEN=*), INTENT(IN) :: name   ! Name of the file
             INTEGER, INTENT(IN) :: access_flags    ! File access flags
             INTEGER(HID_T), INTENT(OUT) :: file_id ! File identifier 
             INTEGER, INTENT(OUT) :: hdferr         ! Error code 
             INTEGER(HID_T), OPTIONAL, INTENT(IN) :: creation_prp 
                                                    ! File creation propertly
-                                                   ! list identifier
+                                                   ! list identifier 
             INTEGER(HID_T), OPTIONAL, INTENT(IN) :: access_prp
                                                    ! File access property list
                                                    ! identifier
@@ -80,9 +73,9 @@
               INTEGER FUNCTION h5fcreate_c(name, namelen, access_flags, &
                                creation_prp_default, access_prp_default, file_id)
               USE H5GLOBAL
-              !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-              !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5FCREATE_C':: h5fcreate_c
-              !DEC$ ENDIF
+              !DEC$IF DEFINED(HDF5F90_WINDOWS)
+              !DEC$ATTRIBUTES C,reference,decorate,alias:'H5FCREATE_C':: h5fcreate_c
+              !DEC$ENDIF
               !DEC$ATTRIBUTES reference :: name 
               CHARACTER(LEN=*), INTENT(IN) :: name
               INTEGER, INTENT(IN) :: access_flags
@@ -134,13 +127,6 @@
 ! Comment:		
 !----------------------------------------------------------------------
           SUBROUTINE h5fflush_f(object_id, scope, hdferr)
-!
-!This definition is needed for Windows DLLs
-!DEC$if defined(BUILD_HDF5_DLL)
-!DEC$attributes dllexport :: h5fflush_f
-!DEC$endif
-!
-           
             IMPLICIT NONE
             INTEGER(HID_T), INTENT(IN) :: object_id !identifier for any object
                                                     !associate with a file, 
@@ -166,9 +152,9 @@
             INTERFACE
               INTEGER FUNCTION h5fflush_c(object_id, scope)
               USE H5GLOBAL
-              !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-              !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5FFLUSH_C':: h5fflush_c
-              !DEC$ ENDIF
+              !DEC$IF DEFINED(HDF5F90_WINDOWS)
+              !DEC$ATTRIBUTES C,reference,decorate,alias:'H5FFLUSH_C':: h5fflush_c
+              !DEC$ENDIF
               INTEGER(HID_T), INTENT(IN) :: object_id
               INTEGER, INTENT(IN) :: scope
               END FUNCTION h5fflush_c
@@ -207,13 +193,6 @@
 !----------------------------------------------------------------------
  
           SUBROUTINE h5fmount_f(loc_id, name, child_id, hdferr, access_prp)
-!
-!This definition is needed for Windows DLLs
-!DEC$if defined(BUILD_HDF5_DLL)
-!DEC$attributes dllexport :: h5fmount_f
-!DEC$endif
-!
-           
             IMPLICIT NONE
             INTEGER(HID_T), INTENT(IN) :: loc_id   ! Identifier for file or group 
                                                    ! in which dsetname is defined 
@@ -234,9 +213,9 @@
               INTEGER FUNCTION h5fmount_c(loc_id, name, namelen, &
                                           child_id, access_prp_default)
               USE H5GLOBAL
-              !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-              !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5FMOUNT_C':: h5fmount_c
-              !DEC$ ENDIF
+              !DEC$IF DEFINED(HDF5F90_WINDOWS)
+              !DEC$ATTRIBUTES C,reference,decorate,alias:'H5FMOUNT_C':: h5fmount_c
+              !DEC$ENDIF
               !DEC$ATTRIBUTES reference :: name 
               INTEGER(HID_T), INTENT(IN) :: loc_id 
               CHARACTER(LEN=*), INTENT(IN) :: name 
@@ -281,13 +260,6 @@
 !----------------------------------------------------------------------
 
           SUBROUTINE h5funmount_f(loc_id, name, hdferr)
-!
-!This definition is needed for Windows DLLs
-!DEC$if defined(BUILD_HDF5_DLL)
-!DEC$attributes dllexport :: h5funmount_f
-!DEC$endif
-!
-           
             IMPLICIT NONE
             INTEGER(HID_T), INTENT(IN) :: loc_id   ! Identifier for file or group 
                                                    ! at which the specified file 
@@ -302,9 +274,9 @@
             INTERFACE
               INTEGER FUNCTION h5funmount_c(loc_id, name, namelen)
               USE H5GLOBAL
-              !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-              !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5FUNMOUNT_C':: h5funmount_c
-              !DEC$ ENDIF
+              !DEC$IF DEFINED(HDF5F90_WINDOWS)
+              !DEC$ATTRIBUTES C,reference,decorate,alias:'H5FUNMOUNT_C':: h5funmount_c
+              !DEC$ENDIF
               !DEC$ATTRIBUTES reference :: name 
               INTEGER(HID_T), INTENT(IN) :: loc_id 
               CHARACTER(LEN=*), INTENT(IN) :: name 
@@ -346,13 +318,6 @@
 !----------------------------------------------------------------------
           SUBROUTINE h5fopen_f(name, access_flags, file_id, hdferr, &
                                access_prp)
-!
-!This definition is needed for Windows DLLs
-!DEC$if defined(BUILD_HDF5_DLL)
-!DEC$attributes dllexport :: h5fopen_f
-!DEC$endif
-!
-           
             IMPLICIT NONE
             CHARACTER(LEN=*), INTENT(IN) :: name   ! Name of the file
             INTEGER, INTENT(IN) :: access_flags    ! File access flags
@@ -371,9 +336,9 @@
               INTEGER FUNCTION h5fopen_c(name, namelen, access_flags, &
                                access_prp_default, file_id)
               USE H5GLOBAL
-              !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-              !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5FOPEN_C':: h5fopen_c
-              !DEC$ ENDIF
+              !DEC$IF DEFINED(HDF5F90_WINDOWS)
+              !DEC$ATTRIBUTES C,reference,decorate,alias:'H5FOPEN_C':: h5fopen_c
+              !DEC$ENDIF
               !DEC$ATTRIBUTES reference :: name
               CHARACTER(LEN=*), INTENT(IN) :: name
               INTEGER :: namelen
@@ -418,13 +383,6 @@
 !----------------------------------------------------------------------
 
           SUBROUTINE h5freopen_f(file_id, ret_file_id, hdferr)
-!
-!This definition is needed for Windows DLLs
-!DEC$if defined(BUILD_HDF5_DLL)
-!DEC$attributes dllexport :: h5freopen_f
-!DEC$endif
-!
-           
             IMPLICIT NONE
             INTEGER(HID_T), INTENT(IN) :: file_id      ! File identifier 
             INTEGER(HID_T), INTENT(OUT) :: ret_file_id ! New File identifier 
@@ -436,9 +394,9 @@
             INTERFACE
               INTEGER FUNCTION h5freopen_c(file_id, ret_file_id)
               USE H5GLOBAL
-              !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-              !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5FREOPEN_C':: h5freopen_c
-              !DEC$ ENDIF
+              !DEC$IF DEFINED(HDF5F90_WINDOWS)
+              !DEC$ATTRIBUTES C,reference,decorate,alias:'H5FREOPEN_C':: h5freopen_c
+              !DEC$ENDIF
               INTEGER(HID_T), INTENT(IN) :: file_id
               INTEGER(HID_T), INTENT(OUT) :: ret_file_id
               END FUNCTION h5freopen_c
@@ -475,13 +433,6 @@
 !----------------------------------------------------------------------
           
           SUBROUTINE h5fget_create_plist_f(file_id, prop_id, hdferr)
-!
-!This definition is needed for Windows DLLs
-!DEC$if defined(BUILD_HDF5_DLL)
-!DEC$attributes dllexport :: h5fget_create_plist_f
-!DEC$endif
-!
-           
             IMPLICIT NONE
             INTEGER(HID_T), INTENT(IN) :: file_id    ! File identifier 
             INTEGER(HID_T), INTENT(OUT) :: prop_id   ! File creation property
@@ -494,9 +445,9 @@
             INTERFACE
               INTEGER FUNCTION h5fget_create_plist_c(file_id, prop_id)
               USE H5GLOBAL
-              !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-!DEC$ ATTRIBUTES C,reference,decorate,alias:'H5FGET_CREATE_PLIST_C':: h5fget_create_plist_c
-              !DEC$ ENDIF
+              !DEC$IF DEFINED(HDF5F90_WINDOWS)
+!DEC$ATTRIBUTES C,reference,decorate,alias:'H5FGET_CREATE_PLIST_C':: h5fget_create_plist_c
+              !DEC$ENDIF
               INTEGER(HID_T), INTENT(IN) :: file_id
               INTEGER(HID_T), INTENT(OUT) :: prop_id
               END FUNCTION h5fget_create_plist_c
@@ -533,13 +484,6 @@
 !----------------------------------------------------------------------
 
           SUBROUTINE h5fget_access_plist_f(file_id, access_id, hdferr)
-!
-!This definition is needed for Windows DLLs
-!DEC$if defined(BUILD_HDF5_DLL)
-!DEC$attributes dllexport :: h5fget_access_plist_f
-!DEC$endif
-!
-           
             IMPLICIT NONE
             INTEGER(HID_T), INTENT(IN) :: file_id      ! File identifier 
             INTEGER(HID_T), INTENT(OUT) :: access_id   ! File access property
@@ -552,9 +496,9 @@
             INTERFACE
               INTEGER FUNCTION h5fget_access_plist_c(file_id, access_id)
               USE H5GLOBAL
-              !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-!DEC$ ATTRIBUTES C,reference,decorate,alias:'H5FGET_CREATE_PLIST_C':: h5fget_access_plist_c
-              !DEC$ ENDIF
+              !DEC$IF DEFINED(HDF5F90_WINDOWS)
+!DEC$ATTRIBUTES C,reference,decorate,alias:'H5FGET_CREATE_PLIST_C':: h5fget_access_plist_c
+              !DEC$ENDIF
               INTEGER(HID_T), INTENT(IN) :: file_id
               INTEGER(HID_T), INTENT(OUT) :: access_id
               END FUNCTION h5fget_access_plist_c
@@ -591,13 +535,6 @@
 !----------------------------------------------------------------------
  
           SUBROUTINE h5fis_hdf5_f(name, status, hdferr)
-!
-!This definition is needed for Windows DLLs
-!DEC$if defined(BUILD_HDF5_DLL)
-!DEC$attributes dllexport :: h5fis_hdf5_f
-!DEC$endif
-!
-           
             IMPLICIT NONE
             CHARACTER(LEN=*), INTENT(IN) :: name   ! Name of the file
             LOGICAL, INTENT(OUT) :: status         ! Indicates if file
@@ -613,9 +550,9 @@
             INTERFACE
               INTEGER FUNCTION h5fis_hdf5_c(name, namelen, flag)
               USE H5GLOBAL
-              !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-              !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5FIS_HDF5_C':: h5fis_hdf5_c
-              !DEC$ ENDIF
+              !DEC$IF DEFINED(HDF5F90_WINDOWS)
+              !DEC$ATTRIBUTES C,reference,decorate,alias:'H5FIS_HDF5_C':: h5fis_hdf5_c
+              !DEC$ENDIF
               !DEC$ATTRIBUTES reference :: name
               CHARACTER(LEN=*), INTENT(IN) :: name
               INTEGER :: namelen
@@ -655,13 +592,6 @@
 !----------------------------------------------------------------------
           
           SUBROUTINE h5fclose_f(file_id, hdferr)
-!
-!This definition is needed for Windows DLLs
-!DEC$if defined(BUILD_HDF5_DLL)
-!DEC$attributes dllexport :: h5fclose_f
-!DEC$endif
-!
-
             IMPLICIT NONE
             INTEGER(HID_T), INTENT(IN) :: file_id ! File identifier
             INTEGER, INTENT(OUT) :: hdferr        ! Error code
@@ -672,9 +602,9 @@
             INTERFACE
               INTEGER FUNCTION h5fclose_c(file_id)
               USE H5GLOBAL
-              !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-              !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5FCLOSE_C':: h5fclose_c
-              !DEC$ ENDIF
+              !DEC$IF DEFINED(HDF5F90_WINDOWS)
+              !DEC$ATTRIBUTES C,reference,decorate,alias:'H5FCLOSE_C':: h5fclose_c
+              !DEC$ENDIF
               INTEGER(HID_T), INTENT(IN) :: file_id
               END FUNCTION h5fclose_c
             END INTERFACE
@@ -708,33 +638,29 @@
 !		September 30, 2002
 !
 ! Modifications: 	
-!
+!               Changed the type of obj_count to INTEGER(SIZE_T)
+!                               September 25, 2008 EIP
 ! Comment:		
 !----------------------------------------------------------------------
           
           SUBROUTINE h5fget_obj_count_f(file_id, obj_type, obj_count, hdferr)
-!
-!This definition is needed for Windows DLLs
-!DEC$if defined(BUILD_HDF5_DLL)
-!DEC$attributes dllexport :: h5fget_obj_count_f
-!DEC$endif
-!
-
             IMPLICIT NONE
             INTEGER(HID_T), INTENT(IN) :: file_id ! File identifier
             INTEGER, INTENT(IN)  :: obj_type      ! Object type
-            INTEGER, INTENT(OUT) :: obj_count     ! Number of open objects
+            INTEGER(SIZE_T), INTENT(OUT) :: obj_count
+                                                  ! Number of open objects
             INTEGER, INTENT(OUT) :: hdferr        ! Error code
 
             INTERFACE
               INTEGER FUNCTION h5fget_obj_count_c(file_id, obj_type, obj_count)
               USE H5GLOBAL
-              !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-       !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5FGET_OBJ_COUNT_C':: h5fget_obj_count_c
-              !DEC$ ENDIF
+              !DEC$IF DEFINED(HDF5F90_WINDOWS)
+       !DEC$ATTRIBUTES C,reference,decorate,alias:'H5FGET_OBJ_COUNT_C':: h5fget_obj_count_c
+              !DEC$ENDIF
               INTEGER(HID_T), INTENT(IN) :: file_id
               INTEGER, INTENT(IN)  :: obj_type      ! Object type
-              INTEGER, INTENT(OUT) :: obj_count    ! Number of open objects
+              INTEGER(SIZE_T), INTENT(OUT) :: obj_count
+                                                    ! Number of open objects
               END FUNCTION h5fget_obj_count_c
             END INTERFACE
 
@@ -767,40 +693,42 @@
 !		September 30, 2002
 !
 ! Modifications: 	
+!               Added optional parameter num_objs for number of open objects
+!               of the specified type and changed type of max_obj to 
+!               INTEGER(SIZE_T)
+!                               September 25, 2008 EIP
 !
 ! Comment:		
 !----------------------------------------------------------------------
           
-          SUBROUTINE h5fget_obj_ids_f(file_id, obj_type, max_objs, obj_ids, hdferr)
-!
-!This definition is needed for Windows DLLs
-!DEC$if defined(BUILD_HDF5_DLL)
-!DEC$attributes dllexport :: h5fget_obj_ids_f
-!DEC$endif
-!
-
+          SUBROUTINE h5fget_obj_ids_f(file_id, obj_type, max_objs, obj_ids, hdferr, num_objs)
             IMPLICIT NONE
             INTEGER(HID_T), INTENT(IN) :: file_id ! File identifier
             INTEGER, INTENT(IN)  :: obj_type   ! Object type
-            INTEGER, INTENT(IN)  :: max_objs   ! Maximum # of objects to retrieve
+            INTEGER(SIZE_T), INTENT(IN)  :: max_objs   ! Maximum # of objects to retrieve
             INTEGER(HID_T), DIMENSION(*), INTENT(INOUT) :: obj_ids
                                                ! Array of open objects iidentifiers
-            INTEGER, INTENT(OUT) :: hdferr        ! Error code
+            INTEGER, INTENT(OUT) :: hdferr     ! Error code
+            INTEGER(SIZE_T), INTENT(OUT), OPTIONAL :: num_objs 
+            INTEGER(SIZE_T) :: c_num_objs 
+                              ! Number of open objects of the specified type
 
             INTERFACE
-              INTEGER FUNCTION h5fget_obj_ids_c(file_id, obj_type, max_objs, obj_ids)
+              INTEGER FUNCTION h5fget_obj_ids_c(file_id, obj_type, max_objs, obj_ids, c_num_objs)
               USE H5GLOBAL
-              !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-       !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5FGET_OBJ_IDS_C':: h5fget_obj_ids_c
-              !DEC$ ENDIF
+              !DEC$IF DEFINED(HDF5F90_WINDOWS)
+       !DEC$ATTRIBUTES C,reference,decorate,alias:'H5FGET_OBJ_IDS_C':: h5fget_obj_ids_c
+              !DEC$ENDIF
               INTEGER(HID_T), INTENT(IN) :: file_id
               INTEGER, INTENT(IN)  :: obj_type      
-              INTEGER, INTENT(IN)  :: max_objs
+              INTEGER(SIZE_T), INTENT(IN)  :: max_objs
               INTEGER(HID_T), DIMENSION(*), INTENT(INOUT) :: obj_ids  
+              INTEGER(SIZE_T), INTENT(OUT) :: c_num_objs 
               END FUNCTION h5fget_obj_ids_c
             END INTERFACE
 
-            hdferr = h5fget_obj_ids_c(file_id, obj_type, max_objs, obj_ids)
+            hdferr = h5fget_obj_ids_c(file_id, obj_type, max_objs, obj_ids, c_num_objs)
+            if (present(num_objs)) num_objs= c_num_objs
 
           END SUBROUTINE h5fget_obj_ids_f
 
@@ -828,13 +756,6 @@
 !----------------------------------------------------------------------
           
           SUBROUTINE h5fget_freespace_f(file_id, free_space, hdferr)
-!
-!This definition is needed for Windows DLLs
-!DEC$if defined(BUILD_HDF5_DLL)
-!DEC$attributes dllexport :: h5fget_freespace_f
-!DEC$endif
-!
-
             IMPLICIT NONE
             INTEGER(HID_T), INTENT(IN) :: file_id ! File identifier
             INTEGER(HSSIZE_T), INTENT(OUT) :: free_space 
@@ -844,9 +765,9 @@
             INTERFACE
               INTEGER FUNCTION h5fget_freespace_c(file_id, free_space)
               USE H5GLOBAL
-              !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-       !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5FGET_FREESPACE_C':: h5fget_freespace_c
-              !DEC$ ENDIF
+              !DEC$IF DEFINED(HDF5F90_WINDOWS)
+       !DEC$ATTRIBUTES C,reference,decorate,alias:'H5FGET_FREESPACE_C':: h5fget_freespace_c
+              !DEC$ENDIF
               INTEGER(HID_T), INTENT(IN) :: file_id
               INTEGER(HSSIZE_T), INTENT(OUT) :: free_space 
               END FUNCTION h5fget_freespace_c
@@ -878,11 +799,7 @@
 !----------------------------------------------------------------------
 
 
-          SUBROUTINE h5fget_name_f(obj_id, buf, size, hdferr) 
-!This definition is needed for Windows DLLs
-!DEC$if defined(BUILD_HDF5_DLL)
-!DEC$attributes dllexport :: h5fget_name_f
-!DEC$endif
+          SUBROUTINE h5fget_name_f(obj_id, buf, size, hdferr)
             IMPLICIT NONE
             INTEGER(HID_T), INTENT(IN) :: obj_id   ! Object identifier 
             CHARACTER(LEN=*), INTENT(INOUT) :: buf   
@@ -897,9 +814,9 @@
             INTERFACE
               INTEGER FUNCTION h5fget_name_c(obj_id, size, buf, buflen)
               USE H5GLOBAL
-              !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-              !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5FGET_NAME_C'::h5fget_name_c
-              !DEC$ ENDIF
+              !DEC$IF DEFINED(HDF5F90_WINDOWS)
+              !DEC$ATTRIBUTES C,reference,decorate,alias:'H5FGET_NAME_C'::h5fget_name_c
+              !DEC$ENDIF
               !DEC$ATTRIBUTES reference :: buf
               INTEGER(HID_T), INTENT(IN) :: obj_id
               INTEGER(SIZE_T), INTENT(OUT) :: size
@@ -931,11 +848,7 @@
 !----------------------------------------------------------------------
 
 
-          SUBROUTINE h5fget_filesize_f(file_id, size, hdferr) 
-!This definition is needed for Windows DLLs
-!DEC$if defined(BUILD_HDF5_DLL)
-!DEC$attributes dllexport :: h5fget_filesize_f
-!DEC$endif
+          SUBROUTINE h5fget_filesize_f(file_id, size, hdferr)
             IMPLICIT NONE
             INTEGER(HID_T), INTENT(IN) :: file_id  ! file identifier
             INTEGER(HSIZE_T), INTENT(OUT) :: size  ! Size of the file 
@@ -947,9 +860,9 @@
             INTERFACE
               INTEGER FUNCTION h5fget_filesize_c(file_id, size)
               USE H5GLOBAL
-              !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-              !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5FGET_FILESIZE_C'::h5fget_filesize_c
-              !DEC$ ENDIF
+              !DEC$IF DEFINED(HDF5F90_WINDOWS)
+              !DEC$ATTRIBUTES C,reference,decorate,alias:'H5FGET_FILESIZE_C'::h5fget_filesize_c
+              !DEC$ENDIF
               INTEGER(HID_T), INTENT(IN) :: file_id
               INTEGER(HSIZE_T), INTENT(OUT) :: size
               END FUNCTION h5fget_filesize_c
