@@ -28,6 +28,7 @@
 #include "H5Lprivate.h"		/* Links		  		*/
 #include "H5Pprivate.h"		/* Property lists			*/
 #include "H5Tprivate.h"		/* Datatypes				*/
+#include "H5SLprivate.h"    /* Skip lists                           */
 
 
 /****************/
@@ -206,7 +207,7 @@ void
 H5_term_library(void)
 {
     int	pending, ntries = 0, n;
-    unsigned	at = 0;
+    size_t	at = 0;
     char	loop[1024];
     H5E_auto2_t func;
 
@@ -270,6 +271,9 @@ H5_term_library(void)
             /* Don't shut down the ID code until other APIs which use them are shut down */
             if(pending == 0)
                 pending += DOWN(I);
+            /* Don't shut down the skip list code until everything that uses it is down */
+            if(pending == 0)
+                pending += DOWN(SL);
             /* Don't shut down the free list code until _everything_ else is down */
             if(pending == 0)
                 pending += DOWN(FL);
@@ -405,6 +409,9 @@ done:
  *      global lists, up to 3 MB of total storage might be allocated (1MB on
  *      each of regular, array and block type lists).
  *
+ *      The settings for block free lists are duplicated to factory free lists.
+ *      Factory free list limits cannot be set independently currently.
+ *
  * Parameters:
  *  int reg_global_lim;  IN: The limit on all "regular" free list memory used
  *  int reg_list_lim;    IN: The limit on memory used in each "regular" free list
@@ -420,7 +427,9 @@ done:
  * Programmer:	Quincey Koziol
  *              Wednesday, August 2, 2000
  *
- * Modifications:
+ * Modifications:   Neil Fortner
+ *                  Wednesday, April 8, 2009
+ *                  Added support for factory free lists
  *
  *-------------------------------------------------------------------------
  */
@@ -435,7 +444,8 @@ H5set_free_list_limits(int reg_global_lim, int reg_list_lim, int arr_global_lim,
              arr_list_lim, blk_global_lim, blk_list_lim);
 
     /* Call the free list function to actually set the limits */
-    if(H5FL_set_free_list_limits(reg_global_lim, reg_list_lim, arr_global_lim, arr_list_lim, blk_global_lim, blk_list_lim)<0)
+    if(H5FL_set_free_list_limits(reg_global_lim, reg_list_lim, arr_global_lim, arr_list_lim,
+            blk_global_lim, blk_list_lim, blk_global_lim, blk_list_lim)<0)
         HGOTO_ERROR(H5E_RESOURCE, H5E_CANTSET, FAIL, "can't set garbage collection limits")
 
 done:
@@ -476,7 +486,7 @@ H5_debug_mask(const char *s)
     FILE	*stream = stderr;
     char	pkg_name[32], *rest;
     size_t	i;
-    int		clear;
+    hbool_t	clear;
 
     while (s && *s) {
 	if (HDisalpha(*s) || '-'==*s || '+'==*s) {
@@ -499,20 +509,20 @@ H5_debug_mask(const char *s)
 
 	    /* Trace, all, or one? */
 	    if (!HDstrcmp(pkg_name, "trace")) {
-		H5_debug_g.trace = clear?NULL:stream;
+		H5_debug_g.trace = clear ? NULL : stream;
             } else if (!HDstrcmp(pkg_name, "ttop")) {
                 H5_debug_g.trace = stream;
-                H5_debug_g.ttop = !clear;
+                H5_debug_g.ttop = (hbool_t)!clear;
             } else if (!HDstrcmp(pkg_name, "ttimes")) {
                 H5_debug_g.trace = stream;
-                H5_debug_g.ttimes = !clear;
+                H5_debug_g.ttimes = (hbool_t)!clear;
 	    } else if (!HDstrcmp(pkg_name, "all")) {
 		for (i=0; i<(size_t)H5_NPKGS; i++)
-		    H5_debug_g.pkg[i].stream = clear?NULL:stream;
+		    H5_debug_g.pkg[i].stream = clear ? NULL : stream;
 	    } else {
 		for (i=0; i<(size_t)H5_NPKGS; i++) {
 		    if (!HDstrcmp(H5_debug_g.pkg[i].name, pkg_name)) {
-			H5_debug_g.pkg[i].stream = clear?NULL:stream;
+			H5_debug_g.pkg[i].stream = clear ? NULL : stream;
 			break;
 		    }
 		}
@@ -600,6 +610,8 @@ done:
     "The HDF5 header files used to compile this application do not match\n" \
     "the version used by the HDF5 library to which this application is linked.\n" \
     "Data corruption or segmentation faults may occur if the application continues.\n" \
+    "This can happen when an application was compiled by one version of HDF5 but\n" \
+    "linked with a different version of static or shared HDF5 library.\n" \
     "You should recompile the application or check your shared library related\n" \
     "settings such as 'LD_LIBRARY_PATH'.\n"
 
