@@ -91,6 +91,7 @@ static H5FD_t *H5FD_core_open(const char *name, unsigned flags, hid_t fapl_id,
 			      haddr_t maxaddr);
 static herr_t H5FD_core_close(H5FD_t *_file);
 static int H5FD_core_cmp(const H5FD_t *_f1, const H5FD_t *_f2);
+static herr_t H5FD_core_query(const H5FD_t *_f1, unsigned long *flags);
 static haddr_t H5FD_core_get_eoa(const H5FD_t *_file, H5FD_mem_t type);
 static herr_t H5FD_core_set_eoa(H5FD_t *_file, H5FD_mem_t type, haddr_t addr);
 static haddr_t H5FD_core_get_eof(const H5FD_t *_file);
@@ -100,6 +101,7 @@ static herr_t H5FD_core_read(H5FD_t *_file, H5FD_mem_t type, hid_t fapl_id, hadd
 static herr_t H5FD_core_write(H5FD_t *_file, H5FD_mem_t type, hid_t fapl_id, haddr_t addr,
 			      size_t size, const void *buf);
 static herr_t H5FD_core_flush(H5FD_t *_file, hid_t dxpl_id, unsigned closing);
+static herr_t H5FD_core_truncate(H5FD_t *_file, hid_t dxpl_id, hbool_t closing);
 
 static const H5FD_class_t H5FD_core_g = {
     "core",					/*name			*/
@@ -118,7 +120,8 @@ static const H5FD_class_t H5FD_core_g = {
     H5FD_core_open,				/*open			*/
     H5FD_core_close,				/*close			*/
     H5FD_core_cmp,				/*cmp			*/
-    NULL,				        /*query			*/
+    H5FD_core_query,			        /*query			*/
+    NULL,					/*get_type_map		*/
     NULL,					/*alloc			*/
     NULL,					/*free			*/
     H5FD_core_get_eoa,				/*get_eoa		*/
@@ -128,6 +131,7 @@ static const H5FD_class_t H5FD_core_g = {
     H5FD_core_read,				/*read			*/
     H5FD_core_write,				/*write			*/
     H5FD_core_flush,				/*flush			*/
+    H5FD_core_truncate,				/*truncate		*/
     NULL,                                       /*lock                  */
     NULL,                                       /*unlock                */
     H5FD_FLMAP_SINGLE 				/*fl_map		*/
@@ -181,7 +185,7 @@ H5FD_core_init(void)
     FUNC_ENTER_NOAPI(H5FD_core_init, FAIL)
 
     if (H5I_VFL!=H5Iget_type(H5FD_CORE_g))
-        H5FD_CORE_g = H5FD_register(&H5FD_core_g,sizeof(H5FD_class_t));
+        H5FD_CORE_g = H5FD_register(&H5FD_core_g,sizeof(H5FD_class_t),FALSE);
 
     /* Set return value */
     ret_value=H5FD_CORE_g;
@@ -297,11 +301,11 @@ H5Pget_fapl_core(hid_t fapl_id, size_t *increment/*out*/,
     FUNC_ENTER_API(H5Pget_fapl_core, FAIL)
     H5TRACE3("e", "ixx", fapl_id, increment, backing_store);
 
-    if(NULL == (plist = H5P_object_verify(fapl_id,H5P_FILE_ACCESS)))
+    if(NULL == (plist = H5P_object_verify(fapl_id, H5P_FILE_ACCESS)))
         HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a file access property list")
-    if (H5FD_CORE!=H5P_get_driver(plist))
+    if(H5FD_CORE != H5P_get_driver(plist))
         HGOTO_ERROR(H5E_PLIST, H5E_BADVALUE, FAIL, "incorrect VFL driver")
-    if (NULL==(fa=H5P_get_driver_info(plist)))
+    if(NULL == (fa = (H5FD_core_fapl_t *)H5P_get_driver_info(plist)))
         HGOTO_ERROR(H5E_PLIST, H5E_BADVALUE, FAIL, "bad VFL driver info")
 
     if (increment)
@@ -334,16 +338,16 @@ static void *
 H5FD_core_fapl_get(H5FD_t *_file)
 {
     H5FD_core_t		*file = (H5FD_core_t*)_file;
-    H5FD_core_fapl_t	*fa = NULL;
+    H5FD_core_fapl_t	*fa;
     void      *ret_value;       /* Return value */
 
     FUNC_ENTER_NOAPI(H5FD_core_fapl_get, NULL)
 
-    if (NULL==(fa=H5MM_calloc(sizeof(H5FD_core_fapl_t))))
+    if(NULL == (fa = (H5FD_core_fapl_t *)H5MM_calloc(sizeof(H5FD_core_fapl_t))))
         HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
 
     fa->increment = file->increment;
-    fa->backing_store = (file->fd>=0);
+    fa->backing_store = (hbool_t)(file->fd >= 0);
 
     /* Set return value */
     ret_value=fa;
@@ -370,9 +374,9 @@ done:
  * Modifications:
  *		Robb Matzke, 1999-10-19
  *		The backing store file is created and opened if specified.
- *              
+ *
  *              Raymond Lu, 2006-11-30
- *              Enabled the driver to read an existing file depending on 
+ *              Enabled the driver to read an existing file depending on
  *              the setting of the backing_store and file open flags.
  *-------------------------------------------------------------------------
  */
@@ -395,12 +399,12 @@ H5FD_core_open(const char *name, unsigned flags, hid_t fapl_id,
         HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, NULL, "invalid file name")
     if (0==maxaddr || HADDR_UNDEF==maxaddr)
         HGOTO_ERROR(H5E_ARGS, H5E_BADRANGE, NULL, "bogus maxaddr")
-    if (ADDR_OVERFLOW(maxaddr))
+    if(ADDR_OVERFLOW(maxaddr))
         HGOTO_ERROR(H5E_ARGS, H5E_OVERFLOW, NULL, "maxaddr overflow")
-    if (H5P_DEFAULT!=fapl_id) {
-        if(NULL == (plist = H5I_object(fapl_id)))
+    if(H5P_DEFAULT != fapl_id) {
+        if(NULL == (plist = (H5P_genplist_t *)H5I_object(fapl_id)))
             HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, NULL, "not a file access property list")
-        fa = H5P_get_driver_info(plist);
+        fa = (H5FD_core_fapl_t *)H5P_get_driver_info(plist);
     } /* end if */
 
     /* Build the open flags */
@@ -417,10 +421,10 @@ H5FD_core_open(const char *name, unsigned flags, hid_t fapl_id,
     }
 
     /* Create the new file struct */
-    if (NULL==(file=H5MM_calloc(sizeof(H5FD_core_t))))
+    if(NULL == (file = (H5FD_core_t *)H5MM_calloc(sizeof(H5FD_core_t))))
         HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "unable to allocate file struct")
     file->fd = fd;
-    if (name && *name)
+    if(name && *name)
         file->name = H5MM_xstrdup(name);
 
     /*
@@ -555,6 +559,42 @@ done:
     FUNC_LEAVE_NOAPI(ret_value)
 }
 
+
+/*-------------------------------------------------------------------------
+ * Function:	H5FD_core_query
+ *
+ * Purpose:	Set the flags that this VFL driver is capable of supporting.
+ *              (listed in H5FDpublic.h)
+ *
+ * Return:	Success:	non-negative
+ *		Failure:	negative
+ *
+ * Programmer:	Quincey Koziol
+ *              Tuesday, October  7, 2008
+ *
+ *-------------------------------------------------------------------------
+ */
+/* ARGSUSED */
+static herr_t
+H5FD_core_query(const H5FD_t UNUSED * _f, unsigned long *flags /* out */)
+{
+    herr_t ret_value = SUCCEED;         /* Return value */
+
+    FUNC_ENTER_NOAPI(H5FD_core_query, FAIL)
+
+    /* Set the VFL feature flags that this driver supports */
+    if(flags) {
+        *flags = 0;
+        *flags |= H5FD_FEAT_AGGREGATE_METADATA; /* OK to aggregate metadata allocations */
+        *flags |= H5FD_FEAT_ACCUMULATE_METADATA; /* OK to accumulate metadata for faster writes */
+        *flags |= H5FD_FEAT_DATA_SIEVE;       /* OK to perform data sieving for faster raw data reads & writes */
+        *flags |= H5FD_FEAT_AGGREGATE_SMALLDATA; /* OK to aggregate "small" raw data allocations */
+    } /* end if */
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5FD_core_query() */
+
 
 /*-------------------------------------------------------------------------
  * Function:	H5FD_core_get_eoa
@@ -619,18 +659,18 @@ static herr_t
 H5FD_core_set_eoa(H5FD_t *_file, H5FD_mem_t UNUSED type, haddr_t addr)
 {
     H5FD_core_t	*file = (H5FD_core_t*)_file;
-    herr_t      ret_value=SUCCEED;       /* Return value */
+    herr_t ret_value = SUCCEED;       /* Return value */
 
     FUNC_ENTER_NOAPI(H5FD_core_set_eoa, FAIL)
 
-    if (ADDR_OVERFLOW(addr))
+    if(ADDR_OVERFLOW(addr))
         HGOTO_ERROR(H5E_ARGS, H5E_OVERFLOW, FAIL, "address overflow")
 
     file->eoa = addr;
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
-}
+} /* end H5FD_core_set_eoa() */
 
 
 /*-------------------------------------------------------------------------
@@ -794,18 +834,18 @@ static herr_t
 H5FD_core_write(H5FD_t *_file, H5FD_mem_t UNUSED type, hid_t UNUSED dxpl_id, haddr_t addr,
 		size_t size, const void *buf)
 {
-    H5FD_core_t		*file = (H5FD_core_t*)_file;
-    herr_t      ret_value=SUCCEED;       /* Return value */
+    H5FD_core_t *file = (H5FD_core_t*)_file;
+    herr_t ret_value = SUCCEED;       /* Return value */
 
     FUNC_ENTER_NOAPI(H5FD_core_write, FAIL)
 
-    assert(file && file->pub.cls);
-    assert(buf);
+    HDassert(file && file->pub.cls);
+    HDassert(buf);
 
     /* Check for overflow conditions */
-    if (REGION_OVERFLOW(addr, size))
+    if(REGION_OVERFLOW(addr, size))
         HGOTO_ERROR(H5E_IO, H5E_OVERFLOW, FAIL, "file address overflowed")
-    if (addr+size>file->eoa)
+    if(addr + size > file->eoa)
         HGOTO_ERROR(H5E_IO, H5E_OVERFLOW, FAIL, "file address overflowed")
 
     /*
@@ -814,34 +854,33 @@ H5FD_core_write(H5FD_t *_file, H5FD_mem_t UNUSED type, hid_t UNUSED dxpl_id, had
      * careful of non-Posix realloc() that doesn't understand what to do when
      * the first argument is null.
      */
-    if (addr+size>file->eof) {
+    if(addr + size > file->eof) {
         unsigned char *x;
         size_t new_eof;
 
-        H5_ASSIGN_OVERFLOW(new_eof,file->increment*((addr+size)/file->increment),hsize_t,size_t);
-
-        if ((addr+size) % file->increment)
+        /* Determine new size of memory buffer */
+        H5_ASSIGN_OVERFLOW(new_eof, file->increment * ((addr + size) / file->increment), hsize_t, size_t);
+        if((addr + size) % file->increment)
             new_eof += file->increment;
-        if (NULL==file->mem)
-            x = H5MM_malloc(new_eof);
-        else
-            x = H5MM_realloc(file->mem, new_eof);
+
+        /* (Re)allocate memory for the file buffer */
+        if(NULL == (x = (unsigned char *)H5MM_realloc(file->mem, new_eof)))
+            HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "unable to allocate memory block")
 #ifdef H5_CLEAR_MEMORY
 HDmemset(x + file->eof, 0, (size_t)(new_eof - file->eof));
 #endif /* H5_CLEAR_MEMORY */
-        if (!x)
-            HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "unable to allocate memory block")
         file->mem = x;
+
         file->eof = new_eof;
-    }
+    } /* end if */
 
     /* Write from BUF to memory */
-    HDmemcpy(file->mem+addr, buf, size);
+    HDmemcpy(file->mem + addr, buf, size);
     file->dirty = TRUE;
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
-}
+} /* end H5FD_core_write() */
 
 
 /*-------------------------------------------------------------------------
@@ -860,7 +899,7 @@ done:
  * Modifications:
  *              Raymond Lu, 2006-11-30
  *              Added a condition check for backing store flag, for an
- *              existing file can be opened for read and write now. 
+ *              existing file can be opened for read and write now.
  *-------------------------------------------------------------------------
  */
 /* ARGSUSED */
@@ -898,3 +937,70 @@ H5FD_core_flush(H5FD_t *_file, hid_t UNUSED dxpl_id, unsigned UNUSED closing)
 done:
     FUNC_LEAVE_NOAPI(ret_value)
 }
+
+
+/*-------------------------------------------------------------------------
+ * Function:	H5FD_core_truncate
+ *
+ * Purpose:	Makes sure that the true file size is the same (or larger)
+ *		than the end-of-address.
+ *
+ * Return:	Success:	Non-negative
+ *		Failure:	Negative
+ *
+ * Programmer:	Quincey Koziol
+ *              Tuesday, October  7, 2008
+ *
+ *-------------------------------------------------------------------------
+ */
+/* ARGSUSED */
+static herr_t
+H5FD_core_truncate(H5FD_t *_file, hid_t UNUSED dxpl_id, hbool_t UNUSED closing)
+{
+    H5FD_core_t *file = (H5FD_core_t*)_file;
+    size_t new_eof;                             /* New size of memory buffer */
+    herr_t ret_value = SUCCEED;                 /* Return value */
+
+    FUNC_ENTER_NOAPI(H5FD_core_truncate, FAIL)
+
+    HDassert(file);
+
+    /* Determine new size of memory buffer */
+    H5_ASSIGN_OVERFLOW(new_eof, file->increment * (file->eoa / file->increment), hsize_t, size_t);
+    if(file->eoa % file->increment)
+        new_eof += file->increment;
+
+    /* Extend the file to make sure it's large enough */
+    if(!H5F_addr_eq((haddr_t)new_eof, file->eof)) {
+        unsigned char *x;       /* Pointer to new buffer for file data */
+
+        /* (Re)allocate memory for the file buffer */
+        if(NULL == (x = (unsigned char *)H5MM_realloc(file->mem, new_eof)))
+            HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "unable to allocate memory block")
+#ifdef H5_CLEAR_MEMORY
+if(file->eof < new_eof)
+    HDmemset(x + file->eof, 0, (size_t)(new_eof - file->eof));
+#endif /* H5_CLEAR_MEMORY */
+        file->mem = x;
+
+        /* Update backing store, if using it */
+        if(file->fd >= 0 && file->backing_store) {
+#ifdef H5_VMS
+            /* Reset seek offset to the beginning of the file, so that the file isn't 
+             * re-extended later.  This may happen on Open VMS. */
+            if(-1 == HDlseek(file->fd, 0, SEEK_SET))
+                HSYS_GOTO_ERROR(H5E_IO, H5E_SEEKERROR, FAIL, "unable to seek to proper position")
+#endif
+
+            if(-1 == HDftruncate(file->fd, (off_t)new_eof))
+                HSYS_GOTO_ERROR(H5E_IO, H5E_SEEKERROR, FAIL, "unable to extend file properly")
+        } /* end if */
+
+        /* Update the eof value */
+        file->eof = new_eof;
+    } /* end if */
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5FD_core_truncate() */
+
