@@ -190,7 +190,7 @@ static herr_t H5FD_mpiposix_read(H5FD_t *_file, H5FD_mem_t type, hid_t fapl_id,
         size_t size, void *buf);
 static herr_t H5FD_mpiposix_write(H5FD_t *_file, H5FD_mem_t type, hid_t fapl_id, haddr_t addr,
         size_t size, const void *buf);
-static herr_t H5FD_mpiposix_flush(H5FD_t *_file, hid_t dxpl_id, unsigned closing);
+static herr_t H5FD_mpiposix_truncate(H5FD_t *_file, hid_t dxpl_id, hbool_t closing);
 static int H5FD_mpiposix_mpi_rank(const H5FD_t *_file);
 static int H5FD_mpiposix_mpi_size(const H5FD_t *_file);
 static MPI_Comm H5FD_mpiposix_communicator(const H5FD_t *_file);
@@ -221,6 +221,7 @@ static const H5FD_class_mpi_t H5FD_mpiposix_g = {
     H5FD_mpiposix_close,			/*close			*/
     H5FD_mpiposix_cmp,			        /*cmp			*/
     H5FD_mpiposix_query,		        /*query			*/
+    NULL,					/*get_type_map		*/
     NULL,					/*alloc			*/
     NULL,					/*free			*/
     H5FD_mpiposix_get_eoa,			/*get_eoa		*/
@@ -229,7 +230,8 @@ static const H5FD_class_mpi_t H5FD_mpiposix_g = {
     H5FD_mpiposix_get_handle,                   /*get_handle            */
     H5FD_mpiposix_read,				/*read			*/
     H5FD_mpiposix_write,			/*write			*/
-    H5FD_mpiposix_flush,			/*flush			*/
+    NULL,					/*flush			*/
+    H5FD_mpiposix_truncate,			/*truncate		*/
     NULL,                                       /*lock                  */
     NULL,                                       /*unlock                */
     H5FD_FLMAP_SINGLE 				/*fl_map		*/
@@ -287,7 +289,7 @@ H5FD_mpiposix_init(void)
     FUNC_ENTER_NOAPI(H5FD_mpiposix_init, FAIL)
 
     if (H5I_VFL!=H5Iget_type(H5FD_MPIPOSIX_g))
-        H5FD_MPIPOSIX_g = H5FD_register((const H5FD_class_t *)&H5FD_mpiposix_g,sizeof(H5FD_class_mpi_t));
+        H5FD_MPIPOSIX_g = H5FD_register((const H5FD_class_t *)&H5FD_mpiposix_g,sizeof(H5FD_class_mpi_t),FALSE);
 
     /* Set return value */
     ret_value=H5FD_MPIPOSIX_g;
@@ -1213,7 +1215,9 @@ H5FD_mpiposix_write(H5FD_t *_file, H5FD_mem_t type, hid_t dxpl_id, haddr_t addr,
 		size_t size, const void *buf)
 {
     H5FD_mpiposix_t	*file = (H5FD_mpiposix_t*)_file;
+#if 0 /* JRM */
     int			mpi_code;	/* MPI return code */
+#endif /* JRM */
     ssize_t	        nbytes;         /* Number of bytes written each I/O call */
     H5P_genplist_t      *plist;         /* Property list pointer */
     herr_t             	ret_value=SUCCEED;      /* Return value */
@@ -1365,9 +1369,10 @@ done:
 
 
 /*-------------------------------------------------------------------------
- * Function:    H5FD_mpiposix_flush
+ * Function:    H5FD_mpiposix_truncate
  *
- * Purpose:     Makes sure that all data is on disk.  This is collective.
+ * Purpose:	Makes sure that the true file size is the same (or larger)
+ *		than the end-of-address.
  *
  * Return:      Success:	Non-negative
  * 		Failure:	Negative
@@ -1375,12 +1380,10 @@ done:
  * Programmer:  Quincey Koziol
  *              Thursday, July 11, 2002
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5FD_mpiposix_flush(H5FD_t *_file, hid_t UNUSED dxpl_id, unsigned UNUSED closing)
+H5FD_mpiposix_truncate(H5FD_t *_file, hid_t UNUSED dxpl_id, hbool_t UNUSED closing)
 {
     H5FD_mpiposix_t	*file = (H5FD_mpiposix_t*)_file;
 #ifdef _WIN32
@@ -1388,15 +1391,15 @@ H5FD_mpiposix_flush(H5FD_t *_file, hid_t UNUSED dxpl_id, unsigned UNUSED closing
     LARGE_INTEGER li;   /* 64-bit integer for SetFilePointer() call */
 #endif /* _WIN32 */
     int			mpi_code;	/* MPI return code */
-    herr_t              ret_value=SUCCEED;
+    herr_t              ret_value = SUCCEED;
 
-    FUNC_ENTER_NOAPI(H5FD_mpiposix_flush, FAIL)
+    FUNC_ENTER_NOAPI(H5FD_mpiposix_truncate, FAIL)
 
-    assert(file);
-    assert(H5FD_MPIPOSIX==file->pub.driver_id);
+    HDassert(file);
+    HDassert(H5FD_MPIPOSIX == file->pub.driver_id);
 
     /* Extend the file to make sure it's large enough */
-    if(file->eoa>file->last_eoa) {
+    if(file->eoa > file->last_eoa) {
         /* Use the round-robin process to truncate (extend) the file */
         if(file->mpi_rank == H5_PAR_META_WRITE) {
 #ifdef _WIN32
@@ -1406,8 +1409,8 @@ H5FD_mpiposix_flush(H5FD_t *_file, hid_t UNUSED dxpl_id, unsigned UNUSED closing
             /* Translate 64-bit integers into form Windows wants */
             /* [This algorithm is from the Windows documentation for SetFilePointer()] */
             li.QuadPart = file->eoa;
-            SetFilePointer((HANDLE)filehandle,li.LowPart,&li.HighPart,FILE_BEGIN);
-            if(SetEndOfFile((HANDLE)filehandle)==0)
+            SetFilePointer((HANDLE)filehandle, li.LowPart, &li.HighPart, FILE_BEGIN);
+            if(SetEndOfFile((HANDLE)filehandle) == 0)
                 HGOTO_ERROR(H5E_IO, H5E_SEEKERROR, FAIL, "unable to extend file properly")
 #else /* _WIN32 */
             if(-1==file_truncate(file->fd, (file_offset_t)file->eoa))
@@ -1421,11 +1424,11 @@ H5FD_mpiposix_flush(H5FD_t *_file, hid_t UNUSED dxpl_id, unsigned UNUSED closing
          * it the shorter length, potentially truncating the file and dropping
          * the new data written)
          */
-        if (MPI_SUCCESS!= (mpi_code=MPI_Barrier(file->comm)))
+        if(MPI_SUCCESS != (mpi_code = MPI_Barrier(file->comm)))
             HMPI_GOTO_ERROR(FAIL, "MPI_Barrier failed", mpi_code)
 
         /* Update the 'last' eoa and eof values */
-        file->last_eoa=file->eoa;
+        file->last_eoa = file->eoa;
         file->eof = file->eoa;
 
         /* Reset last file I/O information */
@@ -1435,7 +1438,7 @@ H5FD_mpiposix_flush(H5FD_t *_file, hid_t UNUSED dxpl_id, unsigned UNUSED closing
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
-} /* end H5FD_mpiposix_flush() */
+} /* end H5FD_mpiposix_truncate() */
 
 
 /*-------------------------------------------------------------------------
