@@ -38,7 +38,7 @@ typedef H5SL_t H5FO_t;       /* Currently, all open objects are stored in skip l
 /* Private routines */
 H5_DLL herr_t H5FO_create(const H5F_t *f);
 H5_DLL void  *H5FO_opened(const H5F_t *f, haddr_t addr);
-H5_DLL herr_t H5FO_insert(const H5F_t *f, haddr_t addr, void *obj);
+H5_DLL herr_t H5FO_insert(const H5F_t *f, haddr_t addr, void *obj, hbool_t delete_flag);
 H5_DLL herr_t H5FO_delete(H5F_t *f, hid_t dxpl_id, haddr_t addr);
 H5_DLL herr_t H5FO_mark(const H5F_t *f, haddr_t addr, hbool_t deleted);
 H5_DLL htri_t H5FO_marked(const H5F_t *f, haddr_t addr);
