@@ -36,6 +36,7 @@
 #include "H5private.h"		/* Generic Functions			*/
 #include "H5Eprivate.h"		/* Error handling		  	*/
 #include "H5FSpkg.h"		/* File free space			*/
+#include "H5MFprivate.h"	/* File memory management		*/
 #include "H5Vprivate.h"		/* Vectors and arrays 			*/
 #include "H5WBprivate.h"        /* Wrapped Buffers                      */
 
@@ -181,7 +182,7 @@ HDfprintf(stderr, "%s: Load free space header, addr = %a\n", FUNC, addr);
     size = H5FS_HEADER_SIZE(f);
 
     /* Get a pointer to a buffer that's large enough for header */
-    if(NULL == (hdr = H5WB_actual(wb, size)))
+    if(NULL == (hdr = (uint8_t *)H5WB_actual(wb, size)))
         HGOTO_ERROR(H5E_FSPACE, H5E_NOSPACE, NULL, "can't get actual buffer")
 
     /* Read header from disk */
@@ -191,9 +192,9 @@ HDfprintf(stderr, "%s: Load free space header, addr = %a\n", FUNC, addr);
     p = hdr;
 
     /* Magic number */
-    if(HDmemcmp(p, H5FS_HDR_MAGIC, (size_t)H5FS_SIZEOF_MAGIC))
+    if(HDmemcmp(p, H5FS_HDR_MAGIC, (size_t)H5_SIZEOF_MAGIC))
 	HGOTO_ERROR(H5E_FSPACE, H5E_CANTLOAD, NULL, "wrong free space header signature")
-    p += H5FS_SIZEOF_MAGIC;
+    p += H5_SIZEOF_MAGIC;
 
     /* Version */
     if(*p++ != H5FS_HDR_VERSION)
@@ -299,12 +300,68 @@ H5FS_cache_hdr_flush(H5F_t *f, hid_t dxpl_id, hbool_t destroy, haddr_t addr, H5F
     FUNC_ENTER_NOAPI_NOINIT(H5FS_cache_hdr_flush)
 #ifdef QAK
 HDfprintf(stderr, "%s: Flushing free space header, addr = %a, destroy = %u\n", FUNC, addr, (unsigned)destroy);
+HDfprintf(stderr, "%s: fspace->sect_addr = %a, fspace->sinfo = %p\n", FUNC, fspace->sect_addr, fspace->sinfo);
+HDfprintf(stderr, "%s: fspace->alloc_sect_size = %Hu, fspace->sect_size = %Hu\n", FUNC, fspace->alloc_sect_size, fspace->sect_size);
 #endif /* QAK */
 
     /* check arguments */
     HDassert(f);
     HDassert(H5F_addr_defined(addr));
     HDassert(fspace);
+    HDassert(H5F_addr_defined(fspace->addr));
+
+    /* Check if the header "owns" the section info */
+    if(fspace->sinfo) {
+        /* Sanity check - should not be trying to destroy header if it still
+         *      "owns" section info
+         */
+        HDassert(!destroy);
+
+        /* Check if the section info is dirty */
+        if(fspace->sinfo->dirty) {
+            if(fspace->serial_sect_count > 0) {
+                /* Check if we need to allocate space for  section info */
+                if(!H5F_addr_defined(fspace->sect_addr)) {
+                    /* Sanity check */
+                    HDassert(fspace->sect_size > 0);
+
+                    /* Allocate space for the section info in file */
+                    if(HADDR_UNDEF == (fspace->sect_addr = H5MF_alloc(f, H5FD_MEM_FSPACE_SINFO, dxpl_id, fspace->sect_size)))
+                        HGOTO_ERROR(H5E_FSPACE, H5E_NOSPACE, FAIL, "file allocation failed for free space sections")
+                    fspace->alloc_sect_size = (size_t)fspace->sect_size;
+
+                    /* Mark header dirty */
+                    /* (don't use cache API, since we're in a callback) */
+                    fspace->cache_info.is_dirty = TRUE;
+                } /* end if */
+
+                /* Write section info to file */
+                if(H5FS_cache_sinfo_flush(f, dxpl_id, FALSE, fspace->sect_addr, fspace->sinfo, NULL) < 0)
+                    HGOTO_ERROR(H5E_FSPACE, H5E_CANTFLUSH, FAIL, "unable to save free space section info to disk")
+            } /* end if */
+            else {
+                /* Sanity check that section info doesn't have address */
+                HDassert(!H5F_addr_defined(fspace->sect_addr));
+            } /* end else */
+#ifdef QAK
+HDfprintf(stderr, "%s: Check 2.0\n", FUNC);
+HDfprintf(stderr, "%s: fspace->sect_addr = %a, fspace->sinfo = %p\n", FUNC, fspace->sect_addr, fspace->sinfo);
+HDfprintf(stderr, "%s: fspace->alloc_sect_size = %Hu, fspace->sect_size = %Hu\n", FUNC, fspace->alloc_sect_size, fspace->sect_size);
+#endif /* QAK */
+
+            /* Mark section info clean */
+            fspace->sinfo->dirty = FALSE;
+        } /* end if */
+    } /* end if */
+    else {
+        /* Just sanity checks... */
+        if(fspace->serial_sect_count > 0)
+            /* Sanity check that section info has address */
+            HDassert(H5F_addr_defined(fspace->sect_addr));
+        else
+            /* Sanity check that section info doesn't have address */
+            HDassert(!H5F_addr_defined(fspace->sect_addr));
+    } /* end else */
 
     if(fspace->cache_info.is_dirty) {
         uint8_t	*hdr;                   /* Pointer to header buffer */
@@ -320,15 +377,15 @@ HDfprintf(stderr, "%s: Flushing free space header, addr = %a, destroy = %u\n", F
         size = H5FS_HEADER_SIZE(f);
 
         /* Get a pointer to a buffer that's large enough for header */
-        if(NULL == (hdr = H5WB_actual(wb, size)))
+        if(NULL == (hdr = (uint8_t *)H5WB_actual(wb, size)))
             HGOTO_ERROR(H5E_FSPACE, H5E_NOSPACE, FAIL, "can't get actual buffer")
 
         /* Get temporary pointer to header */
         p = hdr;
 
         /* Magic number */
-        HDmemcpy(p, H5FS_HDR_MAGIC, (size_t)H5FS_SIZEOF_MAGIC);
-        p += H5FS_SIZEOF_MAGIC;
+        HDmemcpy(p, H5FS_HDR_MAGIC, (size_t)H5_SIZEOF_MAGIC);
+        p += H5_SIZEOF_MAGIC;
 
         /* Version # */
         *p++ = H5FS_HDR_VERSION;
@@ -412,9 +469,8 @@ done:
  *
  *-------------------------------------------------------------------------
  */
-/* ARGSUSED */
 herr_t
-H5FS_cache_hdr_dest(H5F_t UNUSED *f, H5FS_t *fspace)
+H5FS_cache_hdr_dest(H5F_t *f, H5FS_t *fspace)
 {
     unsigned u;                 /* Local index variable */
     herr_t ret_value = SUCCEED; /* Return value */
@@ -426,6 +482,12 @@ H5FS_cache_hdr_dest(H5F_t UNUSED *f, H5FS_t *fspace)
      */
     HDassert(fspace);
 
+    /* We should not still be holding on to the free space section info */
+    HDassert(!fspace->sinfo);
+
+    /* If we're going to free the space on disk, the address must be valid */
+    HDassert(!fspace->cache_info.free_file_space_on_destroy || H5F_addr_defined(fspace->cache_info.addr));
+
     /* Terminate the section classes for this free space list */
     for(u = 0; u < fspace->nclasses ; u++) {
         /* Call the class termination routine, if there is one */
@@ -434,12 +496,23 @@ H5FS_cache_hdr_dest(H5F_t UNUSED *f, H5FS_t *fspace)
                 HGOTO_ERROR(H5E_RESOURCE, H5E_CANTRELEASE, FAIL, "unable to finalize section class")
     } /* end for */
 
+    /* Check for freeing file space for free space header */
+    if(fspace->cache_info.free_file_space_on_destroy) {
+        /* Sanity check */
+        HDassert(H5F_addr_defined(fspace->addr));
+
+        /* Release the space on disk */
+        /* (XXX: Nasty usage of internal DXPL value! -QAK) */
+        if(H5MF_xfree(f, H5FD_MEM_FSPACE_HDR, H5AC_dxpl_id, fspace->cache_info.addr, (hsize_t)H5FS_HEADER_SIZE(f)) < 0)
+            HGOTO_ERROR(H5E_FSPACE, H5E_CANTFREE, FAIL, "unable to free free space header")
+    } /* end if */
+
     /* Release the memory for the free space section classes */
     if(fspace->sect_cls)
-        fspace->sect_cls = H5FL_SEQ_FREE(H5FS_section_class_t, fspace->sect_cls);
+        fspace->sect_cls = (H5FS_section_class_t *)H5FL_SEQ_FREE(H5FS_section_class_t, fspace->sect_cls);
 
     /* Free free space info */
-    H5FL_FREE(H5FS_t, fspace);
+    (void)H5FL_FREE(H5FS_t, fspace);
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
@@ -555,11 +628,6 @@ HDfprintf(stderr, "%s: Load free space sections, addr = %a\n", FUNC, addr);
     /* Allocate a new free space section info */
     if(NULL == (sinfo = H5FS_sinfo_new(f, fspace)))
 	HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
-    
-    /* Link free space manager to section info */
-    /* (for deserializing sections) */
-    HDassert(fspace->sinfo == NULL);
-    fspace->sinfo = sinfo;
 
     /* Sanity check address */
     if(H5F_addr_ne(addr, fspace->sect_addr))
@@ -581,9 +649,9 @@ HDfprintf(stderr, "%s: fspace->sect_size = %Hu\n", FUNC, fspace->sect_size);
     p = buf;
 
     /* Magic number */
-    if(HDmemcmp(p, H5FS_SINFO_MAGIC, (size_t)H5FS_SIZEOF_MAGIC))
+    if(HDmemcmp(p, H5FS_SINFO_MAGIC, (size_t)H5_SIZEOF_MAGIC))
 	HGOTO_ERROR(H5E_FSPACE, H5E_CANTLOAD, NULL, "wrong free space sections signature")
-    p += H5FS_SIZEOF_MAGIC;
+    p += H5_SIZEOF_MAGIC;
 
     /* Version */
     if(*p++ != H5FS_SINFO_VERSION)
@@ -606,7 +674,7 @@ HDfprintf(stderr, "%s: fspace->addr = %a, fs_addr = %a\n", FUNC, fspace->addr, f
         unsigned sect_cnt_size;         /* The size of the section size counts */
 
         /* Compute the size of the section counts */
-        sect_cnt_size = MAX(1, (H5V_log2_gen(fspace->serial_sect_count) + 7) / 8);
+        sect_cnt_size = H5V_limit_enc_size((uint64_t)fspace->serial_sect_count);
 #ifdef QAK
 HDfprintf(stderr, "%s: sect_cnt_size = %u\n", FUNC, sect_cnt_size);
 HDfprintf(stderr, "%s: fspace->sect_len_size = %u\n", FUNC, fspace->sect_len_size);
@@ -713,7 +781,7 @@ HDfprintf(stderr, "%s: fspace->sect_cls[%u].serial_size = %Zu\n", FUNC, sect_typ
 
 done:
     if(buf)
-        H5FL_BLK_FREE(sect_block, buf);
+        (void)H5FL_BLK_FREE(sect_block, buf);
     if(!ret_value && sinfo)
         (void)H5FS_cache_sinfo_dest(f, sinfo);
 
@@ -872,7 +940,7 @@ HDfprintf(stderr, "%s: Flushing free space header, addr = %a, destroy = %u\n", F
     HDassert(sinfo->fspace);
     HDassert(sinfo->fspace->sect_cls);
 
-    if(sinfo->cache_info.is_dirty) {
+    if(sinfo->cache_info.is_dirty || sinfo->dirty) {
         H5FS_iter_ud_t udata;       /* User data for callbacks */
         uint8_t	*buf = NULL;        /* Temporary raw data buffer */
         uint8_t *p;                 /* Pointer into raw data buffer */
@@ -890,8 +958,8 @@ HDfprintf(stderr, "%s: Flushing free space header, addr = %a, destroy = %u\n", F
         p = buf;
 
         /* Magic number */
-        HDmemcpy(p, H5FS_SINFO_MAGIC, (size_t)H5FS_SIZEOF_MAGIC);
-        p += H5FS_SIZEOF_MAGIC;
+        HDmemcpy(p, H5FS_SINFO_MAGIC, (size_t)H5_SIZEOF_MAGIC);
+        p += H5_SIZEOF_MAGIC;
 
         /* Version # */
         *p++ = H5FS_SINFO_VERSION;
@@ -905,7 +973,7 @@ HDfprintf(stderr, "%s: sinfo->fspace->addr = %a\n", FUNC, sinfo->fspace->addr);
         /* Set up user data for iterator */
         udata.sinfo = sinfo;
         udata.p = &p;
-        udata.sect_cnt_size = MAX(1, (H5V_log2_gen(sinfo->fspace->serial_sect_count) + 7) / 8);
+        udata.sect_cnt_size = H5V_limit_enc_size((uint64_t)sinfo->fspace->serial_sect_count);
 #ifdef QAK
 HDfprintf(stderr, "%s: udata.sect_cnt_size = %u\n", FUNC, udata.sect_cnt_size);
 #endif /* QAK */
@@ -941,9 +1009,10 @@ HDfprintf(stderr, "%s: sinfo->fspace->alloc_sect_size = %Hu\n", FUNC, sinfo->fsp
         if(H5F_block_write(f, H5FD_MEM_FSPACE_SINFO, sinfo->fspace->sect_addr, (size_t)sinfo->fspace->sect_size, dxpl_id, buf) < 0)
             HGOTO_ERROR(H5E_FSPACE, H5E_CANTFLUSH, FAIL, "unable to save free space sections to disk")
 
-        H5FL_BLK_FREE(sect_block, buf);
+        (void)H5FL_BLK_FREE(sect_block, buf);
 
 	sinfo->cache_info.is_dirty = FALSE;
+        sinfo->dirty = FALSE;
     } /* end if */
 
     if(destroy)
@@ -966,6 +1035,10 @@ done:
  * Programmer:	Quincey Koziol
  *              Saturday, March 11, 2006
  *
+ * Modifications:
+ *	Vailin Choi, July 29th, 2008
+ *	  Add HDassert() to make sure "free" method exists before calling
+ *
  *-------------------------------------------------------------------------
  */
 static herr_t
@@ -978,6 +1051,7 @@ H5FS_sinfo_free_sect_cb(void *_sect, void UNUSED *key, void *op_data)
 
     HDassert(sect);
     HDassert(sinfo);
+    HDassert(sinfo->fspace->sect_cls[sect->type].free);
 
     /* Call the section's class 'free' method on the section */
     (*sinfo->fspace->sect_cls[sect->type].free)(sect);
@@ -1014,7 +1088,7 @@ H5FS_sinfo_free_node_cb(void *item, void UNUSED *key, void *op_data)
     H5SL_destroy(fspace_node->sect_list, H5FS_sinfo_free_sect_cb, op_data);
 
     /* Release free space list node */
-    H5FL_FREE(H5FS_node_t, fspace_node);
+    (void)H5FL_FREE(H5FS_node_t, fspace_node);
 
     FUNC_LEAVE_NOAPI(0)
 }   /* H5FS_sinfo_free_node_cb() */
@@ -1033,7 +1107,6 @@ H5FS_sinfo_free_node_cb(void *item, void UNUSED *key, void *op_data)
  *
  *-------------------------------------------------------------------------
  */
-/* ARGSUSED */
 herr_t
 H5FS_cache_sinfo_dest(H5F_t *f, H5FS_sinfo_t *sinfo)
 {
@@ -1041,6 +1114,9 @@ H5FS_cache_sinfo_dest(H5F_t *f, H5FS_sinfo_t *sinfo)
     herr_t ret_value = SUCCEED; /* Return value */
 
     FUNC_ENTER_NOAPI_NOINIT(H5FS_cache_sinfo_dest)
+#ifdef QAK
+HDfprintf(stderr, "%s: Destroying section info, sinfo->fspace->addr = %a\n", FUNC, sinfo->fspace->addr);
+#endif /* QAK */
 
     /*
      * Check arguments.
@@ -1049,6 +1125,20 @@ H5FS_cache_sinfo_dest(H5F_t *f, H5FS_sinfo_t *sinfo)
     HDassert(sinfo->fspace);
     HDassert(sinfo->bins);
 
+    /* If we're going to free the space on disk, the address must be valid */
+    HDassert(!sinfo->cache_info.free_file_space_on_destroy || H5F_addr_defined(sinfo->cache_info.addr));
+
+    /* Check for freeing file space for free space section info */
+    if(sinfo->cache_info.free_file_space_on_destroy) {
+        /* Sanity check */
+        HDassert(sinfo->fspace->alloc_sect_size > 0);
+
+        /* Release the space on disk */
+        /* (XXX: Nasty usage of internal DXPL value! -QAK) */
+        if(H5MF_xfree(f, H5FD_MEM_FSPACE_SINFO, H5AC_dxpl_id, sinfo->cache_info.addr, (hsize_t)sinfo->fspace->alloc_sect_size) < 0)
+            HGOTO_ERROR(H5E_FSPACE, H5E_CANTFREE, FAIL, "unable to free free space section info")
+    } /* end if */
+
     /* Clear out lists of nodes */
     for(u = 0; u < sinfo->nbins; u++)
         if(sinfo->bins[u].bin_list) {
@@ -1057,22 +1147,24 @@ H5FS_cache_sinfo_dest(H5F_t *f, H5FS_sinfo_t *sinfo)
         } /* end if */
 
     /* Release bins for skip lists */
-    sinfo->bins = H5FL_SEQ_FREE(H5FS_bin_t, sinfo->bins);
+    sinfo->bins = (H5FS_bin_t *)H5FL_SEQ_FREE(H5FS_bin_t, sinfo->bins);
 
     /* Release skip list for merging sections */
     if(sinfo->merge_list)
         if(H5SL_close(sinfo->merge_list) < 0)
             HGOTO_ERROR(H5E_FSPACE, H5E_CANTCLOSEOBJ, FAIL, "can't destroy section merging skip list")
 
-    /* Unpin the free space header in the cache */
+    /* Decrement the reference count on free space header */
     /* (make certain this is last action with section info, to allow for header
      *  disappearing immediately)
      */
-    if(H5AC_unpin_entry(f, sinfo->fspace) < 0)
-        HGOTO_ERROR(H5E_FSPACE, H5E_CANTUNPIN, FAIL, "unable to unpin free space header")
+    sinfo->fspace->sinfo = NULL;
+    if(H5FS_decr(f, sinfo->fspace) < 0)
+        HGOTO_ERROR(H5E_FSPACE, H5E_CANTDEC, FAIL, "unable to decrement ref. count on free space header")
+    sinfo->fspace = NULL;
 
     /* Release free space section info */
-    H5FL_FREE(H5FS_sinfo_t, sinfo);
+    (void)H5FL_FREE(H5FS_sinfo_t, sinfo);
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
@@ -1141,7 +1233,7 @@ H5FS_cache_sinfo_size(const H5F_t UNUSED *f, const H5FS_sinfo_t *sinfo, size_t *
     HDassert(size_ptr);
 
     /* Set size value */
-    H5_ASSIGN_OVERFLOW(/* To: */ *size_ptr, /* From: */ sinfo->fspace->sect_size, /* From: */ hsize_t, /* To: */ size_t);
+    H5_ASSIGN_OVERFLOW(/* To: */ *size_ptr, /* From: */ sinfo->fspace->alloc_sect_size, /* From: */ hsize_t, /* To: */ size_t);
 
     FUNC_LEAVE_NOAPI(SUCCEED)
 } /* H5FS_cache_sinfo_size() */
