@@ -63,7 +63,7 @@ H5F_debug(H5F_t *f, FILE *stream, int indent, int fwidth)
     HDassert(fwidth >= 0);
 
     /* Get property list */
-    if(NULL == (plist = H5I_object(f->shared->fcpl_id)))
+    if(NULL == (plist = (H5P_genplist_t *)H5I_object(f->shared->fcpl_id)))
         HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property list")
 
     /* Retrieve file creation properties */
@@ -82,7 +82,7 @@ H5F_debug(H5F_t *f, FILE *stream, int indent, int fwidth)
     HDfprintf(stream, "%*s%-*s %u\n", indent, "", fwidth,
 	      "File open reference count:", f->shared->nrefs);
     HDfprintf(stream, "%*s%-*s %a (abs)\n", indent, "", fwidth,
-	      "Address of super block:", f->shared->super_addr);
+	      "Address of super block:", f->shared->base_addr);
     HDfprintf(stream, "%*s%-*s %Hu bytes\n", indent, "", fwidth,
 	      "Size of userblock:", userblock_size);
 
@@ -107,8 +107,6 @@ H5F_debug(H5F_t *f, FILE *stream, int indent, int fwidth)
 	      "Symbol table internal node 1/2 rank:", f->shared->btree_k[H5B_SNODE_ID]);
     HDfprintf(stream, "%*s%-*s 0x%02x\n", indent, "", fwidth,
 	      "File status flags:", (unsigned)(f->shared->status_flags));
-    HDfprintf(stream, "%*s%-*s %a (abs)\n", indent, "", fwidth,
-	      "Base address:", f->shared->base_addr);
     HDfprintf(stream, "%*s%-*s %a (rel)\n", indent, "", fwidth,
 	      "Superblock extension address:", f->shared->extension_addr);
     HDfprintf(stream, "%*s%-*s %a (rel)\n", indent, "", fwidth,
@@ -125,21 +123,26 @@ H5F_debug(H5F_t *f, FILE *stream, int indent, int fwidth)
 	      "Root group symbol table entry:",
 	      f->shared->root_grp ? "" : "(none)");
     if(f->shared->root_grp) {
-        H5O_loc_t *root_oloc;   /* Root object location */
-        H5G_entry_t root_ent;   /* Constructed root symbol table entry */
+        if(f->shared->root_ent) /* Use real root group symbol table entry */
+            H5G_ent_debug(f, f->shared->root_ent, stream, indent + 3,
+                MAX(0, fwidth - 3), NULL);
+        else {
+            H5O_loc_t *root_oloc;   /* Root object location */
+            H5G_entry_t root_ent;   /* Constructed root symbol table entry */
 
-        /* Reset the root group entry */
-        H5G_ent_reset(&root_ent);
+            /* Reset the root group entry */
+            H5G_ent_reset(&root_ent);
 
-        /* Build up a simulated root group symbol table entry */
-        root_oloc = H5G_oloc(f->shared->root_grp);
-        HDassert(root_oloc);
-        root_ent.type = H5G_NOTHING_CACHED;
-        root_ent.header = root_oloc->addr;
-        root_ent.file = f;
+            /* Build up a simulated root group symbol table entry */
+            root_oloc = H5G_oloc(f->shared->root_grp);
+            HDassert(root_oloc);
+            root_ent.type = H5G_NOTHING_CACHED;
+            root_ent.header = root_oloc->addr;
+            root_ent.file = f;
 
-        /* Display root group symbol table entry info */
-	H5G_ent_debug(f, &root_ent, stream, indent + 3, MAX(0, fwidth - 3), NULL);
+            /* Display root group symbol table entry info */
+            H5G_ent_debug(f, &root_ent, stream, indent + 3, MAX(0, fwidth - 3), NULL);
+        }
     } /* end if */
 
 done:
