@@ -21,8 +21,18 @@
 
 /* Public header files needed by this file */
 #include "H5public.h"
+#include "H5Cpublic.h"
+#include "H5ACpublic.h"
 #include "H5Ipublic.h"
 
+/* When this header is included from a private header, don't make calls to H5check() */
+#undef H5CHECK
+#ifndef _H5private_H
+#define H5CHECK          H5check(),
+#else   /* _H5private_H */
+#define H5CHECK
+#endif  /* _H5private_H */
+
 /*
  * These are the bits that can be passed to the `flags' argument of
  * H5Fcreate() and H5Fopen(). Use the bit-wise OR operator (|) to combine
@@ -32,18 +42,7 @@
  * We're assuming that these constants are used rather early in the hdf5
  * session.
  *
- * NOTE: When adding H5F_ACC_* macros, remember to redefine them in H5Fprivate.h
- *
  */
-
-/* When this header is included from a private header, don't make calls to H5check() */
-#undef H5CHECK
-#ifndef _H5private_H
-#define H5CHECK          H5check(),
-#else   /* _H5private_H */
-#define H5CHECK
-#endif  /* _H5private_H */
-
 #define H5F_ACC_RDONLY	(H5CHECK 0x0000u)	/*absence of rdwr => rd-only */
 #define H5F_ACC_RDWR	(H5CHECK 0x0001u)	/*open for read and write    */
 #define H5F_ACC_TRUNC	(H5CHECK 0x0002u)	/*overwrite existing files   */
@@ -51,6 +50,10 @@
 #define H5F_ACC_DEBUG	(H5CHECK 0x0008u)	/*print debug info	     */
 #define H5F_ACC_CREAT	(H5CHECK 0x0010u)	/*create non-existing files  */
 
+/* Value passed to H5Pset_elink_acc_flags to cause flags to be taken from the
+ * parent file. */
+#define H5F_ACC_DEFAULT (H5CHECK 0xffffu)	/*ignore setting on lapl     */
+
 /* Flags for H5Fget_obj_count() & H5Fget_obj_ids() calls */
 #define H5F_OBJ_FILE	(0x0001u)       /* File objects */
 #define H5F_OBJ_DATASET	(0x0002u)       /* Dataset objects */
@@ -61,6 +64,8 @@
 #define H5F_OBJ_LOCAL   (0x0020u)       /* Restrict search to objects opened through current file ID */
                                         /* (as opposed to objects opened through any file ID accessing this file) */
 
+#define H5F_FAMILY_DEFAULT (hsize_t)0
+
 #ifdef H5_HAVE_PARALLEL
 /*
  * Use this constant string as the MPI_Info key to set H5Fmpio debug flags.
@@ -96,31 +101,61 @@ typedef enum H5F_close_degree_t {
     H5F_CLOSE_STRONG    = 3
 } H5F_close_degree_t;
 
+/* Current "global" information about file */
+/* (just size info currently) */
+typedef struct H5F_info_t {
+    hsize_t		super_ext_size;	/* Superblock extension size */
+    struct {
+	hsize_t		hdr_size;       /* Shared object header message header size */
+	H5_ih_info_t	msgs_info;      /* Shared object header message index & heap size */
+    } sohm;
+} H5F_info_t;
+
+/* Library's file format versions */
+typedef enum H5F_libver_t {
+    H5F_LIBVER_EARLIEST,        /* Use the earliest possible format for storing objects */
+    H5F_LIBVER_LATEST           /* Use the latest possible format available for storing objects*/
+} H5F_libver_t;
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
 /* Functions in H5F.c */
-H5_DLL htri_t H5Fis_hdf5 (const char *filename);
-H5_DLL hid_t  H5Fcreate (const char *filename, unsigned flags,
+H5_DLL htri_t H5Fis_hdf5(const char *filename);
+H5_DLL hid_t  H5Fcreate(const char *filename, unsigned flags,
 		  	  hid_t create_plist, hid_t access_plist);
-H5_DLL hid_t  H5Fopen (const char *filename, unsigned flags,
+H5_DLL hid_t  H5Fopen(const char *filename, unsigned flags,
 		        hid_t access_plist);
 H5_DLL hid_t  H5Freopen(hid_t file_id);
 H5_DLL herr_t H5Fflush(hid_t object_id, H5F_scope_t scope);
-H5_DLL herr_t H5Fclose (hid_t file_id);
-H5_DLL hid_t  H5Fget_create_plist (hid_t file_id);
-H5_DLL hid_t  H5Fget_access_plist (hid_t file_id);
-H5_DLL int H5Fget_obj_count(hid_t file_id, unsigned types);
-H5_DLL int H5Fget_obj_ids(hid_t file_id, unsigned types, int max_objs, hid_t *obj_id_list);
-H5_DLL herr_t H5Fget_vfd_handle(hid_t file_id, hid_t fapl, void** file_handle);
+H5_DLL herr_t H5Fclose(hid_t file_id);
+H5_DLL hid_t  H5Fget_create_plist(hid_t file_id);
+H5_DLL hid_t  H5Fget_access_plist(hid_t file_id);
+H5_DLL herr_t H5Fget_intent(hid_t file_id, unsigned * intent);
+H5_DLL ssize_t H5Fget_obj_count(hid_t file_id, unsigned types);
+H5_DLL ssize_t H5Fget_obj_ids(hid_t file_id, unsigned types, size_t max_objs, hid_t *obj_id_list);
+H5_DLL herr_t H5Fget_vfd_handle(hid_t file_id, hid_t fapl, void **file_handle);
 H5_DLL herr_t H5Fmount(hid_t loc, const char *name, hid_t child, hid_t plist);
 H5_DLL herr_t H5Funmount(hid_t loc, const char *name);
 H5_DLL hssize_t H5Fget_freespace(hid_t file_id);
 H5_DLL herr_t H5Fget_filesize(hid_t file_id, hsize_t *size);
+H5_DLL herr_t H5Fget_mdc_config(hid_t file_id,
+				H5AC_cache_config_t * config_ptr);
+H5_DLL herr_t H5Fset_mdc_config(hid_t file_id,
+				H5AC_cache_config_t * config_ptr);
+H5_DLL herr_t H5Fget_mdc_hit_rate(hid_t file_id, double * hit_rate_ptr);
+H5_DLL herr_t H5Fget_mdc_size(hid_t file_id,
+                              size_t * max_size_ptr,
+                              size_t * min_clean_size_ptr,
+                              size_t * cur_size_ptr,
+                              int * cur_num_entries_ptr);
+H5_DLL herr_t H5Freset_mdc_hit_rate_stats(hid_t file_id);
 H5_DLL ssize_t H5Fget_name(hid_t obj_id, char *name, size_t size);
+H5_DLL herr_t H5Fget_info(hid_t obj_id, H5F_info_t *bh_info);
 
 #ifdef __cplusplus
 }
 #endif
-#endif
+#endif /* _H5Fpublic_H */
+
