@@ -43,9 +43,6 @@
 #  undef H5G_DEBUG
 #endif
 
-#define H5G_NODE_MAGIC  "SNOD"          /*symbol table node magic number     */
-#define H5G_NODE_SIZEOF_MAGIC 4         /*sizeof symbol node magic number    */
-
 /*
  * The disk size for a symbol table entry...
  */
@@ -101,6 +98,12 @@
                                                     H5G_CRT_GINFO_EST_NAME_LEN \
                                                 }
 
+/* If the module using this macro is allowed access to the private variables, access them directly */
+#ifdef H5G_PACKAGE
+#define H5G_MOUNTED(G)              ((G)->shared->mounted)
+#else /* H5G_PACKAGE */
+#define H5G_MOUNTED(G)              (H5G_mounted(G))
+#endif /* H5G_PACKAGE */
 
 /* Type of operation being performed for call to H5G_name_replace() */
 typedef enum {
@@ -142,12 +145,13 @@ typedef struct {
 
 typedef struct H5G_t H5G_t;
 typedef struct H5G_shared_t H5G_shared_t;
+typedef struct H5G_entry_t H5G_entry_t;
 
 /*
  * Library prototypes...  These are the ones that other packages routinely
  * call.
  */
-H5_DLL herr_t H5G_mkroot(H5F_t *f, hid_t dxpl_id, H5G_loc_t *root_loc);
+H5_DLL herr_t H5G_mkroot(H5F_t *f, hid_t dxpl_id, hbool_t create_root);
 H5_DLL struct H5O_loc_t *H5G_oloc(H5G_t *grp);
 H5_DLL H5G_t *H5G_rootof(H5F_t *f);
 H5_DLL H5G_name_t * H5G_nameof(H5G_t *grp);
@@ -158,6 +162,7 @@ H5_DLL herr_t H5G_close(H5G_t *grp);
 H5_DLL herr_t H5G_free_grp_name(H5G_t *grp);
 H5_DLL herr_t H5G_get_shared_count(H5G_t *grp);
 H5_DLL herr_t H5G_mount(H5G_t *grp);
+H5_DLL hbool_t H5G_mounted(H5G_t *grp);
 H5_DLL herr_t H5G_unmount(H5G_t *grp);
 #ifndef H5_NO_DEPRECATED_SYMBOLS
 H5_DLL H5G_obj_t H5G_map_obj_type(H5O_type_t obj_type);
@@ -176,10 +181,8 @@ H5_DLL herr_t H5G_node_debug(H5F_t *f, hid_t dxpl_id, haddr_t addr, FILE *stream
 /*
  * These functions operate on group object locations.
  */
-H5_DLL herr_t H5G_obj_ent_decode(H5F_t *f, const uint8_t **pp,
-    struct H5O_loc_t *oloc);
-H5_DLL herr_t H5G_obj_ent_encode(const H5F_t *f, uint8_t **pp,
-    const struct H5O_loc_t *oloc);
+H5_DLL herr_t H5G_root_ent_decode(H5F_t *f, const uint8_t **pp);
+H5_DLL herr_t H5G_root_ent_encode(H5F_t *f, uint8_t **pp);
 
 /*
  * These functions operate on group hierarchy names.
