@@ -32,6 +32,10 @@
 #include "H5Fprivate.h"         /* File access				*/
 #include "H5FDprivate.h"	/* File Drivers				*/
 
+/**************************/
+/* Library Private Macros */
+/**************************/
+
 /*
  * Feature: Define H5MF_DEBUG on the compiler command line if you want to
  *	    see diagnostics from this layer.
@@ -40,18 +44,44 @@
 #  undef H5MF_DEBUG
 #endif
 
-/*
- * Library prototypes...
- */
+/****************************/
+/* Library Private Typedefs */
+/****************************/
+
+
+/*****************************/
+/* Library-private Variables */
+/*****************************/
+
+
+/***************************************/
+/* Library-private Function Prototypes */
+/***************************************/
+
+/* File space manager routines */
+H5_DLL herr_t H5MF_init_merge_flags(H5F_t *f);
+H5_DLL hssize_t H5MF_get_freespace(H5F_t *f, hid_t dxpl_id);
+H5_DLL herr_t H5MF_close(H5F_t *f, hid_t dxpl_id);
+
+/* File space allocation routines */
 H5_DLL haddr_t H5MF_alloc(H5F_t *f, H5FD_mem_t type, hid_t dxpl_id, hsize_t size);
 H5_DLL herr_t H5MF_xfree(H5F_t *f, H5FD_mem_t type, hid_t dxpl_id, haddr_t addr,
 			  hsize_t size);
-H5_DLL haddr_t H5MF_realloc(H5F_t *f, H5FD_mem_t type, hid_t dxpl_id, haddr_t old_addr,
-			     hsize_t old_size, hsize_t new_size);
-H5_DLL htri_t H5MF_can_extend(H5F_t *f, H5FD_mem_t type, haddr_t addr,
-    hsize_t size, hsize_t extra_requested);
-H5_DLL herr_t H5MF_extend(H5F_t *f, H5FD_mem_t type, haddr_t addr, hsize_t size,
-    hsize_t extra_requested);
+H5_DLL herr_t H5MF_try_extend(H5F_t *f, hid_t dxpl_id, H5FD_mem_t type,
+    haddr_t addr, hsize_t size, hsize_t extra_requested);
+H5_DLL htri_t H5MF_try_shrink(H5F_t *f, H5FD_mem_t alloc_type, hid_t dxpl_id,
+    haddr_t addr, hsize_t size);
+
+/* 'block aggregator' routines */
+H5_DLL herr_t H5MF_aggr_reset(H5F_t *file, hid_t dxpl_id, H5F_blk_aggr_t *aggr);
+
+/* Debugging routines */
+#ifdef H5MF_DEBUGGING
+#ifdef NOT_YET
+H5_DLL herr_t H5MF_sects_debug(H5F_t *f, hid_t dxpl_id, haddr_t addr,
+    FILE *stream, int indent, int fwidth);
+#endif /* NOT_YET */
+#endif /* H5MF_DEBUGGING */
 
 #endif /* end _H5MFprivate_H */
 
