@@ -33,7 +33,8 @@
 
 
 /* PRIVATE PROTOTYPES */
-static void *H5O_layout_decode(H5F_t *f, hid_t dxpl_id, unsigned mesg_flags, const uint8_t *p);
+static void *H5O_layout_decode(H5F_t *f, hid_t dxpl_id, H5O_t *open_oh,
+    unsigned mesg_flags, unsigned *ioflags, const uint8_t *p);
 static herr_t H5O_layout_encode(H5F_t *f, hbool_t disable_shared, uint8_t *p, const void *_mesg);
 static void *H5O_layout_copy(const void *_mesg, void *_dest);
 static size_t H5O_layout_size(const H5F_t *f, hbool_t disable_shared, const void *_mesg);
@@ -71,12 +72,6 @@ const H5O_msg_class_t H5O_MSG_LAYOUT[1] = {{
     H5O_layout_debug       	/*debug the message             */
 }};
 
-/* For forward and backward compatibility.  Version is 1 when space is
- * allocated; 2 when space is delayed for allocation; 3 is default now and
- * is revised to just store information needed for each storage type. */
-#define H5O_LAYOUT_VERSION_1	1
-#define H5O_LAYOUT_VERSION_2	2
-#define H5O_LAYOUT_VERSION_3	3
 
 /* Declare a free list to manage the H5O_layout_t struct */
 H5FL_DEFINE(H5O_layout_t);
@@ -98,8 +93,8 @@ H5FL_DEFINE(H5O_layout_t);
  *-------------------------------------------------------------------------
  */
 static void *
-H5O_layout_decode(H5F_t *f, hid_t UNUSED dxpl_id, unsigned UNUSED mesg_flags,
-    const uint8_t *p)
+H5O_layout_decode(H5F_t *f, hid_t UNUSED dxpl_id, H5O_t UNUSED *open_oh,
+    unsigned UNUSED mesg_flags, unsigned UNUSED *ioflags, const uint8_t *p)
 {
     H5O_layout_t           *mesg = NULL;
     unsigned               u;
@@ -115,7 +110,6 @@ H5O_layout_decode(H5F_t *f, hid_t UNUSED dxpl_id, unsigned UNUSED mesg_flags,
     if(NULL == (mesg = H5FL_CALLOC(H5O_layout_t)))
         HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
 
-    /* Version. 1 when space allocated; 2 when space allocation is delayed */
     mesg->version = *p++;
     if(mesg->version < H5O_LAYOUT_VERSION_1 || mesg->version > H5O_LAYOUT_VERSION_3)
         HGOTO_ERROR(H5E_OHDR, H5E_CANTLOAD, NULL, "bad version number for layout message")
@@ -136,10 +130,30 @@ H5O_layout_decode(H5F_t *f, hid_t UNUSED dxpl_id, unsigned UNUSED mesg_flags,
         p += 5;
 
         /* Address */
-        if(mesg->type == H5D_CONTIGUOUS)
+        if(mesg->type == H5D_CONTIGUOUS) {
             H5F_addr_decode(f, &p, &(mesg->u.contig.addr));
-        else if(mesg->type == H5D_CHUNKED)
-            H5F_addr_decode(f, &p, &(mesg->u.chunk.addr));
+
+            /* Set the layout operations */
+            mesg->ops = H5D_LOPS_CONTIG;
+        } /* end if */
+        else if(mesg->type == H5D_CHUNKED) {
+            H5F_addr_decode(f, &p, &(mesg->u.chunk.u.btree.addr));
+
+            /* Set the layout operations */
+            mesg->ops = H5D_LOPS_CHUNK;
+
+            /* Set the chunk operations */
+            /* (Only "btree" indexing type currently supported in this version) */
+            mesg->u.chunk.idx_type = H5D_CHUNK_BTREE;
+            mesg->u.chunk.ops = H5D_COPS_BTREE;
+        } /* end if */
+        else {
+            /* Sanity check */
+            HDassert(mesg->type == H5D_COMPACT);
+
+            /* Set the layout operations */
+            mesg->ops = H5D_LOPS_COMPACT;
+        } /* end else */
 
         /* Read the size */
         if(mesg->type != H5D_CHUNKED) {
@@ -185,11 +199,17 @@ H5O_layout_decode(H5F_t *f, hid_t UNUSED dxpl_id, unsigned UNUSED mesg_flags,
                     HDmemcpy(mesg->u.compact.buf, p, mesg->u.compact.size);
                     p += mesg->u.compact.size;
                 } /* end if */
+
+                /* Set the layout operations */
+                mesg->ops = H5D_LOPS_COMPACT;
                 break;
 
             case H5D_CONTIGUOUS:
                 H5F_addr_decode(f, &p, &(mesg->u.contig.addr));
                 H5F_DECODE_LENGTH(f, p, mesg->u.contig.size);
+
+                /* Set the layout operations */
+                mesg->ops = H5D_LOPS_CONTIG;
                 break;
 
             case H5D_CHUNKED:
@@ -199,7 +219,7 @@ H5O_layout_decode(H5F_t *f, hid_t UNUSED dxpl_id, unsigned UNUSED mesg_flags,
                     HGOTO_ERROR(H5E_OHDR, H5E_CANTLOAD, NULL, "dimensionality is too large")
 
                 /* B-tree address */
-                H5F_addr_decode(f, &p, &(mesg->u.chunk.addr));
+                H5F_addr_decode(f, &p, &(mesg->u.chunk.u.btree.addr));
 
                 /* Chunk dimensions */
                 for(u = 0; u < mesg->u.chunk.ndims; u++)
@@ -208,6 +228,14 @@ H5O_layout_decode(H5F_t *f, hid_t UNUSED dxpl_id, unsigned UNUSED mesg_flags,
                 /* Compute chunk size */
                 for(u = 1, mesg->u.chunk.size = mesg->u.chunk.dim[0]; u < mesg->u.chunk.ndims; u++)
                     mesg->u.chunk.size *= mesg->u.chunk.dim[u];
+
+                /* Set the chunk operations */
+                /* (Only "btree" indexing type supported with v3 of message format) */
+                mesg->u.chunk.idx_type = H5D_CHUNK_BTREE;
+                mesg->u.chunk.ops = H5D_COPS_BTREE;
+
+                /* Set the layout operations */
+                mesg->ops = H5D_LOPS_CHUNK;
                 break;
 
             default:
@@ -221,7 +249,7 @@ H5O_layout_decode(H5F_t *f, hid_t UNUSED dxpl_id, unsigned UNUSED mesg_flags,
 done:
     if(ret_value == NULL)
         if(mesg)
-            H5FL_FREE(H5O_layout_t, mesg);
+            (void)H5FL_FREE(H5O_layout_t, mesg);
 
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5O_layout_decode() */
@@ -271,8 +299,8 @@ H5O_layout_encode(H5F_t *f, hbool_t UNUSED disable_shared, uint8_t *p, const voi
     HDassert(mesg);
     HDassert(p);
 
-    /* Version 3 by default now. */
-    *p++ = H5O_LAYOUT_VERSION_3;
+    /* Message version */
+    *p++ = (uint8_t)H5O_LAYOUT_VERSION_3;
 
     /* Layout class */
     *p++ = mesg->type;
@@ -301,10 +329,10 @@ H5O_layout_encode(H5F_t *f, hbool_t UNUSED disable_shared, uint8_t *p, const voi
         case H5D_CHUNKED:
             /* Number of dimensions */
             HDassert(mesg->u.chunk.ndims > 0 && mesg->u.chunk.ndims <= H5O_LAYOUT_NDIMS);
-            *p++ = mesg->u.chunk.ndims;
+            *p++ = (uint8_t)mesg->u.chunk.ndims;
 
             /* B-tree address */
-            H5F_addr_encode(f, &p, mesg->u.chunk.addr);
+            H5F_addr_encode(f, &p, mesg->u.chunk.u.btree.addr);
 
             /* Dimension sizes */
             for(u = 0; u < mesg->u.chunk.ndims; u++)
@@ -353,7 +381,7 @@ H5O_layout_copy(const void *_mesg, void *_dest)
     *dest = *mesg;
 
     /* Deep copy the buffer for compact datasets also */
-    if(mesg->type == H5D_COMPACT) {
+    if(mesg->type == H5D_COMPACT && mesg->u.compact.size > 0) {
         /* Allocate memory for the raw data */
         if(NULL == (dest->u.compact.buf = H5MM_malloc(dest->u.compact.size)))
             HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "unable to allocate memory for compact dataset")
@@ -369,69 +397,6 @@ done:
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5O_layout_copy() */
 
-
-/*-------------------------------------------------------------------------
- * Function:    H5O_layout_meta_size
- *
- * Purpose:     Returns the size of the raw message in bytes except raw data
- *              part for compact dataset.  This function doesn't take into
- *              account message alignment.
- *
- * Return:      Success:        Message data size in bytes(except raw data
- *                              for compact dataset)
- *              Failure:        0
- *
- * Programmer:  Raymond Lu
- *              August 14, 2002
- *
- *-------------------------------------------------------------------------
- */
-size_t
-H5O_layout_meta_size(const H5F_t *f, const void *_mesg)
-{
-    const H5O_layout_t      *mesg = (const H5O_layout_t *) _mesg;
-    size_t                  ret_value;
-
-    FUNC_ENTER_NOAPI_NOINIT(H5O_layout_meta_size)
-
-    /* check args */
-    HDassert(f);
-    HDassert(mesg);
-
-    ret_value = 1 +                     /* Version number                       */
-                1;                      /* layout class type                    */
-
-    switch(mesg->type) {
-        case H5D_COMPACT:
-            /* Size of raw data */
-            ret_value += 2;
-            break;
-
-        case H5D_CONTIGUOUS:
-            ret_value += H5F_SIZEOF_ADDR(f);    /* Address of data */
-            ret_value += H5F_SIZEOF_SIZE(f);    /* Length of data */
-            break;
-
-        case H5D_CHUNKED:
-            /* Number of dimensions (1 byte) */
-            assert(mesg->u.chunk.ndims > 0 && mesg->u.chunk.ndims <= H5O_LAYOUT_NDIMS);
-            ret_value++;
-
-            /* B-tree address */
-            ret_value += H5F_SIZEOF_ADDR(f);    /* Address of data */
-
-            /* Dimension sizes */
-            ret_value += mesg->u.chunk.ndims * 4;
-            break;
-
-        default:
-            HGOTO_ERROR(H5E_OHDR, H5E_CANTENCODE, 0, "Invalid layout class")
-    } /* end switch */
-
-done:
-    FUNC_LEAVE_NOAPI(ret_value)
-} /* end H5O_layout_meta_size() */
-
 
 /*-------------------------------------------------------------------------
  * Function:    H5O_layout_size
@@ -519,22 +484,22 @@ H5O_layout_reset (void *_mesg)
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5O_layout_free (void *_mesg)
+H5O_layout_free(void *_mesg)
 {
     H5O_layout_t     *mesg = (H5O_layout_t *) _mesg;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_layout_free);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_layout_free)
 
-    assert (mesg);
+    HDassert(mesg);
 
     /* Free the compact storage buffer */
-    if(mesg->type==H5D_COMPACT)
-        mesg->u.compact.buf=H5MM_xfree(mesg->u.compact.buf);
+    if(mesg->type == H5D_COMPACT)
+        mesg->u.compact.buf = H5MM_xfree(mesg->u.compact.buf);
 
-    H5FL_FREE(H5O_layout_t,mesg);
+    (void)H5FL_FREE(H5O_layout_t, mesg);
 
-    FUNC_LEAVE_NOAPI(SUCCEED);
-}
+    FUNC_LEAVE_NOAPI(SUCCEED)
+} /* end H5O_layout_free() */
 
 
 /*-------------------------------------------------------------------------
@@ -576,8 +541,8 @@ H5O_layout_delete(H5F_t *f, hid_t dxpl_id, H5O_t UNUSED *open_oh, void *_mesg)
             break;
 
         case H5D_CHUNKED:       /* Chunked blocks on disk */
-            /* Free the file space for the raw data */
-            if(H5D_istore_delete(f, dxpl_id, mesg) < 0)
+            /* Free the file space for the index & chunk raw data */
+            if(H5D_chunk_delete(f, dxpl_id, mesg) < 0)
                 HGOTO_ERROR(H5E_OHDR, H5E_CANTFREE, FAIL, "unable to free raw data")
             break;
 
@@ -639,9 +604,6 @@ H5O_layout_copy_file(H5F_t *file_src, void *mesg_src, H5F_t *file_dst,
                     HGOTO_ERROR(H5E_IO, H5E_CANTINIT, NULL, "unable to copy chunked storage")
 
             	layout_dst->u.compact.dirty = TRUE;
-
-                /* Freed by copy routine */
-                udata->src_dtype = NULL;
 	    } /* end if */
             break;
 
@@ -658,23 +620,18 @@ H5O_layout_copy_file(H5F_t *file_src, void *mesg_src, H5F_t *file_dst,
                 /* create contig layout */
                 if(H5D_contig_copy(file_src, layout_src, file_dst, layout_dst, udata->src_dtype, cpy_info, dxpl_id) < 0)
                     HGOTO_ERROR(H5E_IO, H5E_CANTINIT, NULL, "unable to copy contiguous storage")
-
-                /* Freed by copy routine */
-                udata->src_dtype = NULL;
             } /*  if ( H5F_addr_defined(layout_src->u.contig.addr)) */
             break;
 
         case H5D_CHUNKED:
-            if(H5F_addr_defined(layout_src->u.chunk.addr)) {
-                /* layout is not created in the destination file, undef btree address */
-                layout_dst->u.chunk.addr = HADDR_UNDEF;
+            if(H5D_chunk_is_space_alloc(layout_src)) {
+                /* Layout is not created in the destination file, reset index address */
+                if(H5D_chunk_idx_reset(layout_dst) < 0)
+                    HGOTO_ERROR(H5E_IO, H5E_CANTINIT, NULL, "unable to reset chunked storage index in dest")
 
-                /* create chunked layout */
-                if(H5D_istore_copy(file_src, layout_src, file_dst, layout_dst, udata->src_dtype, cpy_info, udata->src_pline, dxpl_id) < 0)
+                /* Create chunked layout */
+                if(H5D_chunk_copy(file_src, layout_src, file_dst, layout_dst, udata->src_dtype, cpy_info, udata->src_pline, dxpl_id) < 0)
                     HGOTO_ERROR(H5E_IO, H5E_CANTINIT, NULL, "unable to copy chunked storage")
-
-                /* Freed by copy routine */
-                udata->src_dtype = NULL;
             } /* if ( H5F_addr_defined(layout_srct->u.chunk.addr)) */
             break;
 
@@ -682,13 +639,16 @@ H5O_layout_copy_file(H5F_t *file_src, void *mesg_src, H5F_t *file_dst,
             HGOTO_ERROR(H5E_OHDR, H5E_CANTLOAD, NULL, "Invalid layout class")
     } /* end switch */
 
+    /* Freed by copy routine */
+    udata->src_dtype = NULL;
+
     /* Set return value */
     ret_value = layout_dst;
 
 done:
     if(!ret_value)
 	if(layout_dst)
-	    H5FL_FREE(H5O_layout_t, layout_dst);
+	    (void)H5FL_FREE(H5O_layout_t, layout_dst);
 
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5O_layout_copy_file() */
@@ -709,13 +669,13 @@ done:
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5O_layout_debug(H5F_t UNUSED *f, hid_t UNUSED dxpl_id, const void *_mesg, FILE * stream,
-		 int indent, int fwidth)
+H5O_layout_debug(H5F_t UNUSED *f, hid_t UNUSED dxpl_id, const void *_mesg,
+    FILE * stream, int indent, int fwidth)
 {
     const H5O_layout_t     *mesg = (const H5O_layout_t *) _mesg;
     unsigned                    u;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_layout_debug);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_layout_debug)
 
     /* check args */
     HDassert(f);
@@ -726,37 +686,121 @@ H5O_layout_debug(H5F_t UNUSED *f, hid_t UNUSED dxpl_id, const void *_mesg, FILE
 
     HDfprintf(stream, "%*s%-*s %u\n", indent, "", fwidth,
               "Version:", mesg->version);
-    if(mesg->type==H5D_CHUNKED) {
-        HDfprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
-                  "Type:", "Chunked");
-        HDfprintf(stream, "%*s%-*s %a\n", indent, "", fwidth,
-                  "B-tree address:", mesg->u.chunk.addr);
-        HDfprintf(stream, "%*s%-*s %lu\n", indent, "", fwidth,
-                  "Number of dimensions:",
-                  (unsigned long) (mesg->u.chunk.ndims));
-        /* Size */
-        HDfprintf(stream, "%*s%-*s {", indent, "", fwidth, "Size:");
-        for (u = 0; u < mesg->u.chunk.ndims; u++) {
-            HDfprintf(stream, "%s%lu", u ? ", " : "",
-                      (unsigned long) (mesg->u.chunk.dim[u]));
-        }
-        HDfprintf(stream, "}\n");
-    } /* end if */
-    else if(mesg->type==H5D_CONTIGUOUS) {
-        HDfprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
-                  "Type:", "Contiguous");
-        HDfprintf(stream, "%*s%-*s %a\n", indent, "", fwidth,
-                  "Data address:", mesg->u.contig.addr);
-        HDfprintf(stream, "%*s%-*s %Hu\n", indent, "", fwidth,
-                  "Data Size:", mesg->u.contig.size);
-    } /* end if */
-    else {
-        HDfprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
-                  "Type:", "Compact");
-        HDfprintf(stream, "%*s%-*s %Zu\n", indent, "", fwidth,
-                  "Data Size:", mesg->u.compact.size);
-    } /* end else */
+    switch(mesg->type) {
+        case H5D_CHUNKED:
+            HDfprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
+                      "Type:", "Chunked");
 
-    FUNC_LEAVE_NOAPI(SUCCEED);
-}
+            /* Chunk # of dims & size */
+            HDfprintf(stream, "%*s%-*s %lu\n", indent, "", fwidth,
+                      "Number of dimensions:",
+                      (unsigned long)(mesg->u.chunk.ndims));
+            HDfprintf(stream, "%*s%-*s {", indent, "", fwidth, "Size:");
+            for(u = 0; u < mesg->u.chunk.ndims; u++)
+                HDfprintf(stream, "%s%lu", u ? ", " : "", (unsigned long)(mesg->u.chunk.dim[u]));
+            HDfprintf(stream, "}\n");
+
+            /* Index information */
+            switch(mesg->u.chunk.idx_type) {
+                case H5D_CHUNK_BTREE:
+                    HDfprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
+                              "Index Type:", "v1 B-tree");
+                    HDfprintf(stream, "%*s%-*s %a\n", indent, "", fwidth,
+                              "B-tree address:", mesg->u.chunk.u.btree.addr);
+                    break;
+
+                default:
+                    HDfprintf(stream, "%*s%-*s %s (%u)\n", indent, "", fwidth,
+                              "Index Type:", "Unknown", (unsigned)mesg->u.chunk.idx_type);
+                    break;
+            } /* end switch */
+            break;
+
+        case H5D_CONTIGUOUS:
+            HDfprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
+                      "Type:", "Contiguous");
+            HDfprintf(stream, "%*s%-*s %a\n", indent, "", fwidth,
+                      "Data address:", mesg->u.contig.addr);
+            HDfprintf(stream, "%*s%-*s %Hu\n", indent, "", fwidth,
+                      "Data Size:", mesg->u.contig.size);
+            break;
+
+        case H5D_COMPACT:
+            HDfprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
+                      "Type:", "Compact");
+            HDfprintf(stream, "%*s%-*s %Zu\n", indent, "", fwidth,
+                      "Data Size:", mesg->u.compact.size);
+            break;
+
+        default:
+            HDfprintf(stream, "%*s%-*s %s (%u)\n", indent, "", fwidth,
+                      "Type:", "Unknown", (unsigned)mesg->type);
+            break;
+    } /* end switch */
+
+    FUNC_LEAVE_NOAPI(SUCCEED)
+} /* end H5O_layout_debug() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:    H5O_layout_meta_size
+ *
+ * Purpose:     Returns the size of the raw message in bytes except raw data
+ *              part for compact dataset.  This function doesn't take into
+ *              account message alignment.
+ *
+ * Return:      Success:        Message data size in bytes(except raw data
+ *                              for compact dataset)
+ *              Failure:        0
+ *
+ * Programmer:  Raymond Lu
+ *              August 14, 2002
+ *
+ *-------------------------------------------------------------------------
+ */
+size_t
+H5O_layout_meta_size(const H5F_t *f, const void *_mesg)
+{
+    const H5O_layout_t      *mesg = (const H5O_layout_t *) _mesg;
+    size_t                  ret_value;
+
+    FUNC_ENTER_NOAPI_NOINIT(H5O_layout_meta_size)
+
+    /* check args */
+    HDassert(f);
+    HDassert(mesg);
+
+    ret_value = 1 +                     /* Version number                       */
+                1;                      /* layout class type                    */
+
+    switch(mesg->type) {
+        case H5D_COMPACT:
+            /* Size of raw data */
+            ret_value += 2;
+            break;
+
+        case H5D_CONTIGUOUS:
+            ret_value += H5F_SIZEOF_ADDR(f);    /* Address of data */
+            ret_value += H5F_SIZEOF_SIZE(f);    /* Length of data */
+            break;
+
+        case H5D_CHUNKED:
+            /* Number of dimensions (1 byte) */
+            HDassert(mesg->u.chunk.ndims > 0 && mesg->u.chunk.ndims <= H5O_LAYOUT_NDIMS);
+            ret_value++;
+
+            /* Dimension sizes */
+            ret_value += mesg->u.chunk.ndims * 4;
+
+            /* B-tree address */
+            ret_value += H5F_SIZEOF_ADDR(f);    /* Address of data */
+            break;
+
+        default:
+            HGOTO_ERROR(H5E_OHDR, H5E_CANTENCODE, 0, "Invalid layout class")
+    } /* end switch */
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5O_layout_meta_size() */
 
