@@ -43,19 +43,6 @@ static herr_t H5S_set_extent_simple (H5S_t *space, unsigned rank,
 static htri_t H5S_is_simple(const H5S_t *sdim);
 static herr_t H5S_encode(H5S_t *obj, unsigned char *buf, size_t *nalloc);
 static H5S_t *H5S_decode(const unsigned char *buf);
-static htri_t H5S_extent_equal(const H5S_t *ds1, const H5S_t *ds2);
-
-#ifdef H5S_DEBUG
-/* Names of the selection names, for debugging */
-static const char *H5S_sel_names[]={
-    "none", "point", "hyperslab", "all"
-};
-
-/* The path table, variable length */
-static H5S_iostats_t		**H5S_iostats_g = NULL;
-static size_t			H5S_aiostats_g = 0;	/*entries allocated*/
-static size_t			H5S_niostats_g = 0;	/*entries used*/
-#endif /* H5S_DEBUG */
 
 #ifdef H5_HAVE_PARALLEL
 /* Global vars whose value can be set from environment variable also */
@@ -87,9 +74,9 @@ DESCRIPTION
 static herr_t
 H5S_init_interface(void)
 {
-    herr_t ret_value=SUCCEED;   /* Return value */
+    herr_t ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5S_init_interface);
+    FUNC_ENTER_NOAPI_NOINIT(H5S_init_interface)
 
     /* Initialize the atom group for the file IDs */
     if(H5I_register_type(H5I_DATASPACE, (size_t)H5I_DATASPACEID_HASHSIZE, H5S_RESERVED_ATOMS, (H5I_free_t)H5S_close) < 0)
@@ -105,8 +92,8 @@ H5S_init_interface(void)
 #endif /* H5_HAVE_PARALLEL */
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
-}
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5S_init_interface() */
 
 
 /*--------------------------------------------------------------------------
@@ -129,156 +116,26 @@ done:
 int
 H5S_term_interface(void)
 {
-    int	n=0;
-#ifdef H5S_DEBUG
-    size_t	i;
-    int		j, nprints=0;
-    H5S_iostats_t	*path=NULL;
-    char	buf[256];
-#endif /* H5S_DEBUG */
+    int	n = 0;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5S_term_interface);
-
-    if (H5_interface_initialize_g) {
-	if ((n=H5I_nmembers(H5I_DATASPACE))) {
-	    H5I_clear_type(H5I_DATASPACE, FALSE);
-	} else {
-#ifdef H5S_DEBUG
-	    /*
-	     * Print statistics about each conversion path.
-	     */
-	    if (H5DEBUG(S)) {
-		for (i=0; i<H5S_niostats_g; i++) {
-		    path = H5S_iostats_g[i];
-		    for (j=0; j<2; j++) {
-			if (0==path->stats[j].gath_ncalls &&
-			    0==path->stats[j].scat_ncalls &&
-			    0==path->stats[j].bkg_ncalls &&
-			    0==path->stats[j].read_ncalls &&
-			    0==path->stats[j].write_ncalls) {
-			    continue;
-			}
-			if (0==nprints++) {
-			    fprintf(H5DEBUG(S), "H5S: data space conversion "
-				    "statistics:\n");
-			    fprintf(H5DEBUG(S),
-				    "   %-16s %10s %10s %8s %8s %8s %10s\n",
-				    "Memory <> File", "Bytes", "Calls",
-				    "User", "System", "Elapsed", "Bandwidth");
-			    fprintf(H5DEBUG(S),
-				    "   %-16s %10s %10s %8s %8s %8s %10s\n",
-				    "--------------", "-----", "-----",
-				    "----", "------", "-------", "---------");
-			}
-
-			/* Summary */
-			sprintf(buf, "%s %c %s",
-				H5S_sel_names[path->mtype], 0==j?'>':'<', H5S_sel_names[path->ftype]);
-			fprintf(H5DEBUG(S), "   %-16s\n", buf);
-
-			/* Gather */
-			if (path->stats[j].gath_ncalls) {
-			    H5_bandwidth(buf,
-					 (double)(path->stats[j].gath_nbytes),
-					 path->stats[j].gath_timer.etime);
-			    HDfprintf(H5DEBUG(S),
-				      "   %16s %10Hu %10Hu %8.2f %8.2f %8.2f "
-				      "%10s\n", "gather",
-				      path->stats[j].gath_nbytes,
-				      path->stats[j].gath_ncalls,
-				      path->stats[j].gath_timer.utime,
-				      path->stats[j].gath_timer.stime,
-				      path->stats[j].gath_timer.etime,
-				      buf);
-			}
-
-			/* Scatter */
-			if (path->stats[j].scat_ncalls) {
-			    H5_bandwidth(buf,
-					 (double)(path->stats[j].scat_nbytes),
-					 path->stats[j].scat_timer.etime);
-			    HDfprintf(H5DEBUG(S),
-				      "   %16s %10Hu %10Hu %8.2f %8.2f %8.2f "
-				      "%10s\n", "scatter",
-				      path->stats[j].scat_nbytes,
-				      path->stats[j].scat_ncalls,
-				      path->stats[j].scat_timer.utime,
-				      path->stats[j].scat_timer.stime,
-				      path->stats[j].scat_timer.etime,
-				      buf);
-			}
-
-			/* Background */
-			if (path->stats[j].bkg_ncalls) {
-			    H5_bandwidth(buf,
-					 (double)(path->stats[j].bkg_nbytes),
-					 path->stats[j].bkg_timer.etime);
-			    HDfprintf(H5DEBUG(S),
-				      "   %16s %10Hu %10Hu %8.2f %8.2f %8.2f "
-				      "%10s\n", "background",
-				      path->stats[j].bkg_nbytes,
-				      path->stats[j].bkg_ncalls,
-				      path->stats[j].bkg_timer.utime,
-				      path->stats[j].bkg_timer.stime,
-				      path->stats[j].bkg_timer.etime,
-				      buf);
-			}
-
-			/* Read */
-			if (path->stats[j].read_ncalls) {
-			    H5_bandwidth(buf,
-					 (double)(path->stats[j].read_nbytes),
-					 path->stats[j].read_timer.etime);
-			    HDfprintf(H5DEBUG(S),
-				      "   %16s %10Hu %10Hu %8.2f %8.2f %8.2f "
-				      "%10s\n", "read",
-				      path->stats[j].read_nbytes,
-				      path->stats[j].read_ncalls,
-				      path->stats[j].read_timer.utime,
-				      path->stats[j].read_timer.stime,
-				      path->stats[j].read_timer.etime,
-				      buf);
-			}
-
-			/* Write */
-			if (path->stats[j].write_ncalls) {
-			    H5_bandwidth(buf,
-					 (double)(path->stats[j].write_nbytes),
-					 path->stats[j].write_timer.etime);
-			    HDfprintf(H5DEBUG(S),
-				      "   %16s %10Hu %10Hu %8.2f %8.2f %8.2f "
-				      "%10s\n", "write",
-				      path->stats[j].write_nbytes,
-				      path->stats[j].write_ncalls,
-				      path->stats[j].write_timer.utime,
-				      path->stats[j].write_timer.stime,
-				      path->stats[j].write_timer.etime,
-				      buf);
-			}
-		    }
-		}
-	    }
-#endif /* H5S_DEBUG */
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5S_term_interface)
 
+    if(H5_interface_initialize_g) {
+	if((n = H5I_nmembers(H5I_DATASPACE))) {
+	    H5I_clear_type(H5I_DATASPACE, FALSE, FALSE);
+	} /* end if */
+        else {
 	    /* Free data types */
 	    H5I_dec_type_ref(H5I_DATASPACE);
 
-#ifdef H5S_DEBUG
-	    /* Clear/free conversion table */
-	    for (i=0; i<H5S_niostats_g; i++)
-                H5MM_xfree(H5S_iostats_g[i]);
-	    H5S_iostats_g = H5MM_xfree(H5S_iostats_g);
-	    H5S_niostats_g = H5S_aiostats_g = 0;
-#endif /* H5S_DEBUG */
-
 	    /* Shut down interface */
 	    H5_interface_initialize_g = 0;
 	    n = 1; /*H5I*/
-	}
-    }
+	} /* end else */
+    } /* end if */
 
-    FUNC_LEAVE_NOAPI(n);
-}
+    FUNC_LEAVE_NOAPI(n)
+} /* end H5S_term_interface() */
 
 
 /*--------------------------------------------------------------------------
@@ -307,7 +164,7 @@ H5S_create(H5S_class_t type)
 
     FUNC_ENTER_NOAPI(H5S_create, NULL)
 
-    /* Create a new data space */
+    /* Create a new dataspace */
     if(NULL == (new_ds = H5FL_MALLOC(H5S_t)))
         HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
 
@@ -392,8 +249,8 @@ H5Screate(H5S_class_t type)
         HGOTO_ERROR(H5E_DATASPACE, H5E_CANTCREATE, FAIL, "unable to create dataspace")
 
     /* Atomize */
-    if((ret_value = H5I_register (H5I_DATASPACE, new_ds)) < 0)
-        HGOTO_ERROR(H5E_ATOM, H5E_CANTREGISTER, FAIL, "unable to register data space atom")
+    if((ret_value = H5I_register (H5I_DATASPACE, new_ds, TRUE)) < 0)
+        HGOTO_ERROR(H5E_ATOM, H5E_CANTREGISTER, FAIL, "unable to register dataspace atom")
 
 done:
     if(ret_value < 0 && new_ds)
@@ -442,7 +299,7 @@ done:
 /*-------------------------------------------------------------------------
  * Function:	H5S_close
  *
- * Purpose:	Releases all memory associated with a data space.
+ * Purpose:	Releases all memory associated with a dataspace.
  *
  * Return:	Non-negative on success/Negative on failure
  *
@@ -467,7 +324,7 @@ H5S_close(H5S_t *ds)
     H5S_extent_release(&ds->extent);
 
     /* Release the main structure */
-    H5FL_FREE(H5S_t, ds);
+    (void)H5FL_FREE(H5S_t, ds);
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
@@ -477,7 +334,7 @@ done:
 /*-------------------------------------------------------------------------
  * Function:	H5Sclose
  *
- * Purpose:	Release access to a data space object.
+ * Purpose:	Release access to a dataspace object.
  *
  * Return:	Non-negative on success/Negative on failure
  *
@@ -500,10 +357,10 @@ H5Sclose(hid_t space_id)
 
     /* Check args */
     if (NULL == H5I_object_verify(space_id,H5I_DATASPACE))
-        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a data space")
+        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a dataspace")
 
     /* When the reference count reaches zero the resources are freed */
-    if (H5I_dec_ref(space_id) < 0)
+    if (H5I_dec_ref(space_id, TRUE) < 0)
         HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, FAIL, "problem freeing id")
 
 done:
@@ -539,15 +396,15 @@ H5Scopy(hid_t space_id)
 
     /* Check args */
     if (NULL==(src=(H5S_t *)H5I_object_verify(space_id, H5I_DATASPACE)))
-        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a data space")
+        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a dataspace")
 
     /* Copy */
     if (NULL == (dst = H5S_copy(src, FALSE, TRUE)))
-        HGOTO_ERROR(H5E_DATASPACE, H5E_CANTINIT, FAIL, "unable to copy data space")
+        HGOTO_ERROR(H5E_DATASPACE, H5E_CANTINIT, FAIL, "unable to copy dataspace")
 
     /* Atomize */
-    if ((ret_value=H5I_register (H5I_DATASPACE, dst))<0)
-        HGOTO_ERROR(H5E_ATOM, H5E_CANTREGISTER, FAIL, "unable to register data space atom")
+    if ((ret_value=H5I_register (H5I_DATASPACE, dst, TRUE))<0)
+        HGOTO_ERROR(H5E_ATOM, H5E_CANTREGISTER, FAIL, "unable to register dataspace atom")
 
 done:
     if(ret_value<0) {
@@ -585,9 +442,9 @@ H5Sextent_copy(hid_t dst_id,hid_t src_id)
 
     /* Check args */
     if(NULL == (src = (H5S_t *)H5I_object_verify(src_id, H5I_DATASPACE)))
-        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a data space")
+        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a dataspace")
     if(NULL == (dst = (H5S_t *)H5I_object_verify(dst_id, H5I_DATASPACE)))
-        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a data space")
+        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a dataspace")
 
     /* Copy */
     if(H5S_extent_copy(&(dst->extent), &(src->extent), TRUE) < 0)
@@ -651,7 +508,7 @@ H5S_extent_copy(H5S_extent_t *dst, const H5S_extent_t *src, hbool_t copy_max)
             break;
 
         default:
-            HDassert("unknown data space type" && 0);
+            HDassert("unknown dataspace type" && 0);
             break;
     } /* end switch */
 
@@ -667,7 +524,7 @@ done:
 /*-------------------------------------------------------------------------
  * Function:	H5S_copy
  *
- * Purpose:	Copies a data space, by copying the extent and selection through
+ * Purpose:	Copies a dataspace, by copying the extent and selection through
  *          H5S_extent_copy and H5S_select_copy.  If the SHARE_SELECTION flag
  *          is set, then the selection can be shared between the source and
  *          destination dataspaces.  (This should only occur in situations
@@ -738,17 +595,17 @@ H5S_get_simple_extent_npoints(const H5S_t *ds)
 {
     hssize_t    ret_value;
 
-    FUNC_ENTER_NOAPI(H5S_get_simple_extent_npoints, -1);
+    FUNC_ENTER_NOAPI(H5S_get_simple_extent_npoints, -1)
 
     /* check args */
-    assert(ds);
+    HDassert(ds);
 
     /* Get the number of elements in extent */
-    ret_value = ds->extent.nelem;
+    ret_value = (hssize_t)ds->extent.nelem;
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
-}
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5S_get_simple_extent_npoints() */
 
 
 /*-------------------------------------------------------------------------
@@ -774,30 +631,30 @@ H5Sget_simple_extent_npoints(hid_t space_id)
     H5S_t		   *ds;
     hssize_t		    ret_value;
 
-    FUNC_ENTER_API(H5Sget_simple_extent_npoints, FAIL);
+    FUNC_ENTER_API(H5Sget_simple_extent_npoints, FAIL)
     H5TRACE1("Hs", "i", space_id);
 
     /* Check args */
-    if (NULL == (ds = (H5S_t *)H5I_object_verify(space_id, H5I_DATASPACE)))
-        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a data space")
+    if(NULL == (ds = (H5S_t *)H5I_object_verify(space_id, H5I_DATASPACE)))
+        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a dataspace")
 
-    ret_value = H5S_GET_EXTENT_NPOINTS(ds);
+    ret_value = (hssize_t)H5S_GET_EXTENT_NPOINTS(ds);
 
 done:
-    FUNC_LEAVE_API(ret_value);
-}
+    FUNC_LEAVE_API(ret_value)
+} /* end H5Sget_simple_extent_npoints() */
 
 
 /*-------------------------------------------------------------------------
  * Function:	H5S_get_npoints_max
  *
- * Purpose:	Determines the maximum number of data points a data space may
+ * Purpose:	Determines the maximum number of data points a dataspace may
  *		have.  If the `max' array is null then the maximum number of
  *		data points is the same as the current number of data points
  *		without regard to the hyperslab.  If any element of the `max'
  *		array is zero then the maximum possible size is returned.
  *
- * Return:	Success:	Maximum number of data points the data space
+ * Return:	Success:	Maximum number of data points the dataspace
  *				may have.
  *
  *		Failure:	0
@@ -847,8 +704,8 @@ H5S_get_npoints_max(const H5S_t *ds)
             break;
 
         default:
-            assert("unknown data space class" && 0);
-            HGOTO_ERROR(H5E_DATASPACE, H5E_UNSUPPORTED, 0, "internal error (unknown data space class)")
+            assert("unknown dataspace class" && 0);
+            HGOTO_ERROR(H5E_DATASPACE, H5E_UNSUPPORTED, 0, "internal error (unknown dataspace class)")
     }
 
 done:
@@ -859,9 +716,9 @@ done:
 /*-------------------------------------------------------------------------
  * Function:	H5Sget_simple_extent_ndims
  *
- * Purpose:	Determines the dimensionality of a data space.
+ * Purpose:	Determines the dimensionality of a dataspace.
  *
- * Return:	Success:	The number of dimensions in a data space.
+ * Return:	Success:	The number of dimensions in a dataspace.
  *
  *		Failure:	Negative
  *
@@ -878,24 +735,24 @@ H5Sget_simple_extent_ndims(hid_t space_id)
     H5S_t		   *ds;
     int		   ret_value;
 
-    FUNC_ENTER_API(H5Sget_simple_extent_ndims, FAIL);
+    FUNC_ENTER_API(H5Sget_simple_extent_ndims, FAIL)
     H5TRACE1("Is", "i", space_id);
 
     /* Check args */
-    if (NULL == (ds = (H5S_t *)H5I_object_verify(space_id, H5I_DATASPACE)))
-        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a data space")
+    if(NULL == (ds = (H5S_t *)H5I_object_verify(space_id, H5I_DATASPACE)))
+        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a dataspace")
 
-    ret_value = H5S_GET_EXTENT_NDIMS(ds);
+    ret_value = (int)H5S_GET_EXTENT_NDIMS(ds);
 
 done:
-    FUNC_LEAVE_API(ret_value);
-}
+    FUNC_LEAVE_API(ret_value)
+} /* end H5Sget_simple_extent_ndims() */
 
 
 /*-------------------------------------------------------------------------
  * Function:	H5S_get_simple_extent_ndims
  *
- * Purpose:	Returns the number of dimensions in a data space.
+ * Purpose:	Returns the number of dimensions in a dataspace.
  *
  * Return:	Success:	Non-negative number of dimensions.  Zero
  *				implies a scalar.
@@ -916,35 +773,35 @@ done:
 int
 H5S_get_simple_extent_ndims(const H5S_t *ds)
 {
-    int		    ret_value;
+    int	ret_value;      /* Return value */
 
-    FUNC_ENTER_NOAPI(H5S_get_simple_extent_ndims, FAIL);
+    FUNC_ENTER_NOAPI(H5S_get_simple_extent_ndims, FAIL)
 
     /* check args */
-    assert(ds);
+    HDassert(ds);
 
-    switch (H5S_GET_EXTENT_TYPE(ds)) {
+    switch(H5S_GET_EXTENT_TYPE(ds)) {
         case H5S_NULL:
         case H5S_SCALAR:
         case H5S_SIMPLE:
-            ret_value = ds->extent.rank;
+            ret_value = (int)ds->extent.rank;
             break;
 
         default:
-            assert("unknown data space class" && 0);
-            HGOTO_ERROR(H5E_DATASPACE, H5E_UNSUPPORTED, FAIL, "internal error (unknown data space class)")
-    }
+            HDassert("unknown dataspace class" && 0);
+            HGOTO_ERROR(H5E_DATASPACE, H5E_UNSUPPORTED, FAIL, "internal error (unknown dataspace class)")
+    } /* end switch */
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
-}
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5S_get_simple_extent_ndims() */
 
 
 /*-------------------------------------------------------------------------
  * Function:	H5Sget_simple_extent_dims
  *
  * Purpose:	Returns the size and maximum sizes in each dimension of
- *		a data space DS through	the DIMS and MAXDIMS arguments.
+ *		a dataspace DS through	the DIMS and MAXDIMS arguments.
  *
  * Return:	Success:	Number of dimensions, the same value as
  *				returned by H5Sget_simple_extent_ndims().
@@ -986,8 +843,8 @@ done:
 /*-------------------------------------------------------------------------
  * Function:	H5S_get_simple_extent_dims
  *
- * Purpose:	Returns the size in each dimension of a data space.  This
- *		function may not be meaningful for all types of data spaces.
+ * Purpose:	Returns the size in each dimension of a dataspace.  This
+ *		function may not be meaningful for all types of dataspaces.
  *
  * Return:	Success:	Number of dimensions.  Zero implies scalar.
  *
@@ -1003,48 +860,48 @@ done:
 int
 H5S_get_simple_extent_dims(const H5S_t *ds, hsize_t dims[], hsize_t max_dims[])
 {
-    int	ret_value;
-    int	i;
+    int	i;              /* Local index variable */
+    int	ret_value;      /* Return value */
 
-    FUNC_ENTER_NOAPI(H5S_get_simple_extent_dims, FAIL);
+    FUNC_ENTER_NOAPI(H5S_get_simple_extent_dims, FAIL)
 
     /* check args */
-    assert(ds);
+    HDassert(ds);
 
-    switch (H5S_GET_EXTENT_TYPE(ds)) {
+    switch(H5S_GET_EXTENT_TYPE(ds)) {
         case H5S_NULL:
         case H5S_SCALAR:
             ret_value = 0;
             break;
 
         case H5S_SIMPLE:
-            ret_value = ds->extent.rank;
-            for (i=0; i<ret_value; i++) {
-                if (dims)
+            ret_value = (int)ds->extent.rank;
+            for(i = 0; i < ret_value; i++) {
+                if(dims)
                     dims[i] = ds->extent.size[i];
-                if (max_dims) {
-                    if (ds->extent.max)
+                if(max_dims) {
+                    if(ds->extent.max)
                         max_dims[i] = ds->extent.max[i];
                     else
                         max_dims[i] = ds->extent.size[i];
-                }
-            }
+                } /* end if */
+            } /* end for */
             break;
 
         default:
-            assert("unknown data space class" && 0);
-            HGOTO_ERROR(H5E_DATASPACE, H5E_UNSUPPORTED, FAIL, "internal error (unknown data space class)")
-    }
+            HDassert("unknown dataspace class" && 0);
+            HGOTO_ERROR(H5E_DATASPACE, H5E_UNSUPPORTED, FAIL, "internal error (unknown dataspace class)")
+    } /* end switch */
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
-}
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5S_get_simple_extent_dims() */
 
 
 /*-------------------------------------------------------------------------
  * Function:	H5S_write
  *
- * Purpose:	Updates a data space by writing a message to an object
+ * Purpose:	Updates a dataspace by writing a message to an object
  *		header.
  *
  * Return:	Non-negative on success/Negative on failure
@@ -1078,7 +935,7 @@ done:
 /*-------------------------------------------------------------------------
  * Function:	H5S_append
  *
- * Purpose:	Updates a data space by adding a message to an object
+ * Purpose:	Updates a dataspace by adding a message to an object
  *		header.
  *
  * Return:	Non-negative on success/Negative on failure
@@ -1119,9 +976,9 @@ done:
 /*-------------------------------------------------------------------------
  * Function:	H5S_read
  *
- * Purpose:	Reads the data space from an object header.
+ * Purpose:	Reads the dataspace from an object header.
  *
- * Return:	Success:	Pointer to a new data space.
+ * Return:	Success:	Pointer to a new dataspace.
  *
  *		Failure:	NULL
  *
@@ -1157,7 +1014,7 @@ H5S_read(const H5O_loc_t *loc, hid_t dxpl_id)
 done:
     if(ret_value == NULL) {
         if(ds != NULL)
-            H5FL_FREE(H5S_t, ds);
+            (void)H5FL_FREE(H5S_t, ds);
     } /* end if */
 
     FUNC_LEAVE_NOAPI(ret_value)
@@ -1221,7 +1078,7 @@ H5Sis_simple(hid_t space_id)
 
     /* Check args and all the boring stuff. */
     if ((space = (H5S_t *)H5I_object_verify(space_id,H5I_DATASPACE)) == NULL)
-	HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, FAIL, "not a data space")
+	HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, FAIL, "not a dataspace")
 
     ret_value = H5S_is_simple(space);
 
@@ -1275,7 +1132,7 @@ H5Sset_extent_simple(hid_t space_id, int rank, const hsize_t dims[/*rank*/],
 
     /* Check args */
     if ((space = (H5S_t *)H5I_object_verify(space_id,H5I_DATASPACE)) == NULL)
-        HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, FAIL, "not a data space")
+        HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, FAIL, "not a dataspace")
     if (rank > 0 && dims == NULL)
         HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "no dimensions specified")
     if (rank<0 || rank>H5S_MAX_RANK)
@@ -1384,110 +1241,11 @@ done:
     FUNC_LEAVE_NOAPI(ret_value)
 } /* H5S_set_extent_simple() */
 
-#ifdef H5S_DEBUG
-
-/*-------------------------------------------------------------------------
- * Function:	H5S_find
- *
- * Purpose:	Given two data spaces (MEM_SPACE and FILE_SPACE) this
- *		function returns a pointer to the conversion path information,
- *		creating a new conversion path entry if necessary.
- *
- * Return:	Success:	Ptr to a conversion path entry
- *
- *		Failure:	NULL
- *
- * Programmer:	Robb Matzke
- *		Wednesday, January 21, 1998
- *
- * Modifications:
- *
- * 	Quincey Koziol
- *	Instead of returning a point into the data space conversion table we
- *	copy all the information into a user-supplied CONV buffer and return
- *	non-negative on success or negative on failure.
- *
- * 	Robb Matzke, 11 Aug 1998
- *	Returns a pointer into the conversion path table.  A path entry
- *	contains pointers to the memory and file half of the conversion (the
- *	pointers registered in the H5S_fconv_g[] and H5S_mconv_g[] tables)
- *	along with other data whose scope is the conversion path (like path
- *	statistics).
- *
- *	John Mainzer, 8/30/04
- *	Modified code to check with all other processes that have the
- *	file open before OKing collective I/O.
- *
- *-------------------------------------------------------------------------
- */
-H5S_iostats_t *
-H5S_find (const H5S_t *mem_space, const H5S_t *file_space)
-{
-    H5S_iostats_t	*path=NULL;  /* Space conversion path */
-    size_t	u;      /* Index variable */
-    H5S_iostats_t *ret_value;   /* Return value */
-
-    FUNC_ENTER_NOAPI(H5S_find, NULL);
-
-    /* Check args */
-    assert (mem_space && (H5S_SIMPLE==H5S_GET_EXTENT_TYPE(mem_space) ||
-                          H5S_NULL==H5S_GET_EXTENT_TYPE(mem_space) ||
-			  H5S_SCALAR==H5S_GET_EXTENT_TYPE(mem_space)));
-    assert (file_space && (H5S_SIMPLE==H5S_GET_EXTENT_TYPE(file_space) ||
-                           H5S_NULL==H5S_GET_EXTENT_TYPE(file_space) ||
-			   H5S_SCALAR==H5S_GET_EXTENT_TYPE(file_space)));
-
-    /*
-     * Is this path already present in the data space conversion path table?
-     * If so then return a pointer to that entry.
-     */
-    for (u=0; u<H5S_niostats_g; u++)
-        if (H5S_iostats_g[u]->ftype==H5S_GET_SELECT_TYPE(file_space) &&
-                H5S_iostats_g[u]->mtype==H5S_GET_SELECT_TYPE(mem_space))
-            HGOTO_DONE(H5S_iostats_g[u]);
-
-    /*
-     * The path wasn't found.  Create a new path.
-     */
-    if (NULL==(path = H5MM_calloc(sizeof(*path))))
-        HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed for data space conversion path")
-
-    /* Initialize file & memory conversion functions */
-    path->ftype = H5S_GET_SELECT_TYPE(file_space);
-    path->mtype = H5S_GET_SELECT_TYPE(mem_space);
-
-    /*
-     * Add the new path to the table.
-     */
-    if (H5S_niostats_g>=H5S_aiostats_g) {
-        size_t n = MAX(10, 2*H5S_aiostats_g);
-        H5S_iostats_t **p = H5MM_realloc(H5S_iostats_g, n*sizeof(H5S_iostats_g[0]));
-
-        if (NULL==p)
-            HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed for data space conversion path table")
-        H5S_aiostats_g = n;
-        H5S_iostats_g = p;
-    } /* end if */
-    H5S_iostats_g[H5S_niostats_g++] = path;
-
-    /* Set the return value */
-    ret_value=path;
-
-done:
-    if(ret_value==NULL) {
-        if(path!=NULL)
-            H5MM_xfree(path);
-    } /* end if */
-
-    FUNC_LEAVE_NOAPI(ret_value);
-} /* end H5S_find() */
-#endif /* H5S_DEBUG */
-
 
 /*-------------------------------------------------------------------------
  * Function:	H5Screate_simple
  *
- * Purpose:	Creates a new simple data space object and opens it for
+ * Purpose:	Creates a new simple dataspace object and opens it for
  *		access. The DIMS argument is the size of the simple dataset
  *		and the MAXDIMS argument is the upper limit on the size of
  *		the dataset.  MAXDIMS may be the null pointer in which case
@@ -1496,7 +1254,7 @@ done:
  *		unlimited, otherwise no element of MAXDIMS should be smaller
  *		than the corresponding element of DIMS.
  *
- * Return:	Success:	The ID for the new simple data space object.
+ * Return:	Success:	The ID for the new simple dataspace object.
  *
  *		Failure:	Negative
  *
@@ -1550,7 +1308,7 @@ H5Screate_simple(int rank, const hsize_t dims[/*rank*/],
         HGOTO_ERROR(H5E_DATASPACE, H5E_CANTCREATE, FAIL, "can't create simple dataspace")
 
     /* Atomize */
-    if ((ret_value=H5I_register (H5I_DATASPACE, space))<0)
+    if ((ret_value=H5I_register (H5I_DATASPACE, space, TRUE))<0)
         HGOTO_ERROR(H5E_ATOM, H5E_CANTREGISTER, FAIL, "unable to register dataspace ID")
 
 done:
@@ -1568,7 +1326,7 @@ done:
  *
  * Purpose:	Internal function to create simple dataspace
  *
- * Return:	Success:	The ID for the new simple data space object.
+ * Return:	Success:	The ID for the new simple dataspace object.
  *		Failure:	Negative
  *
  * Errors:
@@ -1647,7 +1405,7 @@ done:
  * Function:	H5S_encode
  *
  * Purpose:	Private function for H5Sencode.  Converts an object
- *              description for data space and its selection into binary
+ *              description for dataspace and its selection into binary
  *              in a buffer.
  *
  * Return:	Success:	non-negative
@@ -1751,7 +1509,7 @@ H5Sdecode(const void *buf)
 	HGOTO_ERROR(H5E_DATASPACE, H5E_CANTDECODE, FAIL, "can't decode object")
 
     /* Register the type and return the ID */
-    if((ret_value = H5I_register(H5I_DATASPACE, ds)) < 0)
+    if((ret_value = H5I_register(H5I_DATASPACE, ds, TRUE)) < 0)
 	HGOTO_ERROR(H5E_DATASPACE, H5E_CANTREGISTER, FAIL, "unable to register dataspace")
 
 done:
@@ -1807,18 +1565,18 @@ H5S_decode(const unsigned char *buf)
 
     /* Decode the extent part of dataspace */
     /* (pass mostly bogus file pointer and bogus DXPL) */
-    if((extent = (H5S_extent_t *)H5O_msg_decode(f, H5P_DEFAULT, H5O_SDSPACE_ID, buf))==NULL)
+    if((extent = (H5S_extent_t *)H5O_msg_decode(f, H5P_DEFAULT, NULL, H5O_SDSPACE_ID, buf))==NULL)
 	HGOTO_ERROR(H5E_DATASPACE, H5E_CANTDECODE, NULL, "can't decode object")
     buf += extent_size;
 
     /* Copy the extent into dataspace structure */
     if((ds = H5FL_CALLOC(H5S_t))==NULL)
-        HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed for data space conversion path table")
+        HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed for dataspace conversion path table")
     if(H5O_msg_copy(H5O_SDSPACE_ID, extent, &(ds->extent)) == NULL)
 	HGOTO_ERROR(H5E_DATASPACE, H5E_CANTCOPY, NULL, "can't copy object")
     if(H5S_extent_release(extent) < 0)
         HGOTO_ERROR(H5E_RESOURCE, H5E_CANTDELETE, NULL, "can't release previous dataspace")
-    H5FL_FREE(H5S_extent_t, extent);
+    (void)H5FL_FREE(H5S_extent_t, extent);
 
     /* Initialize to "all" selection. Deserialization relies on valid existing selection. */
     if(H5S_select_all(ds, FALSE) < 0)
@@ -1941,7 +1699,7 @@ H5Sset_extent_none(hid_t space_id)
 
     /* Check args */
     if (NULL == (space = (H5S_t *)H5I_object_verify(space_id, H5I_DATASPACE)))
-        HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, FAIL, "not a data space")
+        HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, FAIL, "not a dataspace")
 
     /* Clear the previous extent from the dataspace */
     if(H5S_extent_release(&space->extent)<0)
@@ -1981,7 +1739,7 @@ H5Soffset_simple(hid_t space_id, const hssize_t *offset)
 
     /* Check args */
     if (NULL == (space = (H5S_t *)H5I_object_verify(space_id, H5I_DATASPACE)))
-        HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, FAIL, "not a data space")
+        HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, FAIL, "not a dataspace")
     if (space->extent.rank==0 || (H5S_GET_EXTENT_TYPE(space)==H5S_SCALAR
             || H5S_GET_EXTENT_TYPE(space)==H5S_NULL))
         HGOTO_ERROR(H5E_ATOM, H5E_UNSUPPORTED, FAIL, "can't set offset on scalar or null dataspace")
@@ -2000,7 +1758,7 @@ done:
 /*-------------------------------------------------------------------------
  * Function: H5S_set_extent
  *
- * Purpose: Modify the dimensions of a data space. Based on H5S_extend
+ * Purpose: Modify the dimensions of a dataspace. Based on H5S_extend
  *
  * Return: Success: Non-negative
  *
@@ -2061,9 +1819,10 @@ hbool_t
 H5S_has_extent(const H5S_t *ds)
 {
     htri_t ret_value;
+
     FUNC_ENTER_NOAPI(H5S_has_extent, FAIL)
 
-    assert(ds);
+    HDassert(ds);
 
     if(ds->extent.rank==0 && ds->extent.nelem == 0 && ds->extent.type != H5S_NULL)
         ret_value = FALSE;
@@ -2072,13 +1831,13 @@ H5S_has_extent(const H5S_t *ds)
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
-}
+} /* end H5S_has_extent() */
 
 
 /*-------------------------------------------------------------------------
  * Function: H5S_set_extent_real
  *
- * Purpose: Modify the dimensions of a data space. Based on H5S_extend
+ * Purpose: Modify the dimensions of a dataspace. Based on H5S_extend
  *
  * Return: Success: Non-negative
  *
@@ -2174,7 +1933,7 @@ done:
  DESCRIPTION
 	Compare two dataspaces if their extents are identical.
 --------------------------------------------------------------------------*/
-static htri_t
+htri_t
 H5S_extent_equal(const H5S_t *ds1, const H5S_t *ds2)
 {
     unsigned u;                 /* Local index variable */
@@ -2282,7 +2041,7 @@ done:
 /*-------------------------------------------------------------------------
  * Function:	H5S_extend
  *
- * Purpose:	Extend the dimensions of a data space.
+ * Purpose:	Extend the dimensions of a dataspace.
  *
  * Return:	Success:	Number of dimensions whose size increased.
  *
