@@ -18,34 +18,82 @@
  *      in the H5T interface.
  */
 
+/****************/
+/* Module Setup */
+/****************/
+
 #define H5T_PACKAGE		/*suppress error about including H5Tpkg	  */
 
 /* Interface initialization */
 #define H5_INTERFACE_INIT_FUNC	H5T_init_compound_interface
 
 
+/***********/
+/* Headers */
+/***********/
 #include "H5private.h"		/*generic functions			  */
 #include "H5Eprivate.h"		/*error handling			  */
 #include "H5Iprivate.h"		/*ID functions		   		  */
 #include "H5MMprivate.h"	/*memory management			  */
 #include "H5Tpkg.h"		/*data-type functions			  */
 
-/* Static local functions */
+/****************/
+/* Local Macros */
+/****************/
+
+
+/******************/
+/* Local Typedefs */
+/******************/
+/* "Key" (+ user data) for bsearch callback */
+typedef struct{
+    size_t              offset;     /* Offset of member to be added */
+    const H5T_cmemb_t   *max_under; /* Member with maximum offset seen that is not above "offset" */
+} H5T_insert_compar_t;
+
+
+/********************/
+/* Package Typedefs */
+/********************/
+
+
+/********************/
+/* Local Prototypes */
+/********************/
 static herr_t H5T_pack(const H5T_t *dt);
 
+
+/*********************/
+/* Public Variables */
+/*********************/
+
+
+/*********************/
+/* Package Variables */
+/*********************/
+
+
+/*****************************/
+/* Library Private Variables */
+/*****************************/
+
+
+/*******************/
+/* Local Variables */
+/*******************/
+
+
 
 /*--------------------------------------------------------------------------
 NAME
    H5T_init_compound_interface -- Initialize interface-specific information
 USAGE
     herr_t H5T_init_compound_interface()
-
 RETURNS
     Non-negative on success/Negative on failure
 DESCRIPTION
     Initializes any interface-specific data or routines.  (Just calls
-    H5T_init_iterface currently).
-
+    H5T_init() currently).
 --------------------------------------------------------------------------*/
 static herr_t
 H5T_init_compound_interface(void)
@@ -79,24 +127,24 @@ H5T_init_compound_interface(void)
 size_t
 H5Tget_member_offset(hid_t type_id, unsigned membno)
 {
-    H5T_t	*dt = NULL;
-    size_t	ret_value;
+    H5T_t	*dt;            /* Datatype to query */
+    size_t	ret_value;      /* Return value */
 
     FUNC_ENTER_API(H5Tget_member_offset, 0)
     H5TRACE2("z", "iIu", type_id, membno);
 
     /* Check args */
-    if (NULL == (dt = H5I_object_verify(type_id,H5I_DATATYPE)) || H5T_COMPOUND != dt->shared->type)
+    if(NULL == (dt = (H5T_t *)H5I_object_verify(type_id, H5I_DATATYPE)) || H5T_COMPOUND != dt->shared->type)
 	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, 0, "not a compound datatype")
-    if (membno >= dt->shared->u.compnd.nmembs)
+    if(membno >= dt->shared->u.compnd.nmembs)
 	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, 0, "invalid member number")
 
     /* Value */
-    ret_value = H5T_get_member_offset(dt, membno);
+    ret_value = H5T_GET_MEMBER_OFFSET(dt->shared, membno);
 
 done:
     FUNC_LEAVE_API(ret_value)
-}
+} /* end H5Tget_member_offset() */
 
 
 /*-------------------------------------------------------------------------
@@ -116,25 +164,18 @@ done:
  * Programmer:	Raymond Lu
  *		October 8, 2002
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 size_t
 H5T_get_member_offset(const H5T_t *dt, unsigned membno)
 {
-    size_t	ret_value;
-
     FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5T_get_member_offset)
 
-    assert(dt);
-    assert(membno < dt->shared->u.compnd.nmembs);
+    HDassert(dt);
+    HDassert(membno < dt->shared->u.compnd.nmembs);
 
-    /* Value */
-    ret_value = dt->shared->u.compnd.memb[membno].offset;
-
-    FUNC_LEAVE_NOAPI(ret_value)
-}
+    FUNC_LEAVE_NOAPI(dt->shared->u.compnd.memb[membno].offset)
+} /* end H5T_get_member_offset() */
 
 
 /*-------------------------------------------------------------------------
@@ -156,21 +197,22 @@ H5T_get_member_offset(const H5T_t *dt, unsigned membno)
 H5T_class_t
 H5Tget_member_class(hid_t type_id, unsigned membno)
 {
-    H5T_t	*dt = NULL;
-    H5T_class_t	ret_value;
+    H5T_t	*dt;            /* Datatype to query */
+    H5T_class_t	ret_value;      /* Return value */
 
     FUNC_ENTER_API(H5Tget_member_class, H5T_NO_CLASS)
     H5TRACE2("Tt", "iIu", type_id, membno);
 
     /* Check args */
-    if (NULL == (dt = H5I_object_verify(type_id,H5I_DATATYPE)) || H5T_COMPOUND != dt->shared->type)
+    if(NULL == (dt = (H5T_t *)H5I_object_verify(type_id, H5I_DATATYPE)) || H5T_COMPOUND != dt->shared->type)
         HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, H5T_NO_CLASS, "not a compound datatype")
-    if (membno >= dt->shared->u.compnd.nmembs)
+    if(membno >= dt->shared->u.compnd.nmembs)
         HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, H5T_NO_CLASS, "invalid member number")
 
     /* Get the type's class.  We have to use this function to get type class
-     * because of the concern of variable-length string. */
-    ret_value = H5T_get_class(dt->shared->u.compnd.memb[membno].type, FALSE);
+     *  because of the concern of variable-length string.
+     */
+    ret_value = H5T_GET_CLASS(dt->shared->u.compnd.memb[membno].type->shared, FALSE);
 
 done:
     FUNC_LEAVE_API(ret_value)
@@ -193,42 +235,35 @@ done:
  * Programmer:	Robb Matzke
  *		Wednesday, January  7, 1998
  *
- * Modifications:
- *
- * 	Robb Matzke, 4 Jun 1998
- *	If the member type is a named type then this function returns a
- *	handle to the re-opened named type.
- *
  *-------------------------------------------------------------------------
  */
 hid_t
 H5Tget_member_type(hid_t type_id, unsigned membno)
 {
-    H5T_t	*dt = NULL, *memb_dt = NULL;
-    hid_t	ret_value;
+    H5T_t	*dt;                    /* Datatype to query */
+    H5T_t	*memb_dt = NULL;        /* Member datatype */
+    hid_t	ret_value;              /* Return value */
 
     FUNC_ENTER_API(H5Tget_member_type, FAIL)
     H5TRACE2("i", "iIu", type_id, membno);
 
     /* Check args */
-    if (NULL == (dt = H5I_object_verify(type_id,H5I_DATATYPE)) || H5T_COMPOUND != dt->shared->type)
+    if(NULL == (dt = (H5T_t *)H5I_object_verify(type_id, H5I_DATATYPE)) || H5T_COMPOUND != dt->shared->type)
 	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a compound datatype")
-    if (membno >= dt->shared->u.compnd.nmembs)
+    if(membno >= dt->shared->u.compnd.nmembs)
 	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "invalid member number")
-    if ((memb_dt=H5T_get_member_type(dt, membno))==NULL)
+    if(NULL == (memb_dt = H5T_get_member_type(dt, membno, H5T_COPY_REOPEN)))
 	HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "unable to retrieve member type")
-    if ((ret_value = H5I_register(H5I_DATATYPE, memb_dt)) < 0)
+    if((ret_value = H5I_register(H5I_DATATYPE, memb_dt, TRUE)) < 0)
 	HGOTO_ERROR(H5E_DATATYPE, H5E_CANTREGISTER, FAIL, "unable register datatype atom")
 
 done:
-    if(ret_value<0) {
-        if(memb_dt!=NULL)
-            if(H5T_close(memb_dt)<0)
-	        HDONE_ERROR(H5E_DATATYPE, H5E_CANTCLOSEOBJ, FAIL, "can't close datatype")
-    } /* end if */
+    if(ret_value < 0)
+        if(memb_dt && H5T_close(memb_dt) < 0)
+            HDONE_ERROR(H5E_DATATYPE, H5E_CANTCLOSEOBJ, FAIL, "can't close datatype")
 
     FUNC_LEAVE_API(ret_value)
-}
+} /* end H5Tget_member_type() */
 
 
 /*-------------------------------------------------------------------------
@@ -246,27 +281,25 @@ done:
  * Programmer:	Raymond Lu
  *	        October 8, 2002
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 H5T_t *
-H5T_get_member_type(const H5T_t *dt, unsigned membno)
+H5T_get_member_type(const H5T_t *dt, unsigned membno, H5T_copy_t method)
 {
-    H5T_t	*ret_value = NULL;
+    H5T_t	*ret_value;     /* Return value */
 
     FUNC_ENTER_NOAPI(H5T_get_member_type, NULL)
 
-    assert(dt);
-    assert(membno < dt->shared->u.compnd.nmembs);
+    HDassert(dt);
+    HDassert(membno < dt->shared->u.compnd.nmembs);
 
     /* Copy datatype into an atom */
-    if (NULL == (ret_value = H5T_copy(dt->shared->u.compnd.memb[membno].type, H5T_COPY_REOPEN)))
+    if(NULL == (ret_value = H5T_copy(dt->shared->u.compnd.memb[membno].type, method)))
 	HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, NULL, "unable to copy member datatype")
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
-}
+} /* end H5T_get_member_type() */
 
 
 /*-------------------------------------------------------------------------
@@ -275,31 +308,24 @@ done:
  * Purpose:	Returns the size of the specified member.
  *
  * Return:	Success:	The size in bytes of the member's datatype.
- *
  *		Failure:        0
  *
  * Programmer:	Quincey Koziol
  *	        October 4, 2004
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 size_t
 H5T_get_member_size(const H5T_t *dt, unsigned membno)
 {
-    size_t	ret_value = 0;
-
     FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5T_get_member_size)
 
-    assert(dt);
-    assert(membno < dt->shared->u.compnd.nmembs);
+    HDassert(dt);
+    HDassert(membno < dt->shared->u.compnd.nmembs);
 
-    /* Value */
-    ret_value = dt->shared->u.compnd.memb[membno].type->shared->size;
 
-    FUNC_LEAVE_NOAPI(ret_value)
-}
+    FUNC_LEAVE_NOAPI(dt->shared->u.compnd.memb[membno].type->shared->size)
+} /* end H5T_get_member_size() */
 
 
 /*-------------------------------------------------------------------------
@@ -329,32 +355,32 @@ H5T_get_member_size(const H5T_t *dt, unsigned membno)
 herr_t
 H5Tinsert(hid_t parent_id, const char *name, size_t offset, hid_t member_id)
 {
-    H5T_t	*parent = NULL;		/*the compound parent datatype */
-    H5T_t	*member = NULL;		/*the atomic member type	*/
-    herr_t      ret_value=SUCCEED;       /* Return value */
+    H5T_t	*parent;		/* The compound parent datatype */
+    H5T_t	*member;		/* The member datatype	*/
+    herr_t      ret_value = SUCCEED;    /* Return value */
 
     FUNC_ENTER_API(H5Tinsert, FAIL)
     H5TRACE4("e", "i*szi", parent_id, name, offset, member_id);
 
     /* Check args */
-    if (parent_id==member_id)
+    if(parent_id == member_id)
 	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "can't insert compound datatype within itself")
-    if (NULL == (parent = H5I_object_verify(parent_id,H5I_DATATYPE)) || H5T_COMPOUND != parent->shared->type)
+    if(NULL == (parent = (H5T_t *)H5I_object_verify(parent_id, H5I_DATATYPE)) || H5T_COMPOUND != parent->shared->type)
 	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a compound datatype")
-    if (H5T_STATE_TRANSIENT!=parent->shared->state)
+    if(H5T_STATE_TRANSIENT != parent->shared->state)
 	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "parent type read-only")
-    if (!name || !*name)
+    if(!name || !*name)
 	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "no member name")
-    if (NULL == (member = H5I_object_verify(member_id,H5I_DATATYPE)))
+    if(NULL == (member = (H5T_t *)H5I_object_verify(member_id, H5I_DATATYPE)))
 	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a datatype")
 
     /* Insert */
-    if (H5T_insert(parent, name, offset, member) < 0)
+    if(H5T_insert(parent, name, offset, member) < 0)
         HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINSERT, FAIL, "unable to insert member")
 
 done:
     FUNC_LEAVE_API(ret_value)
-}
+} /* end H5Tinsert() */
 
 
 /*-------------------------------------------------------------------------
@@ -375,23 +401,65 @@ done:
 herr_t
 H5Tpack(hid_t type_id)
 {
-    H5T_t	*dt = NULL;
-    herr_t      ret_value=SUCCEED;       /* Return value */
+    H5T_t	*dt;                    /* Datatype to modify */
+    herr_t      ret_value = SUCCEED;    /* Return value */
 
     FUNC_ENTER_API(H5Tpack, FAIL)
     H5TRACE1("e", "i", type_id);
 
     /* Check args */
-    if (NULL == (dt = H5I_object_verify(type_id,H5I_DATATYPE)) || H5T_detect_class(dt,H5T_COMPOUND)<=0)
+    if(NULL == (dt = (H5T_t *)H5I_object_verify(type_id, H5I_DATATYPE)) || H5T_detect_class(dt, H5T_COMPOUND) <= 0)
 	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a compound datatype")
 
     /* Pack */
-    if (H5T_pack(dt) < 0)
+    if(H5T_pack(dt) < 0)
 	HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "unable to pack compound datatype")
 
 done:
     FUNC_LEAVE_API(ret_value)
-}
+} /* end H5Tpack() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:	H5T_insert_compar
+ *
+ * Purpose:	Callback function for bsearch called from H5T_insert.
+ *              Reports whether obj has a lower of higher offset than
+ *              that stored in key.  Also keeps track of the highest
+ *              offset seen that is not higher than that in key.
+ *
+ * Return:	-1 if key < obj
+ *              0 if key == obj
+ *              1 if key > obj
+ *
+ * Programmer:	Neil Fortner
+ *		Wednesday, January  7, 1998
+ *
+ * Modifications:
+ *
+ *-------------------------------------------------------------------------
+ */
+static int
+H5T_insert_compar(const void *_key, const void *_obj)
+{
+    H5T_insert_compar_t *key = *(H5T_insert_compar_t * const *)_key;    /* User data */
+    const H5T_cmemb_t   *memb = (const H5T_cmemb_t *)_obj;              /* Compound member being examined */
+    int                 ret_value;                                      /* Return value */
+
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5T_insert_compar)
+
+    if(key->offset > memb->offset) {
+        if(key->max_under == NULL || memb->offset > key->max_under->offset)
+            key->max_under = memb;
+        ret_value = 1;
+    } /* end if */
+    else if(key->offset < memb->offset)
+        ret_value = -1;
+    else
+        ret_value = 0;  /* Should not happen */
+
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5T_insert_compar() */
 
 
 /*-------------------------------------------------------------------------
@@ -407,17 +475,17 @@ done:
  * Programmer:	Robb Matzke
  *		Monday, December  8, 1997
  *
- * Modifications:
- *  Took out arrayness parameters - QAK, 10/6/00
- *
  *-------------------------------------------------------------------------
  */
 herr_t
 H5T_insert(H5T_t *parent, const char *name, size_t offset, const H5T_t *member)
 {
-    unsigned	idx, i;
+    unsigned	idx;                        /* Index of member to insert */
     size_t	total_size;
-    herr_t      ret_value = SUCCEED;       /* Return value */
+    H5T_insert_compar_t key;                /* Key for bsearch compare function */
+    H5T_insert_compar_t *keyptr = &key;     /* Pointer to key */
+    unsigned	i;                          /* Local index variable */
+    herr_t      ret_value = SUCCEED;        /* Return value */
 
     FUNC_ENTER_NOAPI(H5T_insert, FAIL)
 
@@ -432,63 +500,112 @@ H5T_insert(H5T_t *parent, const char *name, size_t offset, const H5T_t *member)
 	if(!HDstrcmp(parent->shared->u.compnd.memb[i].name, name))
 	    HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINSERT, FAIL, "member name is not unique")
 
-    /* Does the new member overlap any existing member ? */
     total_size = member->shared->size;
-    for(i = 0; i < parent->shared->u.compnd.nmembs; i++)
-	if((offset <= parent->shared->u.compnd.memb[i].offset &&
-                 offset + total_size > parent->shared->u.compnd.memb[i].offset) ||
-                (parent->shared->u.compnd.memb[i].offset <= offset &&
-                 parent->shared->u.compnd.memb[i].offset +
-                 parent->shared->u.compnd.memb[i].size > offset))
-	    HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINSERT, FAIL, "member overlaps with another member")
 
     /* Does the new member overlap the end of the compound type? */
     if((offset + total_size) > parent->shared->size)
         HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINSERT, FAIL, "member extends past end of compound type")
 
+    if(parent->shared->u.compnd.sorted != H5T_SORT_VALUE)
+        if(H5T_sort_value(parent, NULL) < 0)
+            HGOTO_ERROR(H5E_INTERNAL, H5E_CANTCOMPARE, FAIL, "value sort failed")
+
+    /* Find the position to insert the new member */
+    if(parent->shared->u.compnd.nmembs == 0)
+        idx = 0;
+    else {
+        /* Key value (including user data) for compar callback */
+        key.offset = offset;
+        key.max_under = NULL;
+
+        /* Do a binary search on the offsets of the (now sorted) members.  We do
+         * not expect to find an exact match (if we do it is an error), rely on
+         * the user data in the key to keep track of the closest member below
+         * the new member. */
+        if(NULL != HDbsearch(&keyptr, parent->shared->u.compnd.memb, parent->shared->u.compnd.nmembs,
+                sizeof(parent->shared->u.compnd.memb[0]), H5T_insert_compar))
+            HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINSERT, FAIL, "member overlaps with another member")
+        idx = (key.max_under == NULL) ? 0 : (unsigned) (key.max_under - parent->shared->u.compnd.memb + 1);
+    } /* end else */
+
+    /* Does the new member overlap any existing member ? */
+    if((idx < parent->shared->u.compnd.nmembs && (offset + total_size) > parent->shared->u.compnd.memb[idx].offset) ||
+            (idx && (parent->shared->u.compnd.memb[idx-1].offset + parent->shared->u.compnd.memb[idx-1].size) > offset))
+        HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINSERT, FAIL, "member overlaps with another member")
+
     /* Increase member array if necessary */
     if(parent->shared->u.compnd.nmembs >= parent->shared->u.compnd.nalloc) {
         unsigned na = MAX(1, parent->shared->u.compnd.nalloc * 2);
-        H5T_cmemb_t *x = H5MM_realloc(parent->shared->u.compnd.memb, na * sizeof(H5T_cmemb_t));
+        H5T_cmemb_t *x = (H5T_cmemb_t *)H5MM_realloc(parent->shared->u.compnd.memb, na * sizeof(H5T_cmemb_t));
 
         if(!x)
-            HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "memory allocation failed")
+            HGOTO_ERROR(H5E_DATATYPE, H5E_CANTALLOC, FAIL, "memory allocation failed")
         parent->shared->u.compnd.nalloc = na;
         parent->shared->u.compnd.memb = x;
     } /* end if */
 
-    /* Add member to end of member array */
-    idx = parent->shared->u.compnd.nmembs;
-    parent->shared->u.compnd.memb[idx].name = H5MM_xstrdup(name);
-    parent->shared->u.compnd.memb[idx].offset = offset;
-    parent->shared->u.compnd.memb[idx].size = total_size;
-    parent->shared->u.compnd.memb[idx].type = H5T_copy(member, H5T_COPY_ALL);
-
-    parent->shared->u.compnd.sorted = H5T_SORT_NONE;
-    parent->shared->u.compnd.nmembs++;
-
-    /* Determine if the compound datatype stayed packed */
+    /* Determine if the compound datatype stays packed */
     if(parent->shared->u.compnd.packed) {
         /* Check if the member type is packed */
-        if(H5T_is_packed(parent->shared->u.compnd.memb[idx].type) > 0) {
+        if(H5T_is_packed(member) > 0) {
             if(idx == 0) {
                 /* If the is the first member, the datatype is not packed
                  * if the first member isn't at offset 0
                  */
-                if(parent->shared->u.compnd.memb[idx].offset > 0)
+                if(offset > 0)
                     parent->shared->u.compnd.packed = FALSE;
             } /* end if */
             else {
                 /* If the is not the first member, the datatype is not
                  * packed if the new member isn't adjoining the previous member
                  */
-                if(parent->shared->u.compnd.memb[idx].offset != (parent->shared->u.compnd.memb[idx - 1].offset + parent->shared->u.compnd.memb[idx - 1].size))
+                if(offset != (parent->shared->u.compnd.memb[idx - 1].offset + parent->shared->u.compnd.memb[idx - 1].size))
                     parent->shared->u.compnd.packed = FALSE;
             } /* end else */
         } /* end if */
         else
             parent->shared->u.compnd.packed = FALSE;
     } /* end if */
+    else
+        /* Check if inserting this member causes the parent to become packed */
+        /* First check if it completely closes a gap */
+        /* No need to check if it's being appended to the end */
+        if(idx != parent->shared->u.compnd.nmembs
+                && (offset + total_size) == parent->shared->u.compnd.memb[idx].offset
+                && (idx == 0 ? offset == 0  : (parent->shared->u.compnd.memb[idx-1].offset
+                + parent->shared->u.compnd.memb[idx-1].size) == offset)
+                && H5T_is_packed(member) > 0) {
+
+            /* Start out packed */
+            parent->shared->u.compnd.packed = TRUE;
+
+            /* Check if the entire type is now packed */
+            if((idx != 0 && parent->shared->u.compnd.memb[0].offset != 0)
+                    || !H5T_is_packed(parent->shared->u.compnd.memb[0].type))
+                parent->shared->u.compnd.packed = FALSE;
+            else
+                for(i = 1; i < parent->shared->u.compnd.nmembs; i++)
+                    if((i != idx && parent->shared->u.compnd.memb[i].offset
+                            != (parent->shared->u.compnd.memb[i - 1].offset
+                            + parent->shared->u.compnd.memb[i - 1].size))
+                            || !H5T_is_packed(parent->shared->u.compnd.memb[i].type)) {
+                        parent->shared->u.compnd.packed = FALSE;
+                        break;
+                    } /* end if */
+        } /* end if */
+
+    /* Reshape the memb array to accomodate the new member */
+    if(idx != parent->shared->u.compnd.nmembs)
+        HDmemmove(&parent->shared->u.compnd.memb[idx+1], &parent->shared->u.compnd.memb[idx],
+                (parent->shared->u.compnd.nmembs - idx) * sizeof(parent->shared->u.compnd.memb[0]));
+
+    /* Add member to member array */
+    parent->shared->u.compnd.memb[idx].name = H5MM_xstrdup(name);
+    parent->shared->u.compnd.memb[idx].offset = offset;
+    parent->shared->u.compnd.memb[idx].size = total_size;
+    parent->shared->u.compnd.memb[idx].type = H5T_copy(member, H5T_COPY_ALL);
+
+    parent->shared->u.compnd.nmembs++;
 
     /* Set the "force conversion" flag if the field's datatype indicates */
     if(member->shared->force_conv == TRUE)
@@ -519,28 +636,24 @@ done:
  * Programmer:	Robb Matzke
  *		Wednesday, January  7, 1998
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 static herr_t
 H5T_pack(const H5T_t *dt)
 {
-    unsigned	i;
-    size_t	offset;
-    herr_t      ret_value=SUCCEED;       /* Return value */
+    herr_t      ret_value = SUCCEED;    /* Return value */
 
     FUNC_ENTER_NOAPI_NOINIT(H5T_pack)
 
-    assert(dt);
+    HDassert(dt);
 
-    if(H5T_detect_class(dt,H5T_COMPOUND)>0) {
+    if(H5T_detect_class(dt, H5T_COMPOUND) > 0) {
         /* If datatype has been packed, skip packing it and indicate success */
-        if(H5T_is_packed(dt)== TRUE)
+        if(TRUE == H5T_is_packed(dt))
             HGOTO_DONE(SUCCEED)
 
         /* Check for packing unmodifiable datatype */
-        if (H5T_STATE_TRANSIENT!=dt->shared->state)
+        if(H5T_STATE_TRANSIENT != dt->shared->state)
             HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "datatype is read-only")
 
         if(dt->shared->parent) {
@@ -548,21 +661,28 @@ H5T_pack(const H5T_t *dt)
                 HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "unable to pack parent of datatype")
 
             /* Adjust size of datatype appropriately */
-            if(dt->shared->type==H5T_ARRAY)
+            if(dt->shared->type == H5T_ARRAY)
                 dt->shared->size = dt->shared->parent->shared->size * dt->shared->u.array.nelem;
-            else if(dt->shared->type!=H5T_VLEN)
+            else if(dt->shared->type != H5T_VLEN)
                 dt->shared->size = dt->shared->parent->shared->size;
         } /* end if */
-        else if(dt->shared->type==H5T_COMPOUND) {
+        else if(dt->shared->type == H5T_COMPOUND) {
+            size_t	offset;                 /* Offset of member */
+            unsigned i;                     /* Local index variable */
+
             /* Recursively pack the members */
-            for (i=0; i<dt->shared->u.compnd.nmembs; i++)
-                if (H5T_pack(dt->shared->u.compnd.memb[i].type) < 0)
+            for(i = 0; i < dt->shared->u.compnd.nmembs; i++) {
+                if(H5T_pack(dt->shared->u.compnd.memb[i].type) < 0)
                     HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "unable to pack part of a compound datatype")
 
+                /* Update the member size */
+                dt->shared->u.compnd.memb[i].size = (dt->shared->u.compnd.memb[i].type)->shared->size;
+            } /* end for */
+
             /* Remove padding between members */
-            if(H5T_sort_value(dt, NULL)<0)
+            if(H5T_sort_value(dt, NULL) < 0)
 		HGOTO_ERROR(H5E_INTERNAL, H5E_CANTCOMPARE, FAIL, "value sort failed")
-            for (i=0, offset=0; i<dt->shared->u.compnd.nmembs; i++) {
+            for(i = 0, offset = 0; i < dt->shared->u.compnd.nmembs; i++) {
                 dt->shared->u.compnd.memb[i].offset = offset;
                 offset += dt->shared->u.compnd.memb[i].size;
             }
@@ -571,13 +691,13 @@ H5T_pack(const H5T_t *dt)
             dt->shared->size = MAX(1, offset);
 
             /* Mark the type as packed now */
-            dt->shared->u.compnd.packed=TRUE;
+            dt->shared->u.compnd.packed = TRUE;
         } /* end if */
     } /* end if */
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
-}
+} /* end H5T_pack() */
 
 
 /*-------------------------------------------------------------------------
@@ -598,19 +718,24 @@ done:
 htri_t
 H5T_is_packed(const H5T_t *dt)
 {
-    htri_t      ret_value=TRUE;       /* Return value */
+    htri_t      ret_value = TRUE;       /* Return value */
 
     FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5T_is_packed)
 
-    assert(dt);
+    HDassert(dt);
 
     /* Go up the chain as far as possible */
     while(dt->shared->parent)
-        dt=dt->shared->parent;
+        dt = dt->shared->parent;
 
     /* If this is a compound datatype, check if it is packed */
-    if(dt->shared->type==H5T_COMPOUND)
-        ret_value=(htri_t)dt->shared->u.compnd.packed;
+    if(dt->shared->type == H5T_COMPOUND) {
+        H5T_compnd_t *compnd = &(dt->shared->u.compnd); /* Convenience pointer to compound info */
+        ret_value = (htri_t)(compnd->packed && compnd->nmembs > 0
+                && compnd->memb[compnd->nmembs - 1].offset
+                + compnd->memb[compnd->nmembs - 1].size
+                == dt->shared->size);
+    } /* end if */
 
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5T_is_packed() */
