@@ -23,6 +23,7 @@
 
 #define H5T_PACKAGE
 #include "H5Tpkg.h"		/*to turn off hardware conversions*/
+#include "H5Iprivate.h"
 
 #include "h5test.h"
 
@@ -115,11 +116,12 @@ typedef struct {
     float f, g, h[16], i, j;
     double k, l, m, n;
     long o, p, q;
-    long_long r, s, t;
+    long long r, s, t;
 } stype4;
 
 #define NX	100u
 #define NY	2000u
+#define PACK_NMEMBS     100
 
 
 /*-------------------------------------------------------------------------
@@ -600,7 +602,7 @@ test_compound (char *filename, hid_t fapl)
     if ((s8_m_sid = H5Screate_simple (2, h_size, NULL)) < 0) goto error;
 
     /* Read the dataset */
-    s8 = calloc ((size_t)(h_size[0]*h_size[1]), sizeof(s1_t));
+    s8 = (s1_t *) calloc ((size_t)(h_size[0]*h_size[1]), sizeof(s1_t));
     assert (s8);
     if (H5Dread (dataset, s1_tid, s8_m_sid, s8_f_sid, H5P_DEFAULT, s8) < 0) {
 	goto error;
@@ -767,7 +769,7 @@ test_compound (char *filename, hid_t fapl)
     f_offset[1] = NY/3;
     h_size[0] = 2*NX/3 - f_offset[0];
     h_size[1] = 2*NY/3 - f_offset[1];
-    s11 = malloc ((size_t)h_size[0]*(size_t)h_size[1]*sizeof(s4_t));
+    s11 = (s4_t *) malloc ((size_t)h_size[0]*(size_t)h_size[1]*sizeof(s4_t));
     assert (s11);
 
     /* Initialize */
@@ -935,7 +937,7 @@ initialize_stype2(unsigned char *buf, const size_t num)
  *
  * Purpose:	Initialize data buffer.
  *
- * Return:	Success:	
+ * Return:	Success:
  *
  * Programmer:  Raymond Lu
  *              Friday, 15 June 2007
@@ -1019,7 +1021,7 @@ initialize_stype4(unsigned char *buf, const size_t num)
  *
  * Return:	Success:        datatype ID
  *
- *              Failure:        negative	
+ *              Failure:        negative
  *
  * Programmer:  Raymond Lu
  *              Friday, 15 June 2007
@@ -1053,7 +1055,7 @@ create_stype1(void)
             H5Tinsert(tid, "k", HOFFSET(stype1, k), H5T_NATIVE_DOUBLE) < 0 ||
             H5Tinsert(tid, "l", HOFFSET(stype1, l), H5T_NATIVE_DOUBLE) < 0 ||
             H5Tinsert(tid, "m", HOFFSET(stype1, m), H5T_NATIVE_DOUBLE) < 0 ||
-            H5Tinsert(tid, "n", HOFFSET(stype1, n), H5T_NATIVE_DOUBLE) < 0) 
+            H5Tinsert(tid, "n", HOFFSET(stype1, n), H5T_NATIVE_DOUBLE) < 0)
         goto error;
 
     if(H5Tclose(array_dt1) < 0)
@@ -1075,7 +1077,7 @@ error:
  *
  * Return:	Success:        datatype ID
  *
- *              Failure:        negative	
+ *              Failure:        negative
  *
  * Programmer:  Raymond Lu
  *              Friday, 15 June 2007
@@ -1134,7 +1136,7 @@ error:
  *
  * Return:	Success:        datatype ID
  *
- *              Failure:        negative	
+ *              Failure:        negative
  *
  * Programmer:  Raymond Lu
  *              Friday, 15 June 2007
@@ -1177,7 +1179,7 @@ error:
  *
  * Return:	Success:        datatype ID
  *
- *              Failure:        negative	
+ *              Failure:        negative
  *
  * Programmer:  Raymond Lu
  *              Friday, 15 June 2007
@@ -1239,7 +1241,7 @@ error:
  *
  * Return:	Success:        0
  *
- *              Failure:        negative	
+ *              Failure:        negative
  *
  * Programmer:  Raymond Lu
  *              Friday, 15 June 2007
@@ -1286,14 +1288,14 @@ compare_data(void *src_data, void *dst_data, hbool_t src_subset)
 	    printf("    i=%d\n", i);
 	    printf("    src={a=%d, b=%d, c=[%d,%d,%d,%d,%d,%d,%d,%d], d=%d, e=%d, f=%f, g=%f, h=[%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f], i=%f, j=%f, k=%f, l=%f, m=%f, n=%f}\n",
 		   s_ptr->a, s_ptr->b, s_ptr->c[0], s_ptr->c[1], s_ptr->c[2],
-		   s_ptr->c[3], s_ptr->c[4], s_ptr->c[5], s_ptr->c[6], s_ptr->c[7], 
+		   s_ptr->c[3], s_ptr->c[4], s_ptr->c[5], s_ptr->c[6], s_ptr->c[7],
                    s_ptr->d, s_ptr->e, s_ptr->f, s_ptr->g,s_ptr->h[0],s_ptr->h[1],s_ptr->h[2],
                    s_ptr->h[3],s_ptr->h[4],s_ptr->h[5],s_ptr->h[6],s_ptr->h[7],s_ptr->h[8],
                    s_ptr->h[9],s_ptr->h[10],s_ptr->h[11],s_ptr->h[12],s_ptr->h[13],s_ptr->h[14],
                    s_ptr->h[15], s_ptr->i,s_ptr->j,s_ptr->k,s_ptr->l,s_ptr->m,s_ptr->n);
 	    printf("    dst={a=%d, b=%d, c=[%d,%d,%d,%d,%d,%d,%d,%d], d=%d, e=%d, f=%f, g=%f, h=[%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f], i=%f, j=%f, k=%f, l=%f, m=%f, n=%f}\n",
 		   d_ptr->a, d_ptr->b, d_ptr->c[0], d_ptr->c[1], d_ptr->c[2],
-		   d_ptr->c[3], d_ptr->c[4], d_ptr->c[5], d_ptr->c[6], d_ptr->c[7], 
+		   d_ptr->c[3], d_ptr->c[4], d_ptr->c[5], d_ptr->c[6], d_ptr->c[7],
                    d_ptr->d, d_ptr->e, d_ptr->f, d_ptr->g,d_ptr->h[0],d_ptr->h[1],d_ptr->h[2],
                    d_ptr->h[3],d_ptr->h[4],d_ptr->h[5],d_ptr->h[6],d_ptr->h[7],d_ptr->h[8],
                    d_ptr->h[9],d_ptr->h[10],d_ptr->h[11],d_ptr->h[12],d_ptr->h[13],
@@ -1330,7 +1332,7 @@ error:
  *		Failure:	1
  *
  * Programmer:	Raymond Lu
- *              Friday, 15 June 2007 
+ *              Friday, 15 June 2007
  *
  * Modifications:
  *-------------------------------------------------------------------------
@@ -1338,7 +1340,7 @@ error:
 static int
 test_hdf5_src_subset(char *filename, hid_t fapl)
 {
-    hid_t   file;     
+    hid_t   file;
     hid_t   rew_tid, src_tid, dst_tid;
     hid_t   dataset;
     hid_t   space;
@@ -1415,38 +1417,38 @@ test_hdf5_src_subset(char *filename, hid_t fapl)
 
     /*
      *######################################################################
-     * STEP 2. Rewrite the data with a subset of original data type. 
+     * STEP 2. Rewrite the data with a subset of original data type.
      */
     TESTING("rewriting data with a subset of original data type");
 
     /* Create xfer properties to preserve initialized data */
     if((dxpl = H5Pcreate(H5P_DATASET_XFER)) < 0)
-       goto error;
+       FAIL_STACK_ERROR
 
     if(H5Pset_preserve(dxpl, TRUE) < 0)
-       goto error;
+       FAIL_STACK_ERROR
 
     /* Rewrite contiguous data set */
     if((dataset = H5Dopen2(file, DSET_NAME[0], H5P_DEFAULT)) < 0)
-        goto error;
+        FAIL_STACK_ERROR
 
     /* Write the data to the dataset */
     if(H5Dwrite(dataset, rew_tid, H5S_ALL, H5S_ALL, dxpl, rew_buf) < 0)
-	goto error;
+	FAIL_STACK_ERROR
 
     if(H5Dclose(dataset) < 0)
-        goto error;
+        FAIL_STACK_ERROR
 
     /* Rewrite chunked data set */
     if((dataset = H5Dopen2(file, DSET_NAME[1], H5P_DEFAULT)) < 0)
-        goto error;
+        FAIL_STACK_ERROR
 
     /* Write the data to the dataset */
     if(H5Dwrite(dataset, rew_tid, H5S_ALL, H5S_ALL, dxpl, rew_buf) < 0)
-	goto error;
+	FAIL_STACK_ERROR
 
     if(H5Dclose(dataset) < 0)
-        goto error;
+        FAIL_STACK_ERROR
 
     PASSED();
 
@@ -1458,48 +1460,48 @@ test_hdf5_src_subset(char *filename, hid_t fapl)
 
     /* Check contiguous data set */
     if((dataset = H5Dopen2(file, DSET_NAME[0], H5P_DEFAULT)) < 0)
-        goto error;
+        FAIL_STACK_ERROR
 
     if(H5Dread(dataset, dst_tid, H5S_ALL, H5S_ALL, dxpl, rbuf) < 0)
-        goto error;
+        FAIL_STACK_ERROR
 
-    if(compare_data(orig, rbuf, TRUE) < 0) 
-        goto error;
+    if(compare_data(orig, rbuf, TRUE) < 0)
+        TEST_ERROR
 
     if(H5Dclose(dataset) < 0)
-        goto error;
+        FAIL_STACK_ERROR
 
     /* Check chunked data set */
     if((dataset = H5Dopen2(file, DSET_NAME[1], H5P_DEFAULT)) < 0)
-        goto error;
+        FAIL_STACK_ERROR
 
     if(H5Dread(dataset, dst_tid, H5S_ALL, H5S_ALL, dxpl, rbuf) < 0)
-        goto error;
+        FAIL_STACK_ERROR
 
-    if(compare_data(orig, rbuf, TRUE) < 0) 
-        goto error;
+    if(compare_data(orig, rbuf, TRUE) < 0)
+        TEST_ERROR
 
     if(H5Dclose(dataset) < 0)
-        goto error;
+        FAIL_STACK_ERROR
 
     /* Finishing test and release resources */
     if(H5Sclose(space) < 0)
-        goto error;
+        FAIL_STACK_ERROR
 
     if(H5Pclose(dcpl) < 0)
-        goto error;
+        FAIL_STACK_ERROR
 
     if(H5Pclose(dxpl) < 0)
-        goto error;
+        FAIL_STACK_ERROR
 
     if(H5Tclose(src_tid) < 0)
-        goto error;
+        FAIL_STACK_ERROR
     if(H5Tclose(dst_tid) < 0)
-        goto error;
+        FAIL_STACK_ERROR
     if(H5Tclose(rew_tid) < 0)
-        goto error;
+        FAIL_STACK_ERROR
     if(H5Fclose(file) < 0)
-        goto error;
+        FAIL_STACK_ERROR
 
     free(orig);
     free(rbuf);
@@ -1527,15 +1529,15 @@ error:
  *                      TYPE4 D;               }
  *                      TYPE5 E;
  *                  };
- *              This optimization is for the Chicago company.  This test 
- *              is in opposite of test_hdf5_src_subset. 
+ *              This optimization is for the Chicago company.  This test
+ *              is in opposite of test_hdf5_src_subset.
  *
  * Return:	Success:	0
  *
  *		Failure:	1
  *
  * Programmer:	Raymond Lu
- *              Friday, 15 June 2007 
+ *              Friday, 15 June 2007
  *
  * Modifications:
  *-------------------------------------------------------------------------
@@ -1543,7 +1545,7 @@ error:
 static int
 test_hdf5_dst_subset(char *filename, hid_t fapl)
 {
-    hid_t   file;     
+    hid_t   file;
     hid_t   rew_tid, src_tid, dst_tid;
     hid_t   dataset;
     hid_t   space;
@@ -1619,7 +1621,7 @@ test_hdf5_dst_subset(char *filename, hid_t fapl)
 
     /*
      *######################################################################
-     * STEP 2. Rewrite the data with a subset of original data type. 
+     * STEP 2. Rewrite the data with a subset of original data type.
      */
     TESTING("rewriting data with a subset of original data type");
 
@@ -1667,7 +1669,7 @@ test_hdf5_dst_subset(char *filename, hid_t fapl)
     if(H5Dread(dataset, dst_tid, H5S_ALL, H5S_ALL, dxpl, rbuf) < 0)
         goto error;
 
-    if(compare_data(orig, rbuf, FALSE) < 0) 
+    if(compare_data(orig, rbuf, FALSE) < 0)
         goto error;
 
     if(H5Dclose(dataset) < 0)
@@ -1680,7 +1682,7 @@ test_hdf5_dst_subset(char *filename, hid_t fapl)
     if(H5Dread(dataset, dst_tid, H5S_ALL, H5S_ALL, dxpl, rbuf) < 0)
         goto error;
 
-    if(compare_data(orig, rbuf, FALSE) < 0) 
+    if(compare_data(orig, rbuf, FALSE) < 0)
         goto error;
 
     if(H5Dclose(dataset) < 0)
@@ -1717,16 +1719,250 @@ error:
     return 1;
 }
 
+/* Error macro that outputs the state of the randomly generated variables so the
+ * failure can be reproduced */
+#define PACK_OOO_ERROR                                                         \
+{                                                                              \
+    int _i;                                                                    \
+    H5_FAILED(); AT();                                                         \
+    printf("    Insertion order =");                                           \
+    for(_i=0; _i<PACK_NMEMBS; _i++)                                            \
+        printf(" %d", order[_i]);                                              \
+    printf("\n    Inner compound order = %d, location = %d\n", sub_cmpd_order, order[sub_cmpd_order]); \
+    fflush(stdout);                                                            \
+    goto error;                                                                \
+}
+
+
+/*-------------------------------------------------------------------------
+ * Function:	test_pack_ooo
+ *
+ * Purpose:	Test inserting fields into a compound out of offset order.
+ *              Verifies that the compound 
+ *
+ * Return:	Success:	0
+ *
+ *		Failure:	1
+ *
+ * Programmer:	Neil Fortner
+ *              Thursday, 22 January 2009
+ *
+ * Modifications:
+ *-------------------------------------------------------------------------
+ */
+static int
+test_pack_ooo(void)
+{
+    hid_t       cmpd, sub_cmpd;     /* Datatype IDs */
+    H5T_t       *dt;                /* Datatype pointer */
+    unsigned    order[PACK_NMEMBS]; /* Order of insertion */
+    unsigned    free_order[PACK_NMEMBS]; /* Index of remaining free slots in order */
+    unsigned    num_free;           /* Number of free slots in order */
+    unsigned    sub_cmpd_order;     /* Order to insert the inner compound */
+    char        name[6];            /* Member name */
+    unsigned    i, j;               /* Indices */
+
+    HDsrand((unsigned) time(NULL));
+
+    /* Initialize "free_order" array to indicate that all slots in order are
+     * free */
+    for(i=0; i<PACK_NMEMBS; i++)
+        free_order[i] = i;
+
+    /* Create "order" array */
+    for(i=0; i<PACK_NMEMBS; i++) {
+        /* Generate index into free_order array */
+        num_free = PACK_NMEMBS - i;
+        j = HDrand() % num_free;
+
+        /* Update order array at the randomly generated (but guaranteed to be
+         * free) location */
+        order[free_order[j]] = i;
+
+        /* Reshape free_order to remove j (which is no longer free) */
+        if(j < (num_free - 1))
+            HDmemmove(&free_order[j], &free_order[j+1], (num_free - j - 1) * sizeof(free_order[0]));
+    } /* end for */
+
+    /* Generate order to insert inner compound type */
+    sub_cmpd_order = HDrand() % PACK_NMEMBS;
+
+    TESTING("random member insertion with empty compound subtype");
+
+    /* Create inner compound type.  It will be empty for the first run */
+    if((sub_cmpd = H5Tcreate(H5T_COMPOUND, 4)) < 0) PACK_OOO_ERROR
+
+    /* Create main compound type, with extra space at the end */
+    if((cmpd = H5Tcreate(H5T_COMPOUND, (4 * PACK_NMEMBS) + 1)) < 0) PACK_OOO_ERROR
+
+    /* Insert the compound members in the random order previously generated */
+    for(i=0; i<PACK_NMEMBS; i++) {
+        sprintf(name, "%05d", i);
+        if(i == sub_cmpd_order) {
+            if(H5Tinsert(cmpd, name, 4 * order[i], sub_cmpd) < 0) PACK_OOO_ERROR
+        } else
+            if(H5Tinsert(cmpd, name, 4 * order[i], H5T_STD_I32BE) < 0) PACK_OOO_ERROR
+    } /* end for */
+
+    /* Verify that the compound is not packed */
+    if(NULL == (dt = (H5T_t *) H5I_object_verify(cmpd, H5I_DATATYPE))) PACK_OOO_ERROR
+    if(dt->shared->u.compnd.packed) PACK_OOO_ERROR
+
+    /* Close the main compound */
+    if(H5Tclose(cmpd) < 0) PACK_OOO_ERROR
+
+    PASSED();
+
+    TESTING("random member insertion with full compound subtype");
+
+    /* Complete the inner compound type */
+    if(H5Tinsert(sub_cmpd, "int", 0, H5T_STD_I32LE) < 0) PACK_OOO_ERROR
+
+    /* Recreate main compound type */
+    if((cmpd = H5Tcreate(H5T_COMPOUND, (4 * PACK_NMEMBS) + 1)) < 0) PACK_OOO_ERROR
+
+    /* Insert the compound members in the random order previously generated */
+    for(i=0; i<PACK_NMEMBS; i++) {
+        sprintf(name, "%05d", i);
+        if(i == sub_cmpd_order) {
+            if(H5Tinsert(cmpd, name, 4 * order[i], sub_cmpd) < 0) PACK_OOO_ERROR
+        } else
+            if(H5Tinsert(cmpd, name, 4 * order[i], H5T_STD_I32BE) < 0) PACK_OOO_ERROR
+    } /* end for */
+
+    /* Verify that the compound is packed */
+    if(NULL == (dt = (H5T_t *) H5I_object_verify(cmpd, H5I_DATATYPE))) PACK_OOO_ERROR
+    if(!dt->shared->u.compnd.packed) PACK_OOO_ERROR
+
+    /* Close */
+    if(H5Tclose(cmpd) < 0) PACK_OOO_ERROR
+    if(H5Tclose(sub_cmpd) < 0) PACK_OOO_ERROR
+
+    PASSED();
+
+    TESTING("reverse member insertion with empty compound subtype");
+
+    /* Create inner compound type.  It will be empty for the first run */
+    if((sub_cmpd = H5Tcreate(H5T_COMPOUND, 4)) < 0) PACK_OOO_ERROR
+
+    /* Create main compound type, with extra space at the end */
+    if((cmpd = H5Tcreate(H5T_COMPOUND, (4 * PACK_NMEMBS) + 1)) < 0) PACK_OOO_ERROR
+
+    /* Insert the compound members in reverse order, with compound last */
+    for(i=0; i<PACK_NMEMBS; i++) {
+        sprintf(name, "%05d", i);
+        if(i == PACK_NMEMBS - 1) {
+            if(H5Tinsert(cmpd, name, 4 * (PACK_NMEMBS - i - 1), sub_cmpd) < 0) PACK_OOO_ERROR
+        } else
+            if(H5Tinsert(cmpd, name, 4 * (PACK_NMEMBS - i - 1), H5T_STD_I32BE) < 0) PACK_OOO_ERROR
+    } /* end for */
+
+    /* Verify that the compound is not packed */
+    if(NULL == (dt = (H5T_t *) H5I_object_verify(cmpd, H5I_DATATYPE))) PACK_OOO_ERROR
+    if(dt->shared->u.compnd.packed) PACK_OOO_ERROR
+
+    /* Close the main compound */
+    if(H5Tclose(cmpd) < 0) PACK_OOO_ERROR
+
+    PASSED();
+
+    TESTING("reverse member insertion with full compound subtype");
+
+    /* Complete the inner compound type */
+    if(H5Tinsert(sub_cmpd, "int", 0, H5T_STD_I32LE) < 0) PACK_OOO_ERROR
+
+    /* Recreate main compound type */
+    if((cmpd = H5Tcreate(H5T_COMPOUND, (4 * PACK_NMEMBS) + 1)) < 0) PACK_OOO_ERROR
+
+    /* Insert the compound members in reverse order, with compound last */
+    for(i=0; i<PACK_NMEMBS; i++) {
+        sprintf(name, "%05d", i);
+        if(i == PACK_NMEMBS - 1) {
+            if(H5Tinsert(cmpd, name, 4 * (PACK_NMEMBS - i - 1), sub_cmpd) < 0) PACK_OOO_ERROR
+        } else
+            if(H5Tinsert(cmpd, name, 4 * (PACK_NMEMBS - i - 1), H5T_STD_I32BE) < 0) PACK_OOO_ERROR
+    } /* end for */
+
+    /* Verify that the compound is packed */
+    if(NULL == (dt = (H5T_t *) H5I_object_verify(cmpd, H5I_DATATYPE))) PACK_OOO_ERROR
+    if(!dt->shared->u.compnd.packed) PACK_OOO_ERROR
+
+    /* Close */
+    if(H5Tclose(cmpd) < 0) PACK_OOO_ERROR
+    if(H5Tclose(sub_cmpd) < 0) PACK_OOO_ERROR
+
+    PASSED();
+
+    TESTING("forward member insertion with empty compound subtype");
+
+    /* Create inner compound type.  It will be empty for the first run */
+    if((sub_cmpd = H5Tcreate(H5T_COMPOUND, 4)) < 0) PACK_OOO_ERROR
+
+    /* Create main compound type, with extra space at the end */
+    if((cmpd = H5Tcreate(H5T_COMPOUND, (4 * PACK_NMEMBS) + 1)) < 0) PACK_OOO_ERROR
+
+    /* Insert the compound members in forward order, with compound first */
+    for(i=0; i<PACK_NMEMBS; i++) {
+        sprintf(name, "%05d", i);
+        if(i == 0) {
+            if(H5Tinsert(cmpd, name, 4 * i, sub_cmpd) < 0) PACK_OOO_ERROR
+        } else
+            if(H5Tinsert(cmpd, name, 4 * i, H5T_STD_I32BE) < 0) PACK_OOO_ERROR
+    } /* end for */
+
+    /* Verify that the compound is not packed */
+    if(NULL == (dt = (H5T_t *) H5I_object_verify(cmpd, H5I_DATATYPE))) PACK_OOO_ERROR
+    if(dt->shared->u.compnd.packed) PACK_OOO_ERROR
+
+    /* Close the main compound */
+    if(H5Tclose(cmpd) < 0) PACK_OOO_ERROR
+
+    PASSED();
+
+    TESTING("forward member insertion with full compound subtype");
+
+    /* Complete the inner compound type */
+    if(H5Tinsert(sub_cmpd, "int", 0, H5T_STD_I32LE) < 0) PACK_OOO_ERROR
+
+    /* Recreate main compound type */
+    if((cmpd = H5Tcreate(H5T_COMPOUND, (4 * PACK_NMEMBS) + 1)) < 0) PACK_OOO_ERROR
+
+    /* Insert the compound members in forward order */
+    for(i=0; i<PACK_NMEMBS; i++) {
+        sprintf(name, "%05d", i);
+        if(i == 0) {
+            if(H5Tinsert(cmpd, name, 4 * i, sub_cmpd) < 0) PACK_OOO_ERROR
+        } else
+            if(H5Tinsert(cmpd, name, 4 * i, H5T_STD_I32BE) < 0) PACK_OOO_ERROR
+    } /* end for */
+
+    /* Verify that the compound is packed */
+    if(NULL == (dt = (H5T_t *) H5I_object_verify(cmpd, H5I_DATATYPE))) PACK_OOO_ERROR
+    if(!dt->shared->u.compnd.packed) PACK_OOO_ERROR
+
+    /* Close */
+    if(H5Tclose(cmpd) < 0) PACK_OOO_ERROR
+    if(H5Tclose(sub_cmpd) < 0) PACK_OOO_ERROR
+
+    PASSED();
+
+    return 0;
+
+error:
+    puts("*** DATASET TESTS FAILED ***");
+    return 1;
+}
+
 
 /*-------------------------------------------------------------------------
  * Function:	main
  *
- * Purpose:	Test different cases of I/O for compound data and the 
+ * Purpose:	Test different cases of I/O for compound data and the
  *              compound optimization for the Chicago company.
  *
  * Return:	Success:         0
  *
- *              Failure:         1	
+ *              Failure:         1
  *
  * Programmer:  Raymond Lu
  *              Friday, 15 June 2007
@@ -1768,6 +2004,9 @@ main (int argc, char *argv[])
     h5_fixname(FILENAME[2], fapl_id, fname, sizeof(fname));
     nerrors += test_hdf5_dst_subset(fname, fapl_id);
 
+    puts("Testing that compound types can be packed out of order:");
+    nerrors += test_pack_ooo();
+
     if (nerrors) {
         printf("***** %u FAILURE%s! *****\n",
                nerrors, 1==nerrors?"":"S");
