@@ -40,7 +40,7 @@
     if ((ret) == (val)) {						      \
 	TestErrPrintf("*** UNEXPECTED RETURN from %s is %ld at line %4d "     \
 		   "in %s\n", where, (long)(ret), (int)__LINE__, __FILE__);   \
-	H5Eprint (stdout);						      \
+	H5Eprint2(H5E_DEFAULT, stdout);				      \
     }									      \
 } while(0)
 
@@ -52,7 +52,7 @@
    if ((ret)<0) {							      \
       TestErrPrintf ("*** UNEXPECTED RETURN from %s is %ld line %4d in %s\n", \
                   (where), (long)(ret), (int)__LINE__, __FILE__);	      \
-      H5Eprint (stdout);						      \
+      H5Eprint2(H5E_DEFAULT, stdout);				      \
    }									      \
 }
 
@@ -64,20 +64,21 @@
    if (!(ret)) {							      \
       TestErrPrintf ("*** UNEXPECTED RETURN from %s is NULL line %4d in %s\n", \
                   (where), (int)__LINE__, __FILE__);			      \
-      H5Eprint (stdout);						      \
+      H5Eprint2(H5E_DEFAULT, stdout);				      \
    }									      \
 }
 
-/* Used to make certain a scalar return value _is_ a value */
-#define VERIFY(x, val, where) do {					      \
-    if (GetTestVerbosity()>=VERBO_HI) {					      \
+/* Used to make certain a return value _is_ a value */
+#define VERIFY(_x, _val, where) do {					      \
+    long __x = (long)_x, __val = (long)_val;				      \
+    if(GetTestVerbosity() >= VERBO_HI) {				      \
 	print_func("   Call to routine: %15s at line %4d in %s had value "    \
-		   "%ld \n", (where), (int)__LINE__, __FILE__, (long)(x));    \
+		   "%ld \n", (where), (int)__LINE__, __FILE__, __x);	      \
     }									      \
-    if ((x) != (val)) {							      \
+    if((__x) != (__val)) {						      \
 	TestErrPrintf("*** UNEXPECTED VALUE from %s should be %ld, but is %ld at line %4d " \
-		   "in %s\n", (where), (long)(val), (long)(x), (int)__LINE__, __FILE__); \
-	H5Eprint (stdout);						      \
+		   "in %s\n", (where), __val, __x, (int)__LINE__, __FILE__);  \
+	H5Eprint2(H5E_DEFAULT, stdout);					      \
     }									      \
 } while(0)
 
@@ -90,7 +91,7 @@
     if (HDstrcmp(x, val)) {					              \
 	TestErrPrintf("*** UNEXPECTED VALUE from %s should be %s, but is %s at line %4d " \
 		   "in %s\n", where, val, x, (int)__LINE__, __FILE__);        \
-	H5Eprint (stdout);					              \
+	H5Eprint2(H5E_DEFAULT, stdout);				      \
     }									      \
 } while(0)
 
@@ -101,11 +102,11 @@
 		   "%ld\n", func, (int)__LINE__, __FILE__, (long)(ret));      \
     }									      \
     if (GetTestVerbosity()>=VERBO_HI)					      \
-        H5Eprint(stdout);					              \
+        H5Eprint2(H5E_DEFAULT, stdout);				      \
     if ((ret) == FAIL) {						      \
 	TestErrPrintf("*** UNEXPECTED RETURN from %s is %ld at line %4d "     \
 		   "in %s\n", func, (long)(ret), (int)__LINE__, __FILE__);    \
-	H5Eprint (stdout);						      \
+	H5Eprint2(H5E_DEFAULT, stdout);				      \
     }									      \
 } while(0)
 
@@ -124,12 +125,15 @@ extern "C" {
 
 /* Prototypes for the test routines */
 void                    test_metadata(void);
+void                    test_checksum(void);
 void                    test_tst(void);
 void                    test_heap(void);
 void                    test_refstr(void);
 void                    test_file(void);
+void                    test_h5o(void);
 void                    test_h5t(void);
 void                    test_h5s(void);
+void                    test_coords(void);
 void                    test_h5d(void);
 void                    test_attr(void);
 void                    test_select(void);
@@ -142,12 +146,18 @@ void                    test_array(void);
 void                    test_genprop(void);
 void			test_configure(void);
 void			test_misc(void);
+void			test_ids(void);
 void			test_skiplist(void);
+void			test_sohm(void);
+void			test_unicode(void);
 
 /* Prototypes for the cleanup routines */
 void                    cleanup_metadata(void);
+void                    cleanup_checksum(void);
 void                    cleanup_file(void);
+void                    cleanup_h5o(void);
 void                    cleanup_h5s(void);
+void                    cleanup_coords(void);
 void                    cleanup_attr(void);
 void                    cleanup_select(void);
 void                    cleanup_time(void);
@@ -158,7 +168,9 @@ void                    cleanup_iterate(void);
 void                    cleanup_array(void);
 void                    cleanup_genprop(void);
 void			cleanup_configure(void);
+void			cleanup_sohm(void);
 void			cleanup_misc(void);
+void			cleanup_unicode(void);
 
 #ifdef __cplusplus
 }
