@@ -13,10 +13,12 @@
  * access to either file, you may request a copy from help@hdfgroup.org.     *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
-#include "h5diff.h"
 #include <stdlib.h>
 #include <assert.h>
+#include <memory.h>
+#include "h5diff.h"
 #include "h5diff_common.h"
+#include "h5tools_utils.h"
 
 /*-------------------------------------------------------------------------
  * Function: main
@@ -53,12 +55,12 @@
  *  "Some objects are not comparable"
  *
  * February 2007
- *  Added comparison for dataset regions. 
+ *  Added comparison for dataset regions.
  *  Added support for reading and comparing by hyperslabs for large files.
  *  Inclusion of a relative error formula to compare floating
- *   point numbers in order to deal with floating point uncertainty. 
+ *   point numbers in order to deal with floating point uncertainty.
  *  Printing of dataset dimensions along with dataset name
- *   
+ *
  *  November 19, 2007
  *    adopted the syntax h5diff  [OPTIONS]  file1 file2  [obj1[obj2]]
  *
@@ -69,28 +71,45 @@
 int main(int argc, const char *argv[])
 {
     int        ret;
-    char       *fname1 = NULL;
-    char       *fname2 = NULL;
-    char       *objname1  = NULL;
-    char       *objname2  = NULL;
+    const char *fname1 = NULL;
+    const char *fname2 = NULL;
+    const char *objname1  = NULL;
+    const char *objname2  = NULL;
     hsize_t    nfound=0;
     diff_opt_t options;
 
+    /*-------------------------------------------------------------------------
+    * process the command-line
+    *-------------------------------------------------------------------------
+    */
+
     parse_command_line(argc, argv, &fname1, &fname2, &objname1, &objname2, &options);
 
+    /*-------------------------------------------------------------------------
+    * do the diff
+    *-------------------------------------------------------------------------
+    */
+
     nfound = h5diff(fname1,fname2,objname1,objname2,&options);
 
     print_info(&options);
 
    /*-------------------------------------------------------------------------
     * exit code
-    *   1 if differences, 0 if no differences, -1 if error
+    *   1 if differences, 0 if no differences, 2 if error
     *-------------------------------------------------------------------------
     */
 
     ret = (nfound == 0 ? 0 : 1 );
-    if(options.err_stat)
-        ret = -1;
+
+    /* if graph difference return 1 for differences  */
+    if ( options.contents == 0 )
+        ret = 1;
+
+    /* and return 2 for error */
+    if (options.err_stat)
+        ret = 2;
+
     return ret;
 }
 
