@@ -46,6 +46,8 @@
 #define S_SIMPLE        "SIMPLE"
 #define S_NULL          "NULL"
 #define SOFTLINK        "SOFTLINK"
+#define EXTLINK         "EXTERNAL_LINK"
+#define UDLINK          "USERDEFINED_LINK"
 #define START           "START"
 #define STRIDE          "STRIDE"
 #define STRSIZE         "STRSIZE"
@@ -91,6 +93,10 @@ typedef struct h5dump_header_t {
     const char *dataend;
     const char *softlinkbegin;
     const char *softlinkend;
+    const char *extlinkbegin;
+    const char *extlinkend;
+    const char *udlinkbegin;
+    const char *udlinkend;
     const char *subsettingbegin;
     const char *subsettingend;
     const char *startbegin;
@@ -120,6 +126,10 @@ typedef struct h5dump_header_t {
     const char *datablockend;
     const char *softlinkblockbegin;
     const char *softlinkblockend;
+    const char *extlinkblockbegin;
+    const char *extlinkblockend;
+    const char *udlinkblockbegin;
+    const char *udlinkblockend;
     const char *strblockbegin;
     const char *strblockend;
     const char *enumblockbegin;
