@@ -45,15 +45,11 @@ int main(int argc, char *argv[])
    (void)HDsetvbuf(stderr, (char *) NULL, _IOLBF, 0);
    (void)HDsetvbuf(stdout, (char *) NULL, _IOLBF, 0);
 
-#if defined __MWERKS__
-    argc = ccommand(&argv);
-#endif
-
     if ( argv[1] && (strcmp("-V",argv[1])==0) )
     {
         print_version("h5import");
         exit(EXIT_SUCCESS);
-        
+
     }
 
  /*
@@ -275,7 +271,7 @@ gtoken(char *s)
  * Programmer:  pkmat
  *
  * Modifications: pvn
- *  7/23/2007. Added support for STR type
+ *  7/23/2007. Added support for STR type, extra parameter FILE_ID
  *
  *-------------------------------------------------------------------------
  */
@@ -293,11 +289,52 @@ processDataFile(char *infile, struct Input *in, FILE **strm, hid_t file_id)
   const char *err10 = "Unrecognized input class type.\n";
   const char *err11 = "Error in reading string data.\n";
 
-  if ((*strm = fopen(infile, "r")) == NULL)
+ /*-------------------------------------------------------------------------
+  * special case for opening binary classes in WIN32
+  * "FP" denotes a floating point binary file,
+  * "IN" denotes a signed integer binary file,
+  * "UIN" denotes an unsigned integer binary file,
+  *-------------------------------------------------------------------------
+  */
+  if ( in->inputClass == 4 /* "IN" */ ||
+       in->inputClass == 3 /* "FP" */ ||
+       in->inputClass == 7 /* "UIN" */
+
+      )
   {
-      (void) fprintf(stderr, err1, infile);
-      return(-1);
+
+#ifdef WIN32
+
+      if ((*strm = fopen(infile, "rb")) == NULL)
+      {
+          (void) fprintf(stderr, err1, infile);
+          return(-1);
+      }
+#else
+
+      if ((*strm = fopen(infile, "r")) == NULL)
+      {
+          (void) fprintf(stderr, err1, infile);
+          return(-1);
+      }
+
+#endif
+
   }
+ /*-------------------------------------------------------------------------
+  * if the input class is not binary, just use "r"
+  *-------------------------------------------------------------------------
+  */
+  else
+  {
+      if ((*strm = fopen(infile, "r")) == NULL)
+      {
+          (void) fprintf(stderr, err1, infile);
+          return(-1);
+      }
+  }
+
+
 
   switch(in->inputClass)
   {
@@ -372,7 +409,7 @@ readIntegerData(FILE **strm, struct Input *in)
   H5DT_INT8 *in08;
   H5DT_INT16 *in16, temp;
   H5DT_INT32 *in32;
-#ifndef _WIN32
+#ifndef WIN32
   H5DT_INT64 *in64;
   char buffer[256];
 #endif
@@ -727,8 +764,20 @@ readFloatData(FILE **strm, struct Input *in)
           fp32 = (H5DT_FLOAT32 *) in->data;
         break;
 
+        /* same as TEXTFP */
         case 2: /*TEXTFPE */
-        break;
+            
+            for (i = 0; i < len; i++, fp32++)
+            {
+                if (fscanf(*strm, "%f", fp32) != 1)
+                {
+                    (void) fprintf(stderr, err1);
+                    return (-1);
+                }
+            }
+            
+            fp32 = (H5DT_FLOAT32 *) in->data;
+            break;
 
         case 3: /* FP */
           for (i = 0; i < len; i++, fp32++)
@@ -764,8 +813,20 @@ readFloatData(FILE **strm, struct Input *in)
           fp64 = (H5DT_FLOAT64 *) in->data;
         break;
 
+        /* same as TEXTFP */
         case 2: /*TEXTFPE */
-        break;
+            
+            for (i = 0; i < len; i++, fp64++)
+            {
+                if (fscanf(*strm, "%lf", fp64) != 1)
+                {
+                    (void) fprintf(stderr, err1);
+                    return (-1);
+                }
+            }
+            
+            fp64 = (H5DT_FLOAT64 *) in->data;
+            break;
 
         case 3: /* FP */
           for (i = 0; i < len; i++, fp64++)
@@ -815,20 +876,20 @@ processStrData(FILE **strm, struct Input *in, hid_t file_id)
     char    str[1024];
     char    c;
     int     i = 0, j, nlines = 0, line;
-    
+
 /*-------------------------------------------------------------------------
  * get number of lines in the input file
  *-------------------------------------------------------------------------
  */
 
-    while ( !feof( *strm ) ) 
+    while ( !feof( *strm ) )
     {
         c = fgetc( *strm );
-                   
+
         if ( c == 10 ) /* eol */
         {
             nlines++;
-            
+
         }
     }
 
@@ -853,9 +914,9 @@ processStrData(FILE **strm, struct Input *in, hid_t file_id)
         goto out;
 
     /* disable error reporting */
-    H5E_BEGIN_TRY 
+    H5E_BEGIN_TRY
     {
-        
+
         /* create parent groups */
         if(in->path.count > 1) {
             j = 0;
@@ -876,7 +937,7 @@ processStrData(FILE **strm, struct Input *in, hid_t file_id)
             handle = file_id;
             j = 0;
         }
-        
+
         /*enable error reporting */
     } H5E_END_TRY;
 
@@ -893,11 +954,11 @@ processStrData(FILE **strm, struct Input *in, hid_t file_id)
 
     while(!feof(*strm)) {
         c = fgetc(*strm);
-        
+
         str[i] = c;
-        
+
         i++;
-        
+
         if(c == 10) /* eol */
         {
             char    *str2 = str;
@@ -923,11 +984,11 @@ processStrData(FILE **strm, struct Input *in, hid_t file_id)
 
             i = 0;
             str[ 0 ] = '\0';
-            
+
         }
     }
 
-    
+
     /* close */
     H5Dclose(dset_id);
     H5Sclose(space_id);
@@ -1376,7 +1437,7 @@ processConfigurationFile(char *infile, struct Input *in, FILE **strm)
         if (in->configOptionVector[COMPRESS] == 0)
           in->compressionType = 0;
 
-        in->configOptionVector[COMPRESS] = 1;
+       
       break;
 
       case 12: /* EXTERNAL-STORAGE */
@@ -1929,7 +1990,7 @@ getCompressionType(struct Input *in, FILE** strm)
     return (-1);
   }
 
-  in->outputByteOrder = kindex;
+  in->compressionType = kindex;
   return (0);
 
 }
@@ -2696,7 +2757,7 @@ help(char *name)
   (void) fprintf(stdout, "\t					  to be created.\n\n");
   (void) fprintf(stdout, "\t               INPUT-CLASS:\n");
   (void) fprintf(stdout, "\t			String denoting the type of input data.\n");
-  (void) fprintf(stdout, "\t			(\"TEXTIN\", \"TEXTFP\", \"TEXTFPE\", \"FP\", \"IN\", \n");
+  (void) fprintf(stdout, "\t			(\"TEXTIN\", \"TEXTFP\", \"FP\", \"IN\", \n");
   (void) fprintf(stdout, "\t			\"STR\", \"TEXTUIN\", \"UIN\"). \n");
   (void) fprintf(stdout, "\t			INPUT-CLASS \"TEXTIN\" denotes an ASCII text \n");
   (void) fprintf(stdout, "\t			file with signed integer data in ASCII form,\n");
@@ -2705,9 +2766,6 @@ help(char *name)
   (void) fprintf(stdout, "\t			\"TEXTFP\" denotes an ASCII text file containing\n");
   (void) fprintf(stdout, "\t			floating point data in the fixed notation\n");
   (void) fprintf(stdout, "\t			(325.34),\n");
-  (void) fprintf(stdout, "\t			\"TEXTFPE\" denotes an ASCII text file containing\n");
-  (void) fprintf(stdout, "\t			floating point data in the scientific notation\n");
-  (void) fprintf(stdout, "\t			(3.2534E+02),\n");
   (void) fprintf(stdout, "\t			\"FP\" denotes a floating point binary file,\n");
   (void) fprintf(stdout, "\t			\"IN\" denotes a signed integer binary file,\n");
   (void) fprintf(stdout, "\t			\"UIN\" denotes an unsigned integer binary file,\n");
