@@ -15,6 +15,7 @@
 
 #include "h5repack.h"
 #include "h5test.h"
+#include "h5tools.h"
 
 
 /*-------------------------------------------------------------------------
@@ -97,6 +98,11 @@ int aux_assign_obj(const char* name,            /* object name from traverse lis
                 for ( i=0; i<tmp.chunk.rank; i++)
                     tmp.chunk.chunk_lengths[i]=options->chunk_g.chunk_lengths[i];
                 break;
+            case H5D_LAYOUT_ERROR:
+            case H5D_COMPACT:
+            case H5D_CONTIGUOUS:
+            case H5D_NLAYOUTS:
+                break;
             default:
                 break;
             }/*switch*/
@@ -111,6 +117,11 @@ int aux_assign_obj(const char* name,            /* object name from traverse lis
                 for ( i=0; i<tmp.chunk.rank; i++)
                     tmp.chunk.chunk_lengths[i]=options->op_tbl->objs[idx].chunk.chunk_lengths[i];
                 break;
+            case H5D_LAYOUT_ERROR:
+            case H5D_COMPACT:
+            case H5D_CONTIGUOUS:
+            case H5D_NLAYOUTS:
+                break;
             default:
                 break;
             }/*switch*/
@@ -162,6 +173,11 @@ int aux_assign_obj(const char* name,            /* object name from traverse lis
                 for ( i=0; i<tmp.chunk.rank; i++)
                     tmp.chunk.chunk_lengths[i]=options->chunk_g.chunk_lengths[i];
                 break;
+            case H5D_LAYOUT_ERROR:
+            case H5D_COMPACT:
+            case H5D_CONTIGUOUS:
+            case H5D_NLAYOUTS:
+                break;
             default:
                 break;
             }/*switch*/
@@ -193,6 +209,7 @@ int aux_assign_obj(const char* name,            /* object name from traverse lis
 int apply_filters(const char* name,    /* object name from traverse list */
                   int rank,            /* rank of dataset */
                   hsize_t *dims,       /* dimensions of dataset */
+                  size_t msize,        /* size of type */
                   hid_t dcpl_id,       /* dataset creation property list */
                   pack_opt_t *options, /* repack options */
                   int *has_filter)     /* (OUT) object NAME has a filter */
@@ -250,13 +267,13 @@ int apply_filters(const char* name,    /* object name from traverse list */
         if ((layout = H5Pget_layout(dcpl_id))<0)
             return -1;
         
-        if (layout==H5D_CHUNKED)
+        if (layout == H5D_CHUNKED)
         {
             if ((rank = H5Pget_chunk(dcpl_id,NELMTS(chsize),chsize/*out*/))<0)
                 return -1;
-            obj.layout=H5D_CHUNKED;
-            obj.chunk.rank=rank;
-            for ( i=0; i<rank; i++)
+            obj.layout = H5D_CHUNKED;
+            obj.chunk.rank = rank;
+            for ( i = 0; i < rank; i++)
                 obj.chunk.chunk_lengths[i] = chsize[i];
         }
     }
@@ -283,9 +300,37 @@ int apply_filters(const char* name,    /* object name from traverse list */
     */
         if (obj.layout==-1)
         {
-            obj.chunk.rank=rank;
-            for (i=0; i<rank; i++)
-                obj.chunk.chunk_lengths[i] = dims[i];
+            
+            /* stripmine info */
+            hsize_t sm_size[H5S_MAX_RANK]; /*stripmine size */
+            hsize_t sm_nbytes;             /*bytes per stripmine */
+
+            obj.chunk.rank = rank;
+
+            /*
+            * determine the strip mine size. The strip mine is
+            * a hyperslab whose size is manageable.
+            */
+
+            
+            
+            sm_nbytes = msize;
+            for ( i = rank; i > 0; --i) 
+            {
+                hsize_t size = H5TOOLS_BUFSIZE / sm_nbytes;
+                if ( size == 0) /* datum size > H5TOOLS_BUFSIZE */
+                    size = 1;
+                sm_size[i - 1] = MIN(dims[i - 1], size);
+                sm_nbytes *= sm_size[i - 1];
+                assert(sm_nbytes > 0);
+
+            }
+
+            for ( i = 0; i < rank; i++)
+            {
+                obj.chunk.chunk_lengths[i] = sm_size[i];
+            }
+
         }
         
         for ( i=0; i<obj.nfilters; i++)
