@@ -22,33 +22,42 @@
 
 
 static void usage(const char *prog);
-static int  parse_number(char *str);
-static void parse_command_line(int argc, const char* argv[], pack_opt_t* options);
+static void parse_command_line(int argc, const char **argv, pack_opt_t* options);
 static void read_info(const char *filename,pack_opt_t *options);
 
 
 /* module-scoped variables */
-const char *progname = "h5repack";
-int d_status = EXIT_SUCCESS;
+const char  *progname = "h5repack";
+int         d_status = EXIT_SUCCESS;
+static int  has_i_o = 0;
+const char  *infile  = NULL;
+const char  *outfile = NULL;
+
 
 /*
  * Command-line options: The user can specify short or long-named
  * parameters.
  */
-static const char *s_opts = "hVvf:l:m:e:nLc:i:s:";
+static const char *s_opts = "hVvf:l:m:e:nLc:d:s:u:b:t:a:i:o:";
 static struct long_options l_opts[] = {
     { "help", no_arg, 'h' },
     { "version", no_arg, 'V' },
     { "verbose", no_arg, 'v' },
     { "filter", require_arg, 'f' },
     { "layout", require_arg, 'l' },
-    { "threshold", require_arg, 'm' },
+    { "minimum", require_arg, 'm' },
     { "file", require_arg, 'e' },
     { "native", no_arg, 'n' },
     { "latest", no_arg, 'L' },
     { "compact", require_arg, 'c' },
-    { "indexed", require_arg, 'i' },
+    { "indexed", require_arg, 'd' },
     { "ssize", require_arg, 's' },
+    { "ublock", require_arg, 'u' },
+    { "block", require_arg, 'b' },
+    { "threshold", require_arg, 't' },
+    { "alignment", require_arg, 'a' },
+    { "infile", require_arg, 'i' },   /* -i for backward compability */
+    { "outfile", require_arg, 'o' },  /* -o for backward compability */
     { NULL, 0, '\0' }
 };
 
@@ -68,9 +77,9 @@ static struct long_options l_opts[] = {
  *
  * Comments:
  *
- * Modifications: 
+ * Modifications:
  *  July 2004: Introduced the extra EC or NN option for SZIP
- *  October 2006: Added a new switch -n, that allows to write the dataset 
+ *  October 2006: Added a new switch -n, that allows to write the dataset
  *                using a native type. The default to write is the file type.
  *
  * Modification:
@@ -80,42 +89,52 @@ static struct long_options l_opts[] = {
  *    adopted the syntax h5repack [OPTIONS]  file1 file2
  *   PVN, November 28, 2007
  *    added support for multiple global filters
+ *   PVN, May 16, 2008
+ *    added  backward compatibility for -i infile -o outfile
+ *   PVN, August 20, 2008
+ *    add a user block to repacked file (switches -u -b)
+ *   PVN, August 28, 2008
+ *    add options to set alignment (H5Pset_alignment) (switches -t -a)
  *-------------------------------------------------------------------------
  */
-int main(int argc, char **argv)
+int main(int argc, const char **argv)
 {
-    char          *infile  = NULL;
-    char          *outfile = NULL;
+    
     pack_opt_t    options;            /*the global options */
-    int           ret;
+    int           ret=-1;
     
     /* initialize options  */
-    h5repack_init (&options,0);
-
+    h5repack_init (&options,0);        
+    
     parse_command_line(argc, argv, &options);
     
-    if ( argv[ opt_ind ] != NULL && argv[ opt_ind + 1 ] != NULL ) 
+    /* get file names if they were not yet got */
+    if ( has_i_o == 0 )
     {
-        infile = argv[ opt_ind ];
-        outfile = argv[ opt_ind + 1 ];
-
-        if ( strcmp( infile, outfile ) == 0 )
+        
+        if ( argv[ opt_ind ] != NULL && argv[ opt_ind + 1 ] != NULL )
         {
-            error_msg(progname, "file names cannot be the same\n");
+            infile = argv[ opt_ind ];
+            outfile = argv[ opt_ind + 1 ];
+            
+            if ( strcmp( infile, outfile ) == 0 )
+            {
+                error_msg(progname, "file names cannot be the same\n");
+                usage(progname);
+                exit(EXIT_FAILURE);
+                
+            }
+        }
+        
+        else
+        {
+            error_msg(progname, "file names missing\n");
             usage(progname);
             exit(EXIT_FAILURE);
-            
         }
     }
     
-    else
-    {
-        error_msg(progname, "file names missing\n");
-        usage(progname);
-        exit(EXIT_FAILURE);
-    }
-
-  
+    
     /* pack it */
     ret=h5repack(infile,outfile,&options);
     
@@ -149,19 +168,33 @@ static void usage(const char *prog)
  printf("   -v, --verbose           Verbose mode, print object information\n");
  printf("   -V, --version           Print version number and exit\n");
  printf("   -n, --native            Use a native HDF5 type when repacking\n");
+
  printf("   -L, --latest            Use latest version of file format\n");
  printf("   -c L1, --compact=L1     Maximum number of links in header messages\n");
- printf("   -i L2, --indexed=L2     Minimum number of links in the indexed format\n");
+ printf("   -d L2, --indexed=L2     Minimum number of links in the indexed format\n");
  printf("   -s S[:F], --ssize=S[:F] Shared object header message minimum size\n");
- printf("   -m T, --threshold=T     Do not apply the filter to datasets smaller than T\n");
- printf("   -e M, --file=M          Name of file M with the -f and -l options\n");
+
+ printf("   -m M, --minimum=M       Do not apply the filter to datasets smaller than M\n");
+ printf("   -e E, --file=E          Name of file E with the -f and -l options\n");
+
+ printf("   -u U, --ublock=U        Name of file U with user block data to be added\n");
+ printf("   -b B, --block=B         Size of user block to be added\n");
+ printf("   -t T, --threshold=T     Threshold value for H5Pset_alignment\n");
+ printf("   -a A, --alignment=A     Alignment value for H5Pset_alignment\n");
+
  printf("   -f FILT, --filter=FILT  Filter type\n");
  printf("   -l LAYT, --layout=LAYT  Layout type\n");
- 
+
  printf("\n");
 
- printf("  T - is an integer greater than 1, size of dataset in bytes \n");
- printf("  M - is a filename.\n");
+ printf("  M - is an integer greater than 1, size of dataset in bytes (default is 1024) \n");
+ printf("  E - is a filename.\n");
+ printf("  S - is an integer\n");
+ printf("  U - is a filename.\n");
+ printf("  T - is an integer\n");
+ printf("  A - is an integer greater than zero\n");
+ printf("  B - is the user block size, any value that is 512 or greater and is\n");
+ printf("        a power of 2 (1024 default)\n");
  printf("  F - is the shared object header message type, any of <dspace|dtype|fill|\n");
  printf("        pline|attr>. If F is not specified, S applies to all messages\n");
 
@@ -244,16 +277,30 @@ static void usage(const char *prog)
  *-------------------------------------------------------------------------
  */
 
-static void parse_command_line(int argc, const char* argv[], pack_opt_t* options)
+static 
+void parse_command_line(int argc, const char **argv, pack_opt_t* options)
 {
-    
+
     int opt;
-   
+
     /* parse command line options */
-    while ((opt = get_option(argc, argv, s_opts, l_opts)) != EOF) 
+    while ((opt = get_option(argc, argv, s_opts, l_opts)) != EOF)
     {
-        switch ((char)opt) 
+        switch ((char)opt)
         {
+
+        /* -i for backward compability */
+        case 'i':
+            infile = opt_arg;
+            has_i_o = 1;
+            break;
+        /* -o for backward compability */
+        case 'o':
+            outfile = opt_arg;
+            has_i_o = 1;
+            break;
+
+
         case 'h':
             usage(progname);
             exit(EXIT_SUCCESS);
@@ -264,7 +311,7 @@ static void parse_command_line(int argc, const char* argv[], pack_opt_t* options
             options->verbose = 1;
             break;
         case 'f':
-            
+
             /* parse the -f filter option */
             if (h5repack_addfilter( opt_arg, options)<0)
             {
@@ -273,7 +320,7 @@ static void parse_command_line(int argc, const char* argv[], pack_opt_t* options
             }
             break;
         case 'l':
-            
+
             /* parse the -l layout option */
             if (h5repack_addlayout( opt_arg, options)<0)
             {
@@ -281,18 +328,18 @@ static void parse_command_line(int argc, const char* argv[], pack_opt_t* options
                 exit(EXIT_FAILURE);
             }
             break;
-            
+
 
         case 'm':
 
-            options->threshold = parse_number( opt_arg );
-            if ((int)options->threshold==-1) 
+            options->min_comp = atoi( opt_arg );
+            if ((int)options->min_comp<=0)
             {
-                error_msg(progname, "invalid treshold size <%s>\n", opt_arg );
+                error_msg(progname, "invalid minimum compress size <%s>\n", opt_arg );
                 exit(EXIT_FAILURE);
             }
             break;
-        
+
         case 'e':
             read_info( opt_arg, options);
             break;
@@ -300,20 +347,20 @@ static void parse_command_line(int argc, const char* argv[], pack_opt_t* options
         case 'n':
             options->use_native = 1;
             break;
-        
+
         case 'L':
-            options->latest = 1; 
+            options->latest = 1;
             break;
-        
+
         case 'c':
 
             options->grp_compact = atoi( opt_arg );
             if (options->grp_compact>0)
                 options->latest = 1; /* must use latest format */
             break;
-        
-        
-        case 'i':
+
+
+        case 'd':
 
             options->grp_indexed = atoi( opt_arg );
             if (options->grp_indexed>0)
@@ -323,95 +370,87 @@ static void parse_command_line(int argc, const char* argv[], pack_opt_t* options
         case 's':
 
             {
-                
+
                 int idx = 0;
                 int ssize = 0;
                 char *msgPtr = strchr( opt_arg, ':');
                 options->latest = 1; /* must use latest format */
-                if (msgPtr == NULL) 
+                if (msgPtr == NULL)
                 {
                     ssize = atoi( opt_arg );
                     for (idx=0; idx<5; idx++)
-                        options->msg_size[idx] = ssize; 
+                        options->msg_size[idx] = ssize;
                 }
-                else 
+                else
                 {
                     char msgType[10];
                     strcpy(msgType, msgPtr+1);
                     msgPtr[0] = '\0';
                     ssize = atoi( opt_arg );
-                    if (strcmp(msgType, "dspace") == 0) {
+                    if (strncmp(msgType, "dspace",6) == 0) {
                         options->msg_size[0] = ssize;
                     }
-                    else if (strcmp(msgType, "dtype") == 0) {
+                    else if (strncmp(msgType, "dtype", 5) == 0) {
                         options->msg_size[1] = ssize;
                     }
-                    else if (strcmp(msgType, "fill") == 0) {
+                    else if (strncmp(msgType, "fill", 4) == 0) {
                         options->msg_size[2] = ssize;
                     }
-                    else if (strcmp(msgType, "pline") == 0) {
+                    else if (strncmp(msgType, "pline", 5) == 0) {
                         options->msg_size[3] = ssize;
                     }
-                    else if (strcmp(msgType, "attr") == 0) {
+                    else if (strncmp(msgType, "attr", 4) == 0) {
                         options->msg_size[4] = ssize;
                     }
-                }            
+                }
+            }
+
+            break;
+
+
+        case 'u':
+
+            options->ublock_filename = opt_arg;
+            break;
+
+        case 'b':
+
+            options->ublock_size = atol( opt_arg );
+            break;
+
+        case 't':
+
+            options->threshold = atol( opt_arg );
+          
+            break;
+
+        case 'a':
+
+            options->alignment = atol( opt_arg );
+            if ( options->alignment < 1 )
+            {
+                error_msg(progname, "invalid alignment size\n", opt_arg );
+                exit(EXIT_FAILURE);
             }
-            
             break;
 
         } /* switch */
-        
-        
+
+
     } /* while */
-    
-    /* check for file names to be processed */
-    if (argc <= opt_ind || argv[ opt_ind + 1 ] == NULL) 
+
+    if ( has_i_o == 0 )
     {
-        error_msg(progname, "missing file names\n");
-        usage(progname);
-        exit(EXIT_FAILURE);
-    }
-      
-
- 
- 
-}
-
-/*-------------------------------------------------------------------------
- * Function: parse_number
- *
- * Purpose: read a number from command line argument
- *
- * Return: number, -1 for FAIL
- *
- * Programmer: Pedro Vicente, pvn@ncsa.uiuc.edu
- *
- * Date: September, 23, 2003
- *
- *-------------------------------------------------------------------------
- */
-
-
-int parse_number(char *str)
-{
-    unsigned    i;
-    int         n;
-    char        c;
-    size_t      len=strlen(str);
-    
-    for ( i=0; i<len; i++)
-    {
-        c = str[i];
-        if (!isdigit(c)){
-            return -1;
+        /* check for file names to be processed */
+        if (argc <= opt_ind || argv[ opt_ind + 1 ] == NULL)
+        {
+            error_msg(progname, "missing file names\n");
+            usage(progname);
+            exit(EXIT_FAILURE);
         }
     }
-    str[i]='\0';
-    n=atoi(str);
-    return n;
-}
 
+}
 
 /*-------------------------------------------------------------------------
  * Function: read_info
@@ -431,7 +470,7 @@ static
 void read_info(const char *filename,
                pack_opt_t *options)
 {
-    
+
     char stype[10];
     char comp_info[1024];
     FILE *fp;
@@ -439,33 +478,33 @@ void read_info(const char *filename,
     int  i, rc=1;
     char  *srcdir = getenv("srcdir"); /* the source directory */
     char  data_file[512]="";          /* buffer to hold name of existing file */
-    
+
     /* compose the name of the file to open, using the srcdir, if appropriate */
     if (srcdir){
         strcpy(data_file,srcdir);
         strcat(data_file,"/");
     }
     strcat(data_file,filename);
-    
-    
+
+
     if ((fp = fopen(data_file, "r")) == (FILE *)NULL) {
         error_msg(progname, "cannot open options file %s\n", filename);
         exit(1);
     }
-    
+
     /* cycle until end of file reached */
     while( 1 )
     {
         rc=fscanf(fp, "%s", stype);
         if (rc==-1)
             break;
-        
+
        /*-------------------------------------------------------------------------
         * filter
         *-------------------------------------------------------------------------
         */
         if (strcmp(stype,"-f") == 0) {
-            
+
             /* find begining of info */
             i=0; c='0';
             while( c!=' ' )
@@ -484,7 +523,7 @@ void read_info(const char *filename,
                 if (c==10 /*eol*/) break;
             }
             comp_info[i-1]='\0'; /*cut the last " */
-            
+
             if (h5repack_addfilter(comp_info,options)==-1){
                 error_msg(progname, "could not add compression option\n");
                 exit(1);
@@ -495,7 +534,7 @@ void read_info(const char *filename,
         *-------------------------------------------------------------------------
         */
         else if (strcmp(stype,"-l") == 0) {
-            
+
             /* find begining of info */
             i=0; c='0';
             while( c!=' ' )
@@ -514,7 +553,7 @@ void read_info(const char *filename,
                 if (c==10 /*eol*/) break;
             }
             comp_info[i-1]='\0'; /*cut the last " */
-            
+
             if (h5repack_addlayout(comp_info,options)==-1){
                 error_msg(progname, "could not add chunck option\n");
                 exit(1);
@@ -529,7 +568,7 @@ void read_info(const char *filename,
             exit(1);
         }
     }
-    
+
     fclose(fp);
     return;
 }
