@@ -98,9 +98,7 @@ int               d_status = EXIT_SUCCESS;
 static int        display_all = TRUE;
 static int        display_file_metadata = FALSE;
 static int        display_file = FALSE;
-static int        display_group_metadata = FALSE;
 static int        display_group = FALSE;
-static int        display_dset_metadata = FALSE;
 static int        display_dset = FALSE;
 static int        display_dtype_metadata = FALSE;
 static int        display_object = FALSE;
@@ -309,14 +307,14 @@ attribute_stats(iter_t *iter, const H5O_info_t *oi)
  * Programmer: Quincey Koziol
  *             Tuesday, August 16, 2005
  *
- * Modifications: Refactored code from the walk_function 
- *                EIP, Wednesday, August 16, 2006 
+ * Modifications: Refactored code from the walk_function
+ *                EIP, Wednesday, August 16, 2006
  *
  *		  Vailin Choi 12 July 2007
  *		  1. Gathered storage info for btree and heap
  *		     (groups and attributes)
  *		  2. Gathered info for attributes
- *		  
+ *
  *		  Vailin Choi 14 July 2007
  *		  Cast "num_objs" and "num_attrs" to size_t
  *		  Due to the -Mbounds problem for the pgi-32 bit compiler on indexing
@@ -389,8 +387,8 @@ group_stats(iter_t *iter, const char *name, const H5O_info_t *oi)
  * Programmer:    Quincey Koziol
  *                Tuesday, August 16, 2005
  *
- * Modifications: Refactored code from the walk_function 
- *                EIP, Wednesday, August 16, 2006 
+ * Modifications: Refactored code from the walk_function
+ *                EIP, Wednesday, August 16, 2006
  *
  *                Vailin Choi 12 July 2007
  *                1. Gathered storage info for btree and heap
@@ -698,7 +696,6 @@ parse_command_line(int argc, const char *argv[])
 
             case 'G':
                 display_all = FALSE;
-                display_group_metadata = TRUE;
                 break;
 
             case 'g':
@@ -713,7 +710,6 @@ parse_command_line(int argc, const char *argv[])
 
             case 'D':
                 display_all = FALSE;
-                display_dset_metadata = TRUE;
                 break;
 
             case 'd':
@@ -813,7 +809,7 @@ print_file_info(const iter_t *iter)
 
     return 0;
 }
-        
+
 
 /*-------------------------------------------------------------------------
  * Function: print_file_metadata
@@ -841,9 +837,9 @@ static herr_t
 print_file_metadata(const iter_t *iter)
 {
     printf("Object header size: (total/unused)\n");
-    HDfprintf(stdout, "\tGroups: %Hu/%Hu\n", iter->group_ohdr_info.total_size, 
+    HDfprintf(stdout, "\tGroups: %Hu/%Hu\n", iter->group_ohdr_info.total_size,
                                              iter->group_ohdr_info.free_size);
-    HDfprintf(stdout, "\tDatasets: %Hu/%Hu\n", iter->dset_ohdr_info.total_size, 
+    HDfprintf(stdout, "\tDatasets: %Hu/%Hu\n", iter->dset_ohdr_info.total_size,
                                                iter->dset_ohdr_info.free_size);
 
     printf("Storage information:\n");
@@ -882,6 +878,9 @@ print_file_metadata(const iter_t *iter)
  *             Saturday, August 12, 2006
  *
  * Modifications:
+ *	bug #1253; Oct 6th 2008; Vailin Choi
+ *	Fixed segmentation fault: print iter->group_bins[0] when
+ *	there is iter->group_nbins
  *
  *-------------------------------------------------------------------------
  */
@@ -904,14 +903,14 @@ print_group_info(const iter_t *iter)
 
     printf("Group bins:\n");
     total = 0;
-    if(iter->group_bins[0] > 0) {
+    if((iter->group_nbins > 0) && (iter->group_bins[0] > 0)) {
        printf("\t# of groups of size 0: %lu\n", iter->group_bins[0]);
        total = iter->group_bins[0];
     } /* end if */
     power = 1;
     for(u = 1; u < iter->group_nbins; u++) {
         if(iter->group_bins[u] > 0) {
-           printf("\t# of groups of size %lu - %lu: %lu\n", power, (power * 10) - 1, 
+           printf("\t# of groups of size %lu - %lu: %lu\n", power, (power * 10) - 1,
                     iter->group_bins[u]);
            total += iter->group_bins[u];
         } /* end if */
@@ -961,7 +960,7 @@ print_attr_info(const iter_t *iter)
     power = 1;
     for(u = 1; u < iter->attr_nbins; u++) {
         if(iter->attr_bins[u] > 0) {
-           printf("\t# of objects with %lu - %lu attributes: %lu\n", power, (power * 10) - 1, 
+           printf("\t# of objects with %lu - %lu attributes: %lu\n", power, (power * 10) - 1,
                     iter->attr_bins[u]);
            total += iter->attr_bins[u];
         } /* end if */
@@ -1012,7 +1011,7 @@ print_dataset_info(const iter_t *iter)
         total = 0;
         for(u = 0; u < SIZE_SMALL_DSETS; u++) {
             if(iter->small_dset_dims[u] > 0) {
-                printf("\t\t# of dataset dimensions of size %u: %lu\n", u, 
+                printf("\t\t# of dataset dimensions of size %u: %lu\n", u,
                          iter->small_dset_dims[u]);
                 total += iter->small_dset_dims[u];
             } /* end if */
@@ -1030,7 +1029,7 @@ print_dataset_info(const iter_t *iter)
             power = 1;
             for(u = 1; u < iter->dset_dim_nbins; u++) {
                 if(iter->dset_dim_bins[u] > 0) {
-                    printf("\t\t# of datasets of size %lu - %lu: %lu\n", power, (power * 10) - 1, 
+                    printf("\t\t# of datasets of size %lu - %lu: %lu\n", power, (power * 10) - 1,
                              iter->dset_dim_bins[u]);
                     total += iter->dset_dim_bins[u];
                 } /* end if */
@@ -1067,7 +1066,7 @@ print_dataset_info(const iter_t *iter)
                 H5Tencode(iter->dset_type_info[u].tid, NULL, &dtype_size);
                 printf("\tDataset datatype #%u:\n", u);
                 printf("\t\tCount (total/named) = (%lu/%lu)\n", iter->dset_type_info[u].count, iter->dset_type_info[u].named);
-                printf("\t\tSize (desc./elmt) = (%lu/%lu)\n", (unsigned long)dtype_size, 
+                printf("\t\tSize (desc./elmt) = (%lu/%lu)\n", (unsigned long)dtype_size,
                         (unsigned long)H5Tget_size(iter->dset_type_info[u].tid));
                 H5Tclose(iter->dset_type_info[u].tid);
                 total += iter->dset_type_info[u].count;
@@ -1103,7 +1102,6 @@ print_file_statistics(const iter_t *iter)
         display_file = TRUE;
         display_file_metadata = TRUE;
         display_group = TRUE;
-        display_group_metadata = TRUE;
         display_dset = TRUE;
         display_dtype_metadata = TRUE;
         display_attr = TRUE;
@@ -1143,7 +1141,7 @@ print_object_statistics(const char *name)
 /*-------------------------------------------------------------------------
  * Function: print_statistics
  *
- * Purpose: Prints statistics 
+ * Purpose: Prints statistics
  *
  * Return: Success: 0
  *
@@ -1159,12 +1157,21 @@ print_object_statistics(const char *name)
 static void
 print_statistics(const char *name, const iter_t *iter)
 {
-    if(display_object) 
+    if(display_object)
         print_object_statistics(name);
     else
         print_file_statistics(iter);
 }
 
+/*-------------------------------------------------------------------------
+ * 
+ * Modifications:
+ *	bug #1253; Oct 6th 2008; Vailin Choi
+ *	Call print_statistics() when objects/links are successfully traversed.
+ *	Otherwise, return warning message.
+ *
+ *-------------------------------------------------------------------------
+ */
 
 int
 main(int argc, const char *argv[])
@@ -1175,7 +1182,6 @@ main(int argc, const char *argv[])
     struct handler_t   *hand;
     H5F_info_t      	finfo;
 
-
     /* Disable error reporting */
     H5Eset_auto2(H5E_DEFAULT, NULL, NULL);
 
@@ -1199,7 +1205,7 @@ main(int argc, const char *argv[])
 
     /* Initialize iter structure */
     iter_init(&iter, fid);
-    
+
     /* Get storge info for SOHM's btree/list/heap and superblock extension */
     if(H5Fget_info(fid, &finfo) < 0)
 	warn_msg(progname, "Unable to retrieve SOHM info\n");
@@ -1216,17 +1222,21 @@ main(int argc, const char *argv[])
 
         u = 0;
         while(hand[u].obj) {
-            h5trav_visit(fid, hand[u].obj, TRUE, TRUE, obj_stats, lnk_stats, &iter);
-            print_statistics(hand[u].obj, &iter);
+            if (h5trav_visit(fid, hand[u].obj, TRUE, TRUE, obj_stats, lnk_stats, &iter) < 0) 
+		warn_msg(progname, "Unable to traverse object \"%s\"\n", hand[u].obj);
+	    else
+		print_statistics(hand[u].obj, &iter);
             u++;
         } /* end while */
     } /* end if */
     else {
-        h5trav_visit(fid, "/", TRUE, TRUE, obj_stats, lnk_stats, &iter);
-        print_statistics("/", &iter);
+        if (h5trav_visit(fid, "/", TRUE, TRUE, obj_stats, lnk_stats, &iter) < 0)
+	    warn_msg(progname, "Unable to traverse objects/links in file \"%s\"\n", fname);
+	else
+	    print_statistics("/", &iter);
     } /* end else */
 
-    free(hand);
+    if (hand) free(hand);
 
     if(H5Fclose(fid) < 0) {
         error_msg(progname, "unable to close file \"%s\"\n", fname);
