@@ -53,7 +53,7 @@ print_objname (diff_opt_t * options, hsize_t nfound)
 /*-------------------------------------------------------------------------
  * Function: do_print_objname
  *
- * Purpose: print object name 
+ * Purpose: print object name
  *
  *-------------------------------------------------------------------------
  */
@@ -186,16 +186,18 @@ hsize_t h5diff(const char *fname1,
                const char *objname2,
                diff_opt_t *options)
 {
-    trav_info_t  *info1;
-    trav_info_t  *info2;
-    hid_t        file1_id = (-1), file2_id = (-1);
+    trav_info_t  *info1=NULL;
+    trav_info_t  *info2=NULL;
+    hid_t        file1_id = (-1);
+    hid_t        file2_id = (-1);
     char         filenames[2][1024];
     hsize_t      nfound = 0;
 
     HDmemset(filenames, 0, 1024 * 2);
 
-    if(options->m_quiet && (options->m_verbose || options->m_report)) {
-        printf("Error: -q (quiet mode) cannot be added to verbose or report modes\n");
+    if(options->m_quiet && (options->m_verbose || options->m_report))
+    {
+        parallel_print("Error: -q (quiet mode) cannot be added to verbose or report modes\n");
         options->err_stat=1;
         return 0;
     } /* end if */
@@ -206,10 +208,12 @@ hsize_t h5diff(const char *fname1,
     */
 
     /* disable error reporting */
-    H5E_BEGIN_TRY {
-        /* Open the files */
-        if((file1_id = H5Fopen(fname1, H5F_ACC_RDONLY, H5P_DEFAULT)) < 0) {
-            printf("h5diff: <%s>: unable to open file\n", fname1);
+    H5E_BEGIN_TRY
+    {
+        /* open the files */
+        if((file1_id = H5Fopen(fname1, H5F_ACC_RDONLY, H5P_DEFAULT)) < 0)
+        {
+            parallel_print("h5diff: <%s>: unable to open file\n", fname1);
             options->err_stat = 1;
 
 #ifdef H5_HAVE_PARALLEL
@@ -219,8 +223,9 @@ hsize_t h5diff(const char *fname1,
 #endif
             goto out;
         } /* end if */
-        if((file2_id = H5Fopen(fname2, H5F_ACC_RDONLY, H5P_DEFAULT)) < 0) {
-            printf("h5diff: <%s>: unable to open file\n", fname2);
+        if((file2_id = H5Fopen(fname2, H5F_ACC_RDONLY, H5P_DEFAULT)) < 0)
+        {
+            parallel_print("h5diff: <%s>: unable to open file\n", fname2);
             options->err_stat = 1;
 
 #ifdef H5_HAVE_PARALLEL
@@ -245,7 +250,7 @@ hsize_t h5diff(const char *fname1,
     *-------------------------------------------------------------------------
     */
     if(h5trav_getinfo(file1_id, info1) < 0 || h5trav_getinfo(file2_id, info2) < 0) {
-        printf("Error: Could not get file contents\n");
+        parallel_print("Error: Could not get file contents\n");
         options->err_stat = 1;
 #ifdef H5_HAVE_PARALLEL
         if(g_Parallel)
@@ -255,11 +260,12 @@ hsize_t h5diff(const char *fname1,
         goto out;
     } /* end if */
 
-    /*-------------------------------------------------------------------------
+   /*-------------------------------------------------------------------------
     * object name was supplied
     *-------------------------------------------------------------------------
     */
-    if(objname1) {
+    if( objname1 )
+    {
 #ifdef H5_HAVE_PARALLEL
         if(g_Parallel)
             /* Let tasks know that they won't be needed */
@@ -267,22 +273,31 @@ hsize_t h5diff(const char *fname1,
 #endif
         assert(objname2);
         options->cmn_objs = 1; /* eliminate warning */
-        nfound = diff_compare(file1_id, fname1, objname1, info1,
-                              file2_id, fname2, objname2, info2,
+        nfound = diff_compare(file1_id,
+                              fname1,
+                              objname1,
+                              info1,
+                              file2_id,
+                              fname2,
+                              objname2,
+                              info2,
                               options);
     } /* end if */
 
-    /*-------------------------------------------------------------------------
+   /*-------------------------------------------------------------------------
     * compare all
     *-------------------------------------------------------------------------
     */
 
-    else {
+    else
+    {
 #ifdef H5_HAVE_PARALLEL
-        if(g_Parallel) {
+        if(g_Parallel)
+        {
             int i;
 
-            if((HDstrlen(fname1) > 1024) || (HDstrlen(fname2) > 1024)) {
+            if((HDstrlen(fname1) > 1024) || (HDstrlen(fname2) > 1024))
+            {
                 fprintf(stderr, "The parallel diff only supports path names up to 1024 characters\n");
                 MPI_Abort(MPI_COMM_WORLD, 0);
             } /* end if */
@@ -295,7 +310,12 @@ hsize_t h5diff(const char *fname1,
                 MPI_Send(filenames, (1024 * 2), MPI_CHAR, i, MPI_TAG_PARALLEL, MPI_COMM_WORLD);
         } /* end if */
 #endif
-        nfound = diff_match(file1_id, info1, file2_id, info2, options);
+
+        nfound = diff_match(file1_id,
+                            info1,
+                            file2_id,
+                            info2,
+                            options);
     } /* end else */
 
     trav_info_free(info1);
@@ -303,7 +323,8 @@ hsize_t h5diff(const char *fname1,
 
 out:
     /* close */
-    H5E_BEGIN_TRY {
+    H5E_BEGIN_TRY
+    {
         H5Fclose(file1_id);
         H5Fclose(file2_id);
     } H5E_END_TRY;
@@ -332,26 +353,28 @@ out:
  *-------------------------------------------------------------------------
  */
 hsize_t diff_match(hid_t file1_id,
-                    trav_info_t * info1,
-                    hid_t file2_id,
-                    trav_info_t * info2,
-                    diff_opt_t * options)
+                   trav_info_t *info1,
+                   hid_t file2_id,
+                   trav_info_t *info2,
+                   diff_opt_t *options)
 {
     trav_table_t *table = NULL;
-    size_t curr1, curr2;
-    unsigned infile[2];
-    hsize_t nfound = 0;
-    unsigned i;
+    size_t       curr1;
+    size_t       curr2;
+    unsigned     infile[2];
+    hsize_t      nfound = 0;
+    unsigned     i;
 
     /*-------------------------------------------------------------------------
     * build the list
     *-------------------------------------------------------------------------
     */
-    trav_table_init(&table);
+    trav_table_init( &table );
 
     curr1 = 0;
     curr2 = 0;
-    while(curr1 < info1->nused && curr2 < info2->nused) {
+    while(curr1 < info1->nused && curr2 < info2->nused)
+    {
         /* criteria is string compare */
         int cmp = HDstrcmp(info1->paths[curr1].path, info2->paths[curr2].path);
 
@@ -363,13 +386,15 @@ hsize_t diff_match(hid_t file1_id,
             curr1++;
             curr2++;
         } /* end if */
-        else if(cmp < 0) {
+        else if(cmp < 0)
+        {
             infile[0] = 1;
             infile[1] = 0;
             trav_table_addflags(infile, info1->paths[curr1].path, info1->paths[curr1].type, table);
             curr1++;
         } /* end else-if */
-        else {
+        else
+        {
             infile[0] = 0;
             infile[1] = 1;
             trav_table_addflags(infile, info2->paths[curr2].path, info2->paths[curr2].type, table);
@@ -380,7 +405,8 @@ hsize_t diff_match(hid_t file1_id,
     /* list1 did not end */
     infile[0] = 1;
     infile[1] = 0;
-    while(curr1 < info1->nused) {
+    while(curr1 < info1->nused)
+    {
         trav_table_addflags(infile, info1->paths[curr1].path, info1->paths[curr1].type, table);
         curr1++;
     } /* end while */
@@ -388,30 +414,70 @@ hsize_t diff_match(hid_t file1_id,
     /* list2 did not end */
     infile[0] = 0;
     infile[1] = 1;
-    while(curr2 < info2->nused) {
+    while(curr2 < info2->nused)
+    {
         trav_table_addflags(infile, info2->paths[curr2].path, info2->paths[curr2].type, table);
         curr2++;
     } /* end while */
 
-    /*-------------------------------------------------------------------------
+   /*-------------------------------------------------------------------------
     * print the list
     *-------------------------------------------------------------------------
     */
-    if(options->m_verbose) {
-        printf("\n");
-        printf("file1     file2\n");
-        printf("---------------------------------------\n");
+    if(options->m_verbose)
+    {
+        parallel_print("\n");
+        parallel_print("file1     file2\n");
+        parallel_print("---------------------------------------\n");
         for(i = 0; i < table->nobjs; i++) {
             char c1, c2;
 
             c1 = (table->objs[i].flags[0]) ? 'x' : ' ';
             c2 = (table->objs[i].flags[1]) ? 'x' : ' ';
-            printf("%5c %6c    %-15s\n", c1, c2, table->objs[i].name);
+            parallel_print("%5c %6c    %-15s\n", c1, c2, table->objs[i].name);
         } /* end for */
-        printf ("\n");
+        parallel_print ("\n");
     } /* end if */
 
 
+
+    /*-------------------------------------------------------------------------
+    * regarding the return value of h5diff (0, no difference in files, 1 difference )
+    * 1) the number of objects in file1 must be the same as in file2
+    * 2) the graph must match, i.e same names (absolute path)
+    * 3) objects with the same name must be of the same type
+    *-------------------------------------------------------------------------
+    */     
+       
+    /* number of different objects */
+    if ( info1->nused != info2->nused )
+    {
+        options->contents = 0;
+    }
+    
+    /* objects in one file and not the other */
+    for( i = 0; i < table->nobjs; i++)
+    {
+        if( table->objs[i].flags[0] != table->objs[i].flags[1] )
+        {
+            options->contents = 0;
+        }
+    }
+
+    /* objects with the same name but different HDF5 types */
+    for( i = 0; i < table->nobjs; i++) 
+    {
+        if ( table->objs[i].flags[0] && table->objs[i].flags[1] )
+        {
+            if ( table->objs[i].type != table->objs[i].type )
+            {
+                options->contents = 0;
+            }
+        }
+    }
+
+
+
     /*-------------------------------------------------------------------------
     * do the diff for common objects
     *-------------------------------------------------------------------------
@@ -430,17 +496,21 @@ hsize_t diff_match(hid_t file1_id,
     HDmemset(workerTasks, 1, (g_nTasks - 1));
 #endif
 
-    for(i = 0; i < table->nobjs; i++) {
-        if(table->objs[i].flags[0] && table->objs[i].flags[1]) {
+    for(i = 0; i < table->nobjs; i++)
+    {
+        if( table->objs[i].flags[0] && table->objs[i].flags[1])
+        {
             options->cmn_objs = 1;
-            if(!g_Parallel) {
+            if(!g_Parallel)
+            {
                 nfound += diff(file1_id,
                         table->objs[i].name,
                         file2_id,
                         table->objs[i].name, options, table->objs[i].type);
             } /* end if */
 #ifdef H5_HAVE_PARALLEL
-            else {
+            else
+            {
                 int workerFound = 0;
 
                 h5diffdebug("beginning of big else block\n");
@@ -453,7 +523,8 @@ hsize_t diff_match(hid_t file1_id,
                 */
 
                 /*Set up args to pass to worker task. */
-                if(HDstrlen(table->objs[i].name) > 255) {
+                if(HDstrlen(table->objs[i].name) > 255)
+                {
                     printf("The parallel diff only supports object names up to 255 characters\n");
                     MPI_Abort(MPI_COMM_WORLD, 0);
                 } /* end if */
@@ -464,14 +535,16 @@ hsize_t diff_match(hid_t file1_id,
 
                 h5diffdebug2("busyTasks=%d\n", busyTasks);
                 /* if there are any outstanding print requests, let's handle one. */
-                if(busyTasks > 0) {
+                if(busyTasks > 0)
+                {
                     int incomingMessage;
 
                     /* check if any tasks freed up, and didn't need to print. */
                     MPI_Iprobe(MPI_ANY_SOURCE, MPI_TAG_DONE, MPI_COMM_WORLD, &incomingMessage, &Status);
 
                     /* first block*/
-                    if(incomingMessage) {
+                    if(incomingMessage)
+                    {
                         workerTasks[Status.MPI_SOURCE - 1] = 1;
                         MPI_Recv(&nFoundbyWorker, sizeof(nFoundbyWorker), MPI_BYTE, Status.MPI_SOURCE, MPI_TAG_DONE, MPI_COMM_WORLD, &Status);
                         nfound += nFoundbyWorker.nfound;
@@ -480,7 +553,8 @@ hsize_t diff_match(hid_t file1_id,
                     } /* end if */
 
                     /* check to see if the print token was returned. */
-                    if(!havePrintToken) {
+                    if(!havePrintToken)
+                    {
                         /* If we don't have the token, someone is probably sending us output */
                         print_incoming_data();
 
@@ -499,10 +573,12 @@ hsize_t diff_match(hid_t file1_id,
                     } /* end if */
 
                     /* check to see if anyone needs the print token. */
-                    if(havePrintToken) {
+                    if(havePrintToken)
+                    {
                         /* check incoming queue for print token requests */
                         MPI_Iprobe(MPI_ANY_SOURCE, MPI_TAG_TOK_REQUEST, MPI_COMM_WORLD, &incomingMessage, &Status);
-                        if(incomingMessage) {
+                        if(incomingMessage)
+                        {
                             MPI_Recv(NULL, 0, MPI_BYTE, Status.MPI_SOURCE, MPI_TAG_TOK_REQUEST, MPI_COMM_WORLD, &Status);
                             MPI_Send(NULL, 0, MPI_BYTE, Status.MPI_SOURCE, MPI_TAG_PRINT_TOK, MPI_COMM_WORLD);
                             havePrintToken = 0;
@@ -513,8 +589,10 @@ hsize_t diff_match(hid_t file1_id,
                 /* check array of tasks to see which ones are free.
                 * Manager task never does work, so freeTasks[0] is really
                 * worker task 0. */
-                for(n = 1; (n < g_nTasks) && !workerFound; n++) {
-                    if(workerTasks[n-1]) {
+                for(n = 1; (n < g_nTasks) && !workerFound; n++)
+                {
+                    if(workerTasks[n-1])
+                    {
                         /* send file id's and names to first free worker */
                         MPI_Send(&args, sizeof(args), MPI_BYTE, n, MPI_TAG_ARGS, MPI_COMM_WORLD);
 
@@ -528,20 +606,24 @@ hsize_t diff_match(hid_t file1_id,
                 } /* end for */
 
                 h5diffdebug2("workerfound is %d \n", workerFound);
-                if(!workerFound) {
+                if(!workerFound)
+                {
                     /* if they were all busy, we've got to wait for one free up
                      *  before we can move on.  If we don't have the token, some
                      * task is currently printing so we'll wait for that task to
                      * return it.
                      */
 
-                    if(!havePrintToken) {
-                        while(!havePrintToken) {
+                    if(!havePrintToken)
+                    {
+                        while(!havePrintToken)
+                        {
                             int incomingMessage;
 
                             print_incoming_data();
                             MPI_Iprobe(MPI_ANY_SOURCE, MPI_TAG_TOK_RETURN, MPI_COMM_WORLD, &incomingMessage, &Status);
-                            if(incomingMessage) {
+                            if(incomingMessage)
+                            {
                                 MPI_Recv(&nFoundbyWorker, sizeof(nFoundbyWorker), MPI_BYTE, MPI_ANY_SOURCE, MPI_TAG_TOK_RETURN, MPI_COMM_WORLD, &Status);
                                 havePrintToken = 1;
                                 nfound += nFoundbyWorker.nfound;
@@ -552,23 +634,27 @@ hsize_t diff_match(hid_t file1_id,
                         } /* end while */
                     } /* end if */
                     /* if we do have the token, check for task to free up, or wait for a task to request it */
-                    else {
+                    else
+                    {
                         /* But first print all the data in our incoming queue */
                         print_incoming_data();
                         MPI_Probe(MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, &Status);
-                        if(Status.MPI_TAG == MPI_TAG_DONE) {
+                        if(Status.MPI_TAG == MPI_TAG_DONE)
+                        {
                             MPI_Recv(&nFoundbyWorker, sizeof(nFoundbyWorker), MPI_BYTE, Status.MPI_SOURCE, MPI_TAG_DONE, MPI_COMM_WORLD, &Status);
                             nfound += nFoundbyWorker.nfound;
                             options->not_cmp = options->not_cmp | nFoundbyWorker.not_cmp;
                             MPI_Send(&args, sizeof(args), MPI_BYTE, Status.MPI_SOURCE, MPI_TAG_ARGS, MPI_COMM_WORLD);
                         } /* end if */
-                        else if(Status.MPI_TAG == MPI_TAG_TOK_REQUEST) {
+                        else if(Status.MPI_TAG == MPI_TAG_TOK_REQUEST)
+                        {
                             int incomingMessage;
 
                             MPI_Recv(NULL, 0, MPI_BYTE, Status.MPI_SOURCE, MPI_TAG_TOK_REQUEST, MPI_COMM_WORLD, &Status);
                             MPI_Send(NULL, 0, MPI_BYTE, Status.MPI_SOURCE, MPI_TAG_PRINT_TOK, MPI_COMM_WORLD);
 
-                            do {
+                            do
+                            {
                                 MPI_Iprobe(MPI_ANY_SOURCE, MPI_TAG_TOK_RETURN, MPI_COMM_WORLD, &incomingMessage, &Status);
 
                                 print_incoming_data();
@@ -579,7 +665,8 @@ hsize_t diff_match(hid_t file1_id,
                             options->not_cmp = options->not_cmp | nFoundbyWorker.not_cmp;
                             MPI_Send(&args, sizeof(args), MPI_BYTE, Status.MPI_SOURCE, MPI_TAG_ARGS, MPI_COMM_WORLD);
                         } /* end else-if */
-                        else {
+                        else
+                        {
                             printf("ERROR: Invalid tag (%d) received \n", Status.MPI_TAG);
                             MPI_Abort(MPI_COMM_WORLD, 0);
                             MPI_Finalize();
@@ -593,26 +680,32 @@ hsize_t diff_match(hid_t file1_id,
     h5diffdebug("done with for loop\n");
 
 #ifdef H5_HAVE_PARALLEL
-    if(g_Parallel) {
+    if(g_Parallel)
+    {
         /* make sure all tasks are done */
-        while(busyTasks > 0) {
+        while(busyTasks > 0)
+        {
             MPI_Probe(MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, &Status);
-            if(Status.MPI_TAG == MPI_TAG_DONE) {
+            if(Status.MPI_TAG == MPI_TAG_DONE)
+            {
                 MPI_Recv(&nFoundbyWorker, sizeof(nFoundbyWorker), MPI_BYTE, Status.MPI_SOURCE, MPI_TAG_DONE, MPI_COMM_WORLD, &Status);
                 nfound += nFoundbyWorker.nfound;
                 options->not_cmp = options->not_cmp | nFoundbyWorker.not_cmp;
                 busyTasks--;
             } /* end if */
-            else if(Status.MPI_TAG == MPI_TAG_TOK_RETURN) {
+            else if(Status.MPI_TAG == MPI_TAG_TOK_RETURN)
+            {
                 MPI_Recv(&nFoundbyWorker, sizeof(nFoundbyWorker), MPI_BYTE, Status.MPI_SOURCE, MPI_TAG_DONE, MPI_COMM_WORLD, &Status);
                 nfound += nFoundbyWorker.nfound;
                 options->not_cmp = options->not_cmp | nFoundbyWorker.not_cmp;
                 busyTasks--;
                 havePrintToken = 1;
             } /* end else-if */
-            else if(Status.MPI_TAG == MPI_TAG_TOK_REQUEST) {
+            else if(Status.MPI_TAG == MPI_TAG_TOK_REQUEST)
+            {
                 MPI_Recv(NULL, 0, MPI_BYTE, Status.MPI_SOURCE, MPI_TAG_TOK_REQUEST, MPI_COMM_WORLD, &Status);
-                if(havePrintToken) {
+                if(havePrintToken)
+                {
                     int incomingMessage;
 
                     MPI_Send(NULL, 0, MPI_BYTE, Status.MPI_SOURCE, MPI_TAG_PRINT_TOK, MPI_COMM_WORLD);
@@ -629,11 +722,13 @@ hsize_t diff_match(hid_t file1_id,
                     busyTasks--;
                 } /* end if */
                 /* someone else must have it...wait for them to return it, then give it to the task that just asked for it. */
-                else {
+                else
+                {
                     int source = Status.MPI_SOURCE;
                     int incomingMessage;
 
-                    do {
+                    do
+                    {
                         MPI_Iprobe(MPI_ANY_SOURCE, MPI_TAG_TOK_RETURN, MPI_COMM_WORLD, &incomingMessage, &Status);
 
                         print_incoming_data();
@@ -647,7 +742,8 @@ hsize_t diff_match(hid_t file1_id,
                     MPI_Send(NULL, 0, MPI_BYTE, source, MPI_TAG_PRINT_TOK, MPI_COMM_WORLD);
                 } /* end else */
             } /* end else-if */
-            else if(Status.MPI_TAG == MPI_TAG_TOK_RETURN) {
+            else if(Status.MPI_TAG == MPI_TAG_TOK_RETURN)
+            {
                 MPI_Recv(&nFoundbyWorker, sizeof(nFoundbyWorker), MPI_BYTE, Status.MPI_SOURCE, MPI_TAG_TOK_RETURN, MPI_COMM_WORLD, &Status);
                 nfound += nFoundbyWorker.nfound;
                 options->not_cmp = options->not_cmp | nFoundbyWorker.not_cmp;
@@ -663,7 +759,8 @@ hsize_t diff_match(hid_t file1_id,
 
                 printf("%s", data);
             } /* end else-if */
-            else {
+            else
+            {
                 printf("ph5diff-manager: ERROR!! Invalid tag (%d) received \n", Status.MPI_TAG);
                 MPI_Abort(MPI_COMM_WORLD, 0);
             } /* end else */
@@ -702,59 +799,66 @@ hsize_t diff_match(hid_t file1_id,
  *-------------------------------------------------------------------------
  */
 
-hsize_t diff_compare (hid_t file1_id,
-                      const char *file1_name,
-                      const char *obj1_name,
-                      trav_info_t * info1,
-                      hid_t file2_id,
-                      const char *file2_name,
-                      const char *obj2_name,
-                      trav_info_t * info2,
-                      diff_opt_t * options)
+hsize_t diff_compare(hid_t file1_id,
+                     const char *file1_name,
+                     const char *obj1_name,
+                     trav_info_t *info1,
+                     hid_t file2_id,
+                     const char *file2_name,
+                     const char *obj2_name,
+                     trav_info_t *info2,
+                     diff_opt_t *options)
 {
- int f1 = 0, f2 = 0;
- hsize_t nfound = 0;
+    int     f1 = 0;
+    int     f2 = 0;
+    hsize_t nfound = 0;
 
- ssize_t i = h5trav_getindex (info1, obj1_name);
- ssize_t j = h5trav_getindex (info2, obj2_name);
+    ssize_t i = h5trav_getindex (info1, obj1_name);
+    ssize_t j = h5trav_getindex (info2, obj2_name);
 
- if (i == -1)
- {
-  parallel_print ("Object <%s> could not be found in <%s>\n", obj1_name,
-   file1_name);
-  f1 = 1;
- }
- if (j == -1)
- {
-  parallel_print ("Object <%s> could not be found in <%s>\n", obj2_name,
-   file2_name);
-  f2 = 1;
- }
- if (f1 || f2)
- {
-  options->err_stat = 1;
-  return 0;
- }
+    if (i == -1)
+    {
+        parallel_print ("Object <%s> could not be found in <%s>\n", obj1_name,
+            file1_name);
+        f1 = 1;
+    }
+    if (j == -1)
+    {
+        parallel_print ("Object <%s> could not be found in <%s>\n", obj2_name,
+            file2_name);
+        f2 = 1;
+    }
+    if (f1 || f2)
+    {
+        options->err_stat = 1;
+        return 0;
+    }
 
- /* use the name with "/" first, as obtained by iterator function */
- obj1_name = info1->paths[i].path;
- obj2_name = info2->paths[j].path;
+    /* use the name with "/" first, as obtained by iterator function */
+    obj1_name = info1->paths[i].path;
+    obj2_name = info2->paths[j].path;
 
- /* objects are not the same type */
- if (info1->paths[i].type != info2->paths[j].type)
- {
-  if (options->m_verbose)
-   parallel_print("Comparison not possible: <%s> is of type %s and <%s> is of type %s\n",
-     obj1_name, get_type(info1->paths[i].type), obj2_name,
-     get_type(info2->paths[j].type));
-  options->not_cmp=1;
-  return 0;
- }
+    /* objects are not the same type */
+    if (info1->paths[i].type != info2->paths[j].type)
+    {
+        if (options->m_verbose||options->m_list_not_cmp)
+        {
+            parallel_print("<%s> is of type %s and <%s> is of type %s\n",
+            obj1_name, get_type(info1->paths[i].type), obj2_name,
+            get_type(info2->paths[j].type));
+        }
+        options->not_cmp=1;
+        return 0;
+    }
 
- nfound =
-  diff (file1_id, obj1_name, file2_id, obj2_name, options, info1->paths[i].type);
+    nfound = diff(file1_id,
+                  obj1_name,
+                  file2_id,
+                  obj2_name,
+                  options,
+                  info1->paths[i].type);
 
- return nfound;
+    return nfound;
 }
 
 
@@ -779,19 +883,20 @@ hsize_t diff_compare (hid_t file1_id,
 
 hsize_t diff(hid_t file1_id,
               const char *path1,
-              hid_t file2_id, 
-              const char *path2, 
-              diff_opt_t * options, 
+              hid_t file2_id,
+              const char *path2,
+              diff_opt_t * options,
               h5trav_type_t type)
 {
-    hid_t       type1_id = (-1);
-    hid_t       type2_id = (-1);
-    hid_t       grp1_id = (-1);
-    hid_t       grp2_id = (-1);
-    int ret;
+    hid_t   type1_id = (-1);
+    hid_t   type2_id = (-1);
+    hid_t   grp1_id = (-1);
+    hid_t   grp2_id = (-1);
+    int     ret;
     hsize_t nfound = 0;
-    
-    switch(type) {
+
+    switch(type)
+    {
        /*-------------------------------------------------------------------------
         * H5TRAV_TYPE_DATASET
         *-------------------------------------------------------------------------
@@ -801,7 +906,8 @@ hsize_t diff(hid_t file1_id,
             * verbose, always print name
             *-------------------------------------------------------------------------
             */
-            if(options->m_verbose) {
+            if(options->m_verbose)
+            {
                 if(print_objname(options, (hsize_t)1))
                     do_print_objname("dataset", path1, path2);
                 nfound = diff_dataset(file1_id, file2_id, path1, path2, options);
@@ -813,15 +919,18 @@ hsize_t diff(hid_t file1_id,
             * disabling quiet mode
             *-------------------------------------------------------------------------
             */
-            else {
-                if(options->m_quiet == 0) {
+            else
+            {
+                if(options->m_quiet == 0)
+                {
                     /* shut up temporarily */
                     options->m_quiet = 1;
                     nfound = diff_dataset(file1_id, file2_id, path1, path2, options);
 
                     /* print again */
                     options->m_quiet = 0;
-                    if(nfound) {
+                    if(nfound)
+                    {
                         if(print_objname(options,nfound))
                             do_print_objname("dataset", path1, path2);
                         nfound = diff_dataset(file1_id, file2_id, path1, path2, options);
@@ -833,7 +942,7 @@ hsize_t diff(hid_t file1_id,
                     nfound = diff_dataset(file1_id, file2_id, path1, path2, options);
             } /* end else */
             break;
-            
+
        /*-------------------------------------------------------------------------
         * H5TRAV_TYPE_NAMED_DATATYPE
         *-------------------------------------------------------------------------
@@ -843,20 +952,20 @@ hsize_t diff(hid_t file1_id,
                 goto out;
             if((type2_id = H5Topen2(file2_id, path2, H5P_DEFAULT)) < 0)
                 goto out;
-            
+
             if((ret = H5Tequal(type1_id, type2_id)) < 0)
                 goto out;
-            
+
             /* if H5Tequal is > 0 then the datatypes refer to the same datatype */
             nfound = (ret > 0) ? 0 : 1;
-            
+
             if(print_objname(options,nfound))
                 do_print_objname("datatype", path1, path2);
 
             /* always print the number of differences found in verbose mode */
             if(options->m_verbose)
                 print_found(nfound);
-            
+
             /*-------------------------------------------------------------------------
              * compare attributes
              * the if condition refers to cases when the dataset is a referenced object
@@ -864,30 +973,30 @@ hsize_t diff(hid_t file1_id,
              */
             if(path1)
                 nfound += diff_attr(type1_id, type2_id, path1, path2, options);
-            
+
             if(H5Tclose(type1_id) < 0)
                 goto out;
             if(H5Tclose(type2_id) < 0)
                 goto out;
             break;
-            
+
        /*-------------------------------------------------------------------------
         * H5TRAV_TYPE_GROUP
         *-------------------------------------------------------------------------
         */
         case H5TRAV_TYPE_GROUP:
             ret = HDstrcmp(path1, path2);
-        
+
             /* if "path1" != "path2" then the groups are "different" */
             nfound = (ret != 0) ? 1 : 0;
-            
+
             if(print_objname(options, nfound))
                 do_print_objname("group", path1, path2);
 
             /* always print the number of differences found in verbose mode */
             if(options->m_verbose)
                 print_found(nfound);
-            
+
             if((grp1_id = H5Gopen2(file1_id, path1, H5P_DEFAULT)) < 0)
                 goto out;
             if((grp2_id = H5Gopen2(file2_id, path2, H5P_DEFAULT)) < 0)
@@ -900,14 +1009,14 @@ hsize_t diff(hid_t file1_id,
              */
             if(path1)
                 nfound += diff_attr(grp1_id, grp2_id, path1, path2, options);
-            
+
             if(H5Gclose(grp1_id) < 0)
                 goto out;
             if(H5Gclose(grp2_id) < 0)
                 goto out;
             break;
-            
-            
+
+
        /*-------------------------------------------------------------------------
         * H5TRAV_TYPE_LINK
         *-------------------------------------------------------------------------
@@ -916,37 +1025,37 @@ hsize_t diff(hid_t file1_id,
             {
             H5L_info_t li1, li2;
             char *buf1, *buf2;
-            
+
             if(H5Lget_info(file1_id, path1, &li1, H5P_DEFAULT) < 0)
                 goto out;
             if(H5Lget_info(file1_id, path1, &li2, H5P_DEFAULT) < 0)
                 goto out;
-            
+
             buf1 = HDmalloc(li1.u.val_size);
             buf2 = HDmalloc(li2.u.val_size);
-            
+
             if(H5Lget_val(file1_id, path1, buf1, li1.u.val_size, H5P_DEFAULT) < 0)
                 goto out;
             if(H5Lget_val(file2_id, path2, buf2, li2.u.val_size, H5P_DEFAULT) < 0)
                 goto out;
-            
+
             ret = HDstrcmp(buf1, buf2);
-            
+
             /* if "buf1" != "buf2" then the links are "different" */
             nfound = (ret != 0) ? 1 : 0;
-            
+
             if(print_objname(options, nfound))
                 do_print_objname("link", path1, path2);
 
             /* always print the number of differences found in verbose mode */
             if(options->m_verbose)
                 print_found(nfound);
-            
+
             HDfree(buf1);
             HDfree(buf2);
             }
             break;
-            
+
        /*-------------------------------------------------------------------------
         * H5TRAV_TYPE_UDLINK
         *-------------------------------------------------------------------------
@@ -954,19 +1063,19 @@ hsize_t diff(hid_t file1_id,
         case H5TRAV_TYPE_UDLINK:
             {
             H5L_info_t li1, li2;
-            
+
             if(H5Lget_info(file1_id, path1, &li1, H5P_DEFAULT) < 0)
                 goto out;
             if(H5Lget_info(file1_id, path1, &li2, H5P_DEFAULT) < 0)
                 goto out;
-            
+
             /* Only external links will have a query function registered */
             if(li1.type == H5L_TYPE_EXTERNAL && li2.type == H5L_TYPE_EXTERNAL) {
                 char *buf1, *buf2;
 
                 buf1 = HDmalloc(li1.u.val_size);
                 buf2 = HDmalloc(li2.u.val_size);
-                
+
                 if(H5Lget_val(file1_id, path1, buf1, li1.u.val_size, H5P_DEFAULT) < 0) {
                     HDfree(buf1);
                     HDfree(buf2);
@@ -977,7 +1086,7 @@ hsize_t diff(hid_t file1_id,
                     HDfree(buf2);
                     goto out;
                 } /* end if */
-                
+
                 /* If the buffers are the same size, compare them */
                 if(li1.u.val_size == li2.u.val_size) {
                     if(H5Lget_val(file1_id, path1, buf1, li1.u.val_size, H5P_DEFAULT) < 0) {
@@ -994,20 +1103,20 @@ hsize_t diff(hid_t file1_id,
                 }
                 else
                     ret = 1;
-                
+
                 /* if "buf1" != "buf2" then the links are "different" */
                 nfound = (ret != 0) ? 1 : 0;
-                
+
                 if(print_objname(options, nfound))
                     do_print_objname("external link", path1, path2);
-            
+
                 HDfree(buf1);
                 HDfree(buf2);
             } /* end if */
             else {
                 /* If one or both of these links isn't an external link, we can only
                  * compare information from H5Lget_info since we don't have a query
-                 * function registered for them. 
+                 * function registered for them.
                  *
                  * If the link classes or the buffer length are not the
                  * same, the links are "different"
@@ -1016,27 +1125,27 @@ hsize_t diff(hid_t file1_id,
                     nfound = 1;
                 else
                     nfound = 0;
-                
+
                 if (print_objname (options, nfound))
                     do_print_objname ("user defined link", path1, path2);
             } /* end else */
-            
+
             /* always print the number of differences found in verbose mode */
             if(options->m_verbose)
                 print_found(nfound);
             }
             break;
-            
+
         default:
             if(options->m_verbose)
-                printf("Comparison not supported: <%s> and <%s> are of type %s\n",
+                parallel_print("Comparison not supported: <%s> and <%s> are of type %s\n",
                     path1, path2, get_type(type) );
             options->not_cmp = 1;
             break;
      }
- 
+
     return nfound;
- 
+
 out:
     options->err_stat = 1;
 
